@@ -22,7 +22,7 @@
 #endif
 
 #include <mysql.h>
-#include <myx_aux_functions.h>
+#include <myx_util_public_interface.h>
 #include <stdio.h>
 
 
@@ -44,20 +44,13 @@ extern "C" {
 /// [SCRIPT::LibInterfaceMapper] -public_interface "libmysqlx"
 #define libmysqlx_PUBLIC_INTERFACE_VERSION 10004
 
+/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_util_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\library_util\include\myx_util_public_interface.h"
 
 /*
  * Enums
  */
 
-// When this type is changed then also the error mapper in myx_backup.c must be adjusted!
-typedef enum myx_lib_error
-{
-    MYX_NO_ERROR = 0, MYX_ERROR_CANT_OPEN_FILE, MYX_ERROR_CANT_CONNECT_TO_INSTANCE, MYX_XML_PARSE_ERROR,
-    MYX_XML_NO_VALID_DOCUMENT, MYX_XML_EMPTY_DOCUMENT, MYX_SQL_ERROR, MYX_STOP_EXECUTION, MYX_ALLOC_CHANGE_ERROR,
-    MYX_OBJECT_NOT_FOUND, MYX_CANT_READ_FROM_FILE, MYX_CHARSET_CONVERSION_ERROR,MYX_CHARSET_WRONG_CHARSET_SPECIFIED,
-    MYX_MEMORY_LIMIT_EXCEEDED, MYX_OUT_OF_MEMORY
-} MYX_LIB_ERROR;
-
 typedef enum myx_user_connection_type
 {
     MYX_MYSQL_CONN = 0, MYX_ODBC_CONN, MYX_ORACLE_CONN, MYX_MSSQL_CONN, MYX_SQLITE_CONN
@@ -112,18 +105,6 @@ typedef enum
  */
 
 typedef void (*MYX_QUERY_CALLBACK)(MYSQL *mysql, void *cdata, const char *query, unsigned int length);
-  
-/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_STRINGLIST
-typedef struct {
-    unsigned int strings_num;
-    char **strings;
-} MYX_STRINGLIST;
-
-/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_NAME_VALUE_PAIR
-typedef struct {
-  char *name;
-  char *value;
-} MYX_NAME_VALUE_PAIR;
 
 /// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQL_ERROR_MSG
 typedef struct {
@@ -993,13 +974,6 @@ typedef struct {
   char *definition;
 } MYX_DBM_VIEW_DATA;
 
-  
-
-// ------------------------------
-// International File
-
-typedef struct myx_intl_file MYX_INTL_FILE;
-
 
 // ------------------------------
 // Table Data Exporting
@@ -1095,7 +1069,7 @@ MYX_PUBLIC_FUNC int myx_get_public_interface_version();
 
 MYX_PUBLIC_FUNC void myx_free_lib_str(char *str);
 MYX_PUBLIC_FUNC int myx_str_in_stringlist(MYX_STRINGLIST *stringlist, const char *src);
-MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
+MYX_PUBLIC_FUNC int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist);
 
 //Helper functions
 MYX_PUBLIC_FUNC char * get_value_from_text_ex(const char *txt, int txt_length, const char *regexpr, unsigned int substring_nr);
@@ -1108,6 +1082,8 @@ MYX_PUBLIC_FUNC MYSQL * myx_mysql_embedded_init();
 
 MYX_PUBLIC_FUNC MYSQL * myx_mysql_init();
 
+MYX_PUBLIC_FUNC int myx_init_library(const char *datadir);
+
 MYX_PUBLIC_FUNC void myx_mysql_set_query_hooks(MYSQL *mysql, 
                                                void (*pre_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
                                                void (*post_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
@@ -1149,6 +1125,18 @@ MYX_PUBLIC_FUNC void myx_set_mysql_error(char *error, int errn);
 
 MYX_PUBLIC_FUNC MYX_MYSQL_ERROR_MSGS * myx_mysql_error_msgs_fetch(MYSQL *mysql);
 MYX_PUBLIC_FUNC void myx_mysql_error_msgs_free(MYX_MYSQL_ERROR_MSGS *errors);
+
+MYX_PUBLIC_FUNC void build_field_subst(const char ** column_name,
+                                       const char ** column_name_end,
+                                       MYSQL_FIELD * res_fields,
+                                       MYSQL_FIELD * res_fields_end,
+                                       int * i_field);
+
+MYX_PUBLIC_FUNC int use_schema_store_old_one(MYSQL * mysql,
+                                             const char * schema_name,
+                                             char ** old_schema_name);
+MYX_PUBLIC_FUNC void restore_old_schema(MYSQL * mysql, char * old_schema_name);
+
   
 //Network functions
 MYX_PUBLIC_FUNC unsigned long myx_resolve_network_name(const char *hostname);
@@ -1331,11 +1319,6 @@ MYX_PUBLIC_FUNC int myx_charsets_store(const char *filename, MYX_DBM_CHARSETS *o
 MYX_PUBLIC_FUNC MYX_DBM_CHARSETS * myx_charsets_load(const char *filename, MYX_LIB_ERROR *error_code);
 MYX_PUBLIC_FUNC int myx_free_charsets(MYX_DBM_CHARSETS *obj);
 
-/* International file functions */
-MYX_PUBLIC_FUNC MYX_INTL_FILE * myx_new_intl_file(const char *filename, const char *charset, MYX_LIB_ERROR *error);
-MYX_PUBLIC_FUNC void myx_free_intl_file(MYX_INTL_FILE *intl_file);
-MYX_PUBLIC_FUNC bigint myx_read_char_from_intl_file(MYX_INTL_FILE *intl_file, int *bytes_read, MYX_LIB_ERROR *error);
-
 /* Others */
 MYX_PUBLIC_FUNC MYX_IMAGE_FORMAT myx_guess_image_format(const char *data, unsigned int length);
 
