@@ -23,9 +23,16 @@
 #include "myx_library.h"
 
 /*
- *public functions definitions
+ * public functions definitions
  */
 
+/*
+ * private functions definitions
+ */
+MYX_SCHEMA_VIEW_STATUS * myx_get_schema_view_status(MYSQL *mysql, 
+                                                    const char *catalog_name, 
+                                                    const char *schema_name);
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief load structure of catalogues
     @param mysql connection handler
@@ -264,7 +271,7 @@ MYX_TABLE_INDEX_COLUMN *copy_index_columns(MYX_TABLE_INDEX_COLUMN *src_columns,
 *//////////////////////////////////////////////////////////////////////////////
 void build_field_subst(const char ** column_name, 
                        const char ** column_name_end, MYSQL_FIELD * res_fields,
-                       MYSQL_FIELD * res_fields_end, unsigned int * i_field)
+                       MYSQL_FIELD * res_fields_end, int * i_field)
 {
   for (; column_name != column_name_end; column_name++, i_field++)
   {
@@ -378,7 +385,6 @@ MYX_TABLE_INDEX *get_indices_from_res(MYSQL *mysql, char *table_name,
     index->index_columns=
                       copy_index_columns(tmp_columns,index->index_columns_num);
   }
-
   *index_num= num_filled_indices;
 
   return indices;
@@ -495,6 +501,182 @@ static const char * describe_table_names[]=
 static const char ** describe_table_names_end=
              describe_table_names + sizeof(describe_table_names)/sizeof(char*);
 
+/*
+static void copy_table_status(MYX_TABLE_STATUS *dst, const MYX_TABLE_STATUS *src)
+{
+  dst->table_name= g_strdup(src->table_name);
+  dst->table_type= g_strdup(src->table_type);
+  dst->row_format= g_strdup(src->row_format);
+  dst->rows= g_strdup(src->rows);
+  dst->avg_row_length= g_strdup(src->avg_row_length);
+  dst->data_length= g_strdup(src->data_length);
+  dst->max_data_length= g_strdup(src->max_data_length);
+  dst->index_length= g_strdup(src->index_length);
+  dst->data_free= g_strdup(src->data_free);
+  dst->auto_increment= g_strdup(src->auto_increment);
+  dst->create_time= g_strdup(src->create_time);
+  dst->update_time= g_strdup(src->update_time);
+  dst->check_time= g_strdup(src->check_time);
+  dst->create_options= g_strdup(src->create_options);
+  dst->comment= g_strdup(src->comment);
+
+  dst->indexes_num= src->indexes_num;
+  dst->columns_num= src->columns_num;
+  
+  MYX_TABLE_INDEX *indexes;
+
+  
+  MYX_SCHEMA_TABLE_COLUMN *columns;
+
+
+}
+*/
+
+///////////////////////////////////////////////////////////////////////////////
+/** @brief load status info for schema entities (currently they are tables, SPs, views)
+
+    @param mysql        mysql connection handler
+    @param catalog_name name of catalog to load
+    @param schema_name  name of schema to load
+
+    @return loaded entity status info
+*//////////////////////////////////////////////////////////////////////////////
+
+MYX_SCHEMA_ENTITY_STATUS * myx_get_schema_entity_status(MYSQL *mysql,
+                                                       const char *catalog_name,
+                                                       const char *schema_name)
+{
+  unsigned i, j;
+  MYX_SCHEMA_TABLE_STATUS *tables= myx_get_schema_table_status(mysql, catalog_name, schema_name);
+  MYX_SCHEMA_VIEW_STATUS *views= myx_get_schema_view_status(mysql, catalog_name, schema_name);
+  MYX_SCHEMA_STORED_PROCEDURES *sps= myx_get_schema_sps(mysql, catalog_name, schema_name);
+  MYX_SCHEMA_ENTITY_STATUS *entities= g_malloc0(sizeof(MYX_SCHEMA_ENTITY_STATUS));
+
+  entities->schema_entities_num= tables->schema_tables_num + views->schema_views_num + sps->schema_sps_num;
+
+  entities->schema_entities= g_malloc0(sizeof(MYX_ENTITY_STATUS)*entities->schema_entities_num);
+  
+  for(j= 0, i= 0; j < tables->schema_tables_num; j++, i++)
+  {
+    entities->schema_entities[i].entity_type= MYX_ENTITY_TABLE;
+    entities->schema_entities[i].entity= tables->schema_tables + j;
+  }
+
+  for(j= 0; j < views->schema_views_num; j++, i++)
+  {
+    entities->schema_entities[i].entity_type= MYX_ENTITY_VIEW;
+    entities->schema_entities[i].entity= views->schema_views + j;
+  }
+
+  for(j= 0; j < sps->schema_sps_num; j++, i++)
+  {
+    if(sps->schema_sps[j].sp_type == MSPT_PROCEDURE)
+    {
+      entities->schema_entities[i].entity_type= MYX_ENTITY_PROC;
+    }
+    else
+    {
+      entities->schema_entities[i].entity_type= MYX_ENTITY_FUNC;
+    }
+    entities->schema_entities[i].entity= sps->schema_sps + j;
+  }
+
+  g_free(sps);
+  g_free(tables);
+  g_free(views);
+
+  return entities;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+/** @brief load view status info for schema
+
+    @param mysql        mysql connection handler
+    @param catalog_name name of catalog to load
+    @param schema_name  name of schema to load
+
+    @return loaded table statuses
+*//////////////////////////////////////////////////////////////////////////////
+MYX_SCHEMA_VIEW_STATUS * myx_get_schema_view_status(MYSQL *mysql,
+                                                      const char *catalog_name,
+                                                      const char *schema_name)
+{
+  MYX_SCHEMA_VIEW_STATUS *views= g_malloc0(sizeof(MYX_SCHEMA_VIEW_STATUS));
+  MYX_VIEW_STATUS *view;
+
+  MYSQL_RES *res_tbl;
+  MYSQL_ROW row_tbl;
+  MYSQL_FIELD *fields_tbl;
+  unsigned int num_fields_tbl;
+  unsigned int view_count= 0;
+  char *old_db;
+
+  if (!use_schema_store_old_one(mysql, schema_name,&old_db))
+  {
+    g_free(views);
+    return NULL;
+  }
+
+  if (myx_mysql_query(mysql, "SHOW TABLE STATUS") ||
+      !(res_tbl= mysql_store_result(mysql)))
+  {
+    myx_free_schema_view_status(views);
+    restore_old_schema(mysql, old_db);
+    return NULL;
+  }
+  else
+  {
+    int fi[16];
+    views->schema_views_num= (unsigned int)mysql_num_rows(res_tbl);
+    view= views->schema_views=
+                 g_malloc0(sizeof(MYX_VIEW_STATUS)*views->schema_views_num);
+
+    //Depending on the version of the server there might be different columns
+    num_fields_tbl= mysql_num_fields(res_tbl);
+    fields_tbl= mysql_fetch_fields(res_tbl);
+
+    build_field_subst(table_status_names,table_status_names_end,
+                      fields_tbl,fields_tbl+num_fields_tbl,fi);
+
+    while ((row_tbl= mysql_fetch_row(res_tbl)))
+    {
+#define XSTRDUP(s) g_strdup(s?s:"")
+      // check whether it is a view
+      if(((fi[1]!=-1) && (row_tbl[fi[1]] != NULL)) 
+        || ((fi[15]!=-1) && (row_tbl[fi[15]] != NULL)))
+      {
+        continue;
+      } 
+
+      view_count++;
+      
+      view->view_name= fi[0]==-1  ? 0 : myx_convert_dbstr_utf8(mysql, row_tbl[fi[0]]);      
+      view->comment=   fi[14]==-1 ? 0 : XSTRDUP(row_tbl[fi[14]]);
+
+#undef XSTRDUP
+      view++;
+    }
+    mysql_free_result(res_tbl);
+
+    views->schema_views_num= view_count;
+    if (views->schema_views_num==0)
+    {
+      g_free(views->schema_views);
+      views->schema_views= NULL;
+    }
+    else
+    {
+      //Realloc with real size
+      views->schema_views=
+        g_realloc(views->schema_views, sizeof(MYX_VIEW_STATUS)*views->schema_views_num);
+    }
+  }
+
+  restore_old_schema(mysql, old_db);
+  return views;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief load table statuses for schema (database)
 
@@ -558,12 +740,13 @@ MYX_SCHEMA_TABLE_STATUS * myx_get_schema_table_status(MYSQL *mysql,
       {
         if (table->table_type)
           g_free(table->table_type);
-
         continue;
       }
       else
+      {
         table_count++;
-
+      }
+      
       table->table_name=
                  fi[0]==-1 ? 0 : myx_convert_dbstr_utf8(mysql, row_tbl[fi[0]]);      
       table->row_format=       fi[2]==-1  ? 0 : XSTRDUP(row_tbl[fi[2]]);
@@ -774,14 +957,50 @@ MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *cata
   // 5: param datatype options in ()
   // 6: separator , or )
 
-#define SP_PARAMS "(IN|OUT|INOUT)?\\s?([\\w\\d]+)\\s+([\\w\\d]+)\\s*((\\([\\s\\w\\d,]+\\))?)\\s*(\\)|\\,)"
+#define SP_PARAMS "(IN\\s+|OUT\\s+|INOUT\\s+)?([\\w\\d]+)\\s+([\\w\\d]+)\\s*((\\([\\s\\w\\d,]+\\))?)\\s*(\\)|\\,)"
 
   //use the following regex to get the returntype of a function
   //RETURNS\s+([\w\d\(\)\,\s]+)\s+(LANGUAGE|NOT|DETERMINISTIC|SQL|COMMENT|RETURN|BEGIN)
   // 1: datatype
   // 2: keyword after datatype (not important)
 
-#define SP_RETURN_TYPE "RETURNS\\s+([\\w\\d\\(\\)\\,\\s]+?)\\s+(LANGUAGE|NOT|DETERMINISTIC|SQL|COMMENT|RETURN|BEGIN)"
+//#define SP_RETURN_TYPE "RETURNS\\s+([\\w\\d\\(\\)\\,\\s]+?)\\s+(LANGUAGE|NOT|DETERMINISTIC|SQL|COMMENT|RETURN|BEGIN)"
+
+// this is accroding to the documentation for 5.0.9 server
+#define SP_RETURN_TYPE "RETURNS\\s+(" \
+  "(TINY|BIG|SMALL|MEDIUM)INT(\\s*\\([0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"   \
+  "INT(\\s*\\([0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"       \
+  "INTEGER(\\s*\\([0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"   \
+  "DATE|"         \
+  "TIME|"         \
+  "TIMESTAMP|"    \
+  "DATETIME|"     \
+  "TINYBLOB|"     \
+  "BLOB|"         \
+  "MEDIUMBLOB|"   \
+  "LONGBLOB|"     \
+  "VARCHAR(\\s*\\([0-9]+\\))?(\\s+BINARY)?|"  \
+  "TINYTEXT(\\s+BINARY)?|"                                      \
+  "TEXT(\\s+BINARY)?|"                                          \
+  "MEDIUMTEXT(\\s+BINARY)?|"                                    \
+  "LONGTEXT(\\s+BINARY)?|"                                      \
+  "ENUM(\\s*\\([^)]*\\))?|"                                     \
+  "DOUBLE(\\s*\\([0-9]+\\,[0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"     \
+  "CHAR(\\s*\\([0-9]+\\))?(\\s+BINARY)?(\\s+ASCII)?(\\s+UNICODE)?|" \
+  "SET(\\s*\\([^)]*\\))?|"                                      \
+  "FLOAT(\\s*\\([0-9]+\\,[0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"      \
+  "DECIMAL(\\s*\\([0-9]+\\,[0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"    \
+  "NUMERIC(\\s*\\([0-9]+\\,[0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"    \
+  "REAL(\\s*\\([0-9]+\\,[0-9]+\\))?(\\s+UNSIGNED)?(\\s+ZEROFILL)?|"       \
+  "GEOMETRY|"     \
+  "POINT|"        \
+  "LINESTRING|"   \
+  "POLYGON|"      \
+  "MULTIPOINT|"   \
+  "MULTILINESTRING|"    \
+  "MULTIPOLYGON|" \
+  "GEOMETRYCOLLECTION"  \
+  ")"
 
   pcre_exp= pcre_compile(SP_PARAMS, PCRE_CASELESS, &error_str, &erroffset, NULL);
   if (!pcre_exp)
@@ -814,8 +1033,13 @@ MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *cata
       restore_old_schema(mysql, old_db);
       pcre_free(pcre_exp);
       pcre_free(pcre_exp2);
-      g_free(sps);
-      return NULL;
+      
+      // version < 5.0
+      //g_free(sps);
+      //return NULL;
+      sps->schema_sps_num= 0;
+      sps->schema_sps= NULL;
+      return sps;
     }
     else
     {
@@ -892,9 +1116,11 @@ MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *cata
               int matched[60], rc;
 
               //Get function return type
-              if (sp->sp_type==MSPT_FUNCTION)
+              // Taken out. The RE approach does not consistently work.
+              // TODO: if at all used implement it using a real parser.
+              if (FALSE && sp->sp_type==MSPT_FUNCTION)
               {
-                const char *return_type;
+                const char *return_type = NULL;
                 int matched[60], rc;
 
                 if((rc= pcre_exec(pcre_exp2, NULL, sp_code, (int)strlen(sp_code), 0, 0, matched, sizeof(matched)/sizeof(*matched)))>=0)
@@ -905,53 +1131,62 @@ MYX_SCHEMA_STORED_PROCEDURES * myx_get_schema_sps(MYSQL *mysql, const char *cata
                 }
               }
 
-              //Look only at params
-              sp_code= filter_sp_header(sp_code);
-
-              sp->params= g_malloc(sizeof(MYX_SCHEMA_STORED_PROCEDURE_PARAM)*sp_num_allocated);
-              sp->params_num= 0;              
-              
-              offset= 0;
-
-              while((rc= pcre_exec(pcre_exp, NULL, sp_code, (int)strlen(sp_code), offset, 0, matched, sizeof(matched)/sizeof(*matched)))>=0)
+              if(sp_code[0])
               {
-                if (sp->params_num>=sp_num_allocated)
+                //Look only at params
+                sp_code= filter_sp_header(sp_code);
+
+                sp->params= g_malloc(sizeof(MYX_SCHEMA_STORED_PROCEDURE_PARAM)*sp_num_allocated);
+                sp->params_num= 0;              
+                
+                offset= 0;
+
+                while((rc= pcre_exec(pcre_exp, NULL, sp_code, (int)strlen(sp_code), offset, 0, matched, sizeof(matched)/sizeof(*matched)))>=0)
                 {
-                  sp_num_allocated+= 5;
-                  sp->params= g_realloc(sp->params, sizeof(MYX_SCHEMA_STORED_PROCEDURE_PARAM)*sp_num_allocated);
+                  if (sp->params_num>=sp_num_allocated)
+                  {
+                    sp_num_allocated+= 5;
+                    sp->params= g_realloc(sp->params, sizeof(MYX_SCHEMA_STORED_PROCEDURE_PARAM)*sp_num_allocated);
+                  }
+
+                  param= sp->params + sp->params_num;
+
+                  pcre_get_substring(sp_code, matched, rc, 1, &param_type);
+                  pcre_get_substring(sp_code, matched, rc, 2, &param_name);
+                  pcre_get_substring(sp_code, matched, rc, 3, &param_datatype);
+
+                  param->name= g_strdup(param_name);
+                  param->datatype= g_strdup(param_datatype);
+                  if (strcmp2(param_type, "")==0)
+                    param->param_type= MSPPT_IN;
+                  else
+                    if (g_ascii_strncasecmp(param_type, "OUT", 3)==0)
+                      param->param_type= MSPPT_OUT;
+                    else
+                      if (g_ascii_strncasecmp(param_type, "INOUT", 5)==0)
+                        param->param_type= MSPPT_INOUT;
+                      else
+                        param->param_type= MSPPT_IN;
+
+                  pcre_free_substring(param_type);
+                  pcre_free_substring(param_name);
+                  pcre_free_substring(param_datatype);
+
+                  //Move offset
+                  offset= matched[1];
+
+                  sp->params_num++;
                 }
 
-                param= sp->params + sp->params_num;
-
-                pcre_get_substring(sp_code, matched, rc, 1, &param_type);
-                pcre_get_substring(sp_code, matched, rc, 2, &param_name);
-                pcre_get_substring(sp_code, matched, rc, 3, &param_datatype);
-
-                param->name= g_strdup(param_name);
-                param->datatype= g_strdup(param_datatype);
-                if (strcmp2(param_type, "")==0)
-                  param->param_type= MSPPT_IN;
-                else
-                  if (g_ascii_strncasecmp(param_type, "OUT", 3)==0)
-                    param->param_type= MSPPT_OUT;
-                  else
-                    if (g_ascii_strncasecmp(param_type, "INOUT", 5)==0)
-                      param->param_type= MSPPT_INOUT;
-                    else
-                      param->param_type= MSPPT_IN;
-
-                pcre_free_substring(param_type);
-                pcre_free_substring(param_name);
-                pcre_free_substring(param_datatype);
-
-                //Move offset
-                offset= matched[1];
-
-                sp->params_num++;
+                if (sp->params_num!=sp_num_allocated)
+                  sp->params= g_realloc(sp->params, sizeof(MYX_SCHEMA_STORED_PROCEDURE_PARAM)*sp->params_num);
+              }
+              else
+              {
+                sp->params= NULL;
+                sp->params_num= 0;
               }
 
-              if (sp->params_num!=sp_num_allocated)
-                sp->params= g_realloc(sp->params, sizeof(MYX_SCHEMA_STORED_PROCEDURE_PARAM)*sp->params_num);
             }
           }
           mysql_free_result(res);
@@ -1082,6 +1317,19 @@ int myx_free_schema_tables(MYX_SCHEMA_TABLES *schema_tables)
   return 0;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief free memory for the MYX_VIEW_STATUS struct
+    @param schema_table MYX_VIEW_STATUS struct to free
+    @return 0 always
+*//////////////////////////////////////////////////////////////////////////////
+int myx_free_schema_view_status_struct(MYX_VIEW_STATUS *schema_view)
+{
+  g_free(schema_view->comment);
+  g_free(schema_view->view_name);
+
+  return 0;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief free memory for the MYX_TABLE_STATUS struct
     @param schema_table MYX_TABLE_STATUS struct to free
@@ -1121,6 +1369,39 @@ int myx_free_schema_table_status_struct(MYX_TABLE_STATUS *schema_table)
   return 0;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief free memory for the MYX_SCHEMA_STORED_PROCEDURE struct
+    @param schema_sps MYX_SCHEMA_STORED_PROCEDURE struct to free
+    @return 0 always
+*//////////////////////////////////////////////////////////////////////////////
+int myx_free_schema_sps_struct(MYX_SCHEMA_STORED_PROCEDURE *sp)
+{
+    MYX_SCHEMA_STORED_PROCEDURE_PARAM *param= sp->params;
+    MYX_SCHEMA_STORED_PROCEDURE_PARAM *params_end= param + sp->params_num;
+
+    if (sp->name) 
+      g_free(sp->name);
+    if (sp->definer) 
+      g_free(sp->definer);
+    if (sp->created) 
+      g_free(sp->created);
+    if (sp->modified) 
+      g_free(sp->modified);
+    if (sp->comment) 
+      g_free(sp->comment);
+    if (sp->return_datatype) 
+      g_free(sp->return_datatype);
+
+    for (; param!=params_end; param++)
+    {
+      g_free(param->name);
+      g_free(param->datatype);
+    }
+
+    g_free(sp->params);
+    return 0;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief free memory for the MYX_SCHEMA_INDICES struct
     @param schema_indices MYX_SCHEMA_INDICES struct to free
@@ -1142,6 +1423,38 @@ int myx_free_schema_indices(MYX_SCHEMA_INDICES *schema_indices)
   return 0;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief free memory for the MYX_SCHEMA_TABLE_STATUS struct
+    @param schema_entities MYX_SCHEMA_TABLE_STATUS struct to free
+    @return 0 always
+*//////////////////////////////////////////////////////////////////////////////
+int myx_free_schema_entity_status(MYX_SCHEMA_ENTITY_STATUS *e)
+{
+  unsigned i;
+
+  for(i= 0; i < e->schema_entities_num; i++)
+  {
+    if (e->schema_entities[i].entity == NULL) continue;
+    switch(e->schema_entities[i].entity_type)
+    {
+    case MYX_ENTITY_TABLE:
+      myx_free_schema_table_status_struct((MYX_TABLE_STATUS *)e->schema_entities[i].entity);
+      break;
+    case MYX_ENTITY_VIEW:
+      myx_free_schema_view_status_struct((MYX_VIEW_STATUS *)e->schema_entities[i].entity);
+      break;
+    case MYX_ENTITY_PROC:
+    case MYX_ENTITY_FUNC:
+      myx_free_schema_sps_struct((MYX_SCHEMA_STORED_PROCEDURE *)e->schema_entities[i].entity);
+      break;
+    }
+  }
+  g_free(e->schema_entities);
+  g_free(e);
+
+  return 0;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief free memory for the MYX_SCHEMA_TABLE_STATUS struct
     @param schema_tables MYX_SCHEMA_TABLE_STATUS struct to free
@@ -1161,6 +1474,25 @@ int myx_free_schema_table_status(MYX_SCHEMA_TABLE_STATUS *schema_tables)
   return 0;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+/** @brief free memory for the MYX_SCHEMA_VIEW_STATUS struct
+    @param schema_tables MYX_SCHEMA_VIEW_STATUS struct to free
+    @return 0 always
+*//////////////////////////////////////////////////////////////////////////////
+int myx_free_schema_view_status(MYX_SCHEMA_VIEW_STATUS *schema_views)
+{
+  if (schema_views)
+  {
+    MYX_VIEW_STATUS * view= schema_views->schema_views;
+    MYX_VIEW_STATUS * views_end= view + schema_views->schema_views_num;
+    for (; view!=views_end; view++)
+      myx_free_schema_view_status_struct(view);
+    g_free(schema_views->schema_views);
+    g_free(schema_views);
+  }
+  return 0;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief free memory for the MYX_SCHEMA_STORED_PROCEDURE struct
     @param schema_sps MYX_SCHEMA_STORED_PROCEDURE struct to free
@@ -1174,29 +1506,7 @@ int myx_free_schema_sps(MYX_SCHEMA_STORED_PROCEDURES *schema_sps)
     MYX_SCHEMA_STORED_PROCEDURE *sps_end= sp + schema_sps->schema_sps_num;
     for (; sp!=sps_end; sp++)
     {
-      MYX_SCHEMA_STORED_PROCEDURE_PARAM *param= sp->params;
-      MYX_SCHEMA_STORED_PROCEDURE_PARAM *params_end= param + sp->params_num;
-
-      if (sp->name) 
-        g_free(sp->name);
-      if (sp->definer) 
-        g_free(sp->definer);
-      if (sp->created) 
-        g_free(sp->created);
-      if (sp->modified) 
-        g_free(sp->modified);
-      if (sp->comment) 
-        g_free(sp->comment);
-      if (sp->return_datatype) 
-        g_free(sp->return_datatype);
-
-      for (; param!=params_end; param++)
-      {
-        g_free(param->name);
-        g_free(param->datatype);
-      }
-
-      g_free(sp->params);
+      myx_free_schema_sps_struct(sp);
     }
 
     g_free(schema_sps->schema_sps);
