@@ -16,7 +16,7 @@
 
 
 #include "myx_public_interface.h"
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
 #include "myx_library.h"
 
 #define STR_EMPTY(s)  (!(s) || !*(s))
@@ -1487,7 +1487,7 @@ MYX_PUBLIC_FUNC char * myx_dbm_get_table_sql_diff(MYX_DBM_TABLE_DATA *existing_t
           ex_column= existing_table->columns+i;
           break;
         }
-      }
+      };
 
       //if column is not found anymore, drop it
       if(!column)
