@@ -1,4 +1,4 @@
-/* Copyright (C) 2003 MySQL AB
+/* Copyright (C) 2003, 2004, 2005 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,11 +21,16 @@
 #ifdef __GNUC__
 #include <unistd.h>
 #include <ctype.h>
+#include "config.h"
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#endif
 #endif
 
 #include "myx_library.h"
 #include "myx_network.h"
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
+#include "myx_shared_util_functions.h"
 
 // needed from my_global.h, cant include it because of conflicts
 #ifdef __GNUC__
@@ -257,6 +262,20 @@ __declspec(dllexport) BOOL WINAPI DllMain(
 }
 #endif 
 
+
+int myx_init_library(const char *datadir)
+{
+#ifdef ENABLE_NLS
+  char *path= g_strdup_printf("%s/locale", datadir);
+  bindtextdomain(GETTEXT_PACKAGE, path);
+  g_free(path);
+  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
+  textdomain(GETTEXT_PACKAGE);
+#endif
+  return 0;
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 /** @brief get version of library
     @ingroup Common_functions
@@ -1668,4 +1687,9 @@ int myx_check_utf8_convertible(const char *str, const char *encoding)
   return 1;
 }
 
+int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist)
+{
+  return myx_free_stringlist(stringlist);
+}
+
 /** @} */ // end of bridge_to_MySQL_API
