@@ -21,7 +21,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 #include <libxml/parser.h>
 #include <libxml/xpath.h>
 #include <assert.h>
-#include <myx_xml_aux_functions.h>
+#include <myx_xml_util_functions.h>
 #include <string.h>
 #include <stdlib.h>
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
