@@ -47,17 +47,30 @@ class CGenericCanvas;
  * All Listener classes are meant to be a means for calling back the application. They are implemented and instantiated
  * in the application and must be freed there. Don't forget to remove the listener class before you free it!
  */
-class GENERIC_CANVAS_API CGCListener
+class GENERIC_CANVAS_API CGCListenerInterface
 {
 public:
-  virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType) = 0;
+  virtual ~CGCListenerInterface() {};
+  virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TAction** action) = 0;
   virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason) = 0;
   virtual void __cdecl onDestroy(CGCBase* sender) = 0;
   virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message) = 0;
 };
 
-typedef set<CGCListener*> CGCListeners;
-typedef set<CGCListener*>::iterator CGCListenerIterator;
+typedef set<CGCListenerInterface*> CGCListeners;
+typedef set<CGCListenerInterface*>::iterator CGCListenerIterator;
+
+/**
+ * Basic implementation of a listener. Can be used as ancestor to allow overriding of only
+ * those functions that are necessary for a particular class.
+ */
+class GENERIC_CANVAS_API CGCBaseListener: public CGCListenerInterface
+{
+  virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TAction** action) {};
+  virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason) {};
+  virtual void __cdecl onDestroy(CGCBase* sender) {};
+  virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message) {};
+};
 
 /**
  * CGCBase serves as general base class for all generic canvas classes.
@@ -79,8 +92,8 @@ public:
   CGCBase(CGenericCanvas* canvas); 
   virtual ~CGCBase(void);
 
-  virtual void __cdecl action(CGCBase* origin, TActionType& action);
-  virtual void __cdecl addListener(CGCListener* listener); 
+  virtual void __cdecl action(CGCBase* origin, TAction** action);
+  virtual void __cdecl addListener(CGCListenerInterface* listener); 
   virtual void __cdecl beginUpdate(void);
   virtual CGenericCanvas* __cdecl canvas(void) { return FCanvas; };
   virtual void __cdecl change(CGCBase* origin, TGCChangeReason reason);
@@ -91,13 +104,13 @@ public:
   virtual void __cdecl enableEvents(void) { FDoEvents = true; };
   virtual void __cdecl endUpdate(void);
   virtual void __cdecl error(CGCBase* origin, const char* message);
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index) = 0;
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value) = 0;
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index) = 0;
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value) = 0;
   virtual void __cdecl release(void) { delete this; }; 
-  virtual void __cdecl removeListener(CGCListener* listener);
+  virtual void __cdecl removeListener(CGCListenerInterface* listener);
   virtual bool __cdecl updating(void) { return FUpdateCount > 0; };
-  virtual void __cdecl userData(void* newData) { FUserData = newData; };
-  virtual void* __cdecl userData(void) { return FUserData; };
+  virtual void* __cdecl userDataGet(void) { return FUserData; };
+  virtual void __cdecl userDataSet(void* newData) { FUserData = newData; };
 };
 
 //----------------- CGraphicElement ------------------------------------------------------------------------------------
