@@ -52,16 +52,13 @@ class CFigureElementTemplate
 {
   friend class CFigureElement;
 private:
-  class CStyleListener: private CGCListener
+  class CStyleListener: private CGCBaseListener
   {
     friend class CFigureElementTemplate;
   protected:
     CFigureElementTemplate* template_;
   public:
-    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& action) { };
-    virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
     virtual void __cdecl onDestroy(CGCBase* sender);
-    virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
   };
 
   wstring FId;                         // The XML element id.
@@ -158,8 +155,8 @@ public:
   CFigureElementTemplate* content(void) { return FContent; };
   wstring layoutClass(void) { return FClass; };
   wstring type(void) { return FType; };
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index);
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -218,8 +215,8 @@ public:
 
   void applyAlignment(void);
   virtual void __cdecl makeDirty(TGCChangeReason reason = GC_CHANGE_NONE);
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index);
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value);
   void render(void);
   void text(wstring newText);
   virtual void __cdecl validate(void);
@@ -230,13 +227,13 @@ class GENERIC_CANVAS_API CFigureElement: public CGraphicElement
 {
   friend class CCaptionElement;
 private:
-  class CElementListener: private CGCListener
+  class CElementListener: private CGCBaseListener
   {
     friend class CFigureElement;
   protected:
       CFigureElement* element;
   public:
-    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType);
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TAction** action);
     virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
     virtual void __cdecl onDestroy(CGCBase* sender);
     virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
@@ -259,10 +256,11 @@ private:
   float FForcedHeight;            // >= 0 if a certain height is assigned.
   float FStyleScaleX;             // Horizontal scale factor in case the style (if any) does not have the same size as the element.
   float FStyleScaleY;             // Dito for y direction.
+  bool FNeedStyleScale;           // Flag for fast determination of needed scaling.
 protected:
   void computeBoundingBox(void);
   TVertex computeOffset(CFigureElement* child);
-  TActionType doAction(TAction* action);
+  TAction* doAction(TAction* action);
 public:
   CFigureElement(CFigureElementTemplate* aTemplate, CGenericCanvas* canvas);
   virtual ~CFigureElement(void);
@@ -274,8 +272,8 @@ public:
   CFigureElement* elementFromId(wstring id);
   CFigureElement* elementFromPoint(TVertex coords);
   TVertex elementToFigure(TVertex coords);
-  TActionType executeAssociatedActions(const float X, const float Y);
-  TActionType executeAction(TActionType action);
+  TAction* executeAssociatedActions(const float X, const float Y);
+  TAction* executeAction(TActionType action);
   void expanded(bool isExpanded);
   bool expanded(void) { return FExpanded; };
   CFigure* figure(void) { return FFigure; };
@@ -283,8 +281,8 @@ public:
   virtual TFeedbackInfo __cdecl getFeedbackInfo(TVertex coords);
   TFigureElementLayout layout(void) { return FLayout; };
   virtual void __cdecl makeDirty(TGCChangeReason reason = GC_CHANGE_NONE);
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index);
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value);
   void render(void);
   bool resize(float dX, float dY, TFeedbackInfo info);
   void setCaption(const char* text);
@@ -297,11 +295,7 @@ public:
 
 //----------------------------------------------------------------------------------------------------------------------
 
-#ifdef __GNUC__
 typedef map<wstring, CFigureElement*> CFigureElementMap;
-#else
-typedef hash_map<wstring, CFigureElement*> CFigureElementMap;
-#endif
 
 /**
  * CFigure is the main element in the model and is created from a figure template. It cannot itself appear in a scene but
@@ -312,18 +306,20 @@ class GENERIC_CANVAS_API CFigure: public CGraphicElement
   friend class CGCModel;
   friend class CFigureInstance;
 private:
-  class CFigureElementListener: private CGCListener
+  class CFigureElementListener: private CGCBaseListener
   {
     friend class CFigure;
   protected:
       CFigure* figure;
   public:
-    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType);
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TAction** action);
     virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
     virtual void __cdecl onDestroy(CGCBase* sender);
     virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
   };
 
+  GLuint FDisplayList;
+
   CGCModel* FModel;          // The model to which this figure belongs.
   CFigureElement* FContent;  // The root element of the figure.
   float FScaling[3];         // The factors to scale the figure in each direction.
@@ -354,8 +350,8 @@ public:
   virtual TVertex __cdecl figureToView(TVertex coords, CFigureInstance* instance);
   void freeNotification(CFigureElement* object);
   CGCModel* model(void) { return FModel; };
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index);
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value);
   void removeMapping(CFigureElement* element);
   virtual void __cdecl render(float currentZoom);
   virtual void __cdecl rotate(float Angle, float Rx, float Ry, float Rz);
@@ -384,13 +380,13 @@ class GENERIC_CANVAS_API CFigureInstance: public CGraphicElement
   friend class CFeedbackLayer;
   friend class CFigureListener;
 private:
-  class CFigureListener: private CGCListener
+  class CFigureListener: public CGCBaseListener
   {
     friend class CFigureInstance;
   protected:
     CFigureInstance* instance;
   public:
-    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& action);
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TAction** action);
     virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
     virtual void __cdecl onDestroy(CGCBase* sender);
     virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* Message);
@@ -409,9 +405,7 @@ private:
   CFigureListener FListener;
 protected:
   void applyTransformations(void);
-  void onDestroy(CGCBase* figure);
-  TActionType executeAssociatedActions(TVertex& coords);
-  void replaceFigure(CFigure* figure);
+  TAction* executeAssociatedActions(TVertex& coords);
   void resize(float dX, float dY, TFeedbackInfo info);
   void updateMatrix(void);
 public:
@@ -425,9 +419,11 @@ public:
   virtual CLayer* __cdecl layer(void) { return FLayer; };
   virtual TBoundingBox __cdecl localBounds(void);
   virtual void __cdecl makeDirty(TGCChangeReason reason = GC_CHANGE_NONE);
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  void onDestroy(CGCBase* figure);
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index);
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value);
   virtual void __cdecl render(float currentZoom);
+  void replaceFigure(CFigure* figure);
   virtual void __cdecl rotate(float angle, float rx, float ry, float rz);
   virtual void __cdecl rotateV(float angle, const float axis[3]);
   virtual void __cdecl scale(float sx, float sy, float sz, bool accumulative = false);
