@@ -53,6 +53,7 @@ private:
   CFigureInstances::iterator FFigureInstanceIterator;
 public:
   CFigureInstanceEnumerator(CGCView* view);
+  virtual ~CFigureInstanceEnumerator(void) {};
 
   virtual bool __cdecl hasNext(void);
   virtual CFigureInstance* __cdecl next(void);
@@ -87,13 +88,13 @@ class GENERIC_CANVAS_API CGCView: public CGCBase
   friend class CFigureInstanceEnumerator;
 private:
   // A private listener class.
-  class CLayerListener: private CGCListener
+  class CLayerListener: private CGCBaseListener
   {
     friend class CGCView;
   protected:
     CGCView* view;
   public:
-    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType);
+    virtual void __cdecl onAction(CGCBase* sender, CGCBase* origin, TAction** action);
     virtual void __cdecl onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason);
     virtual void __cdecl onDestroy(CGCBase* object);
     virtual void __cdecl onError(CGCBase* sender, CGCBase* origin, const char* message);
@@ -107,10 +108,12 @@ private:
   GLfloat FColor[4];                   // The background color of the viewer.
   TGCViewport FViewport;               // Coordinates of the output area within the viewer window.
   CLayers FLayers;                     // A list of layers that are visible in this view.
-  float FZoom;                         // The current horizontal zoom factor. It's a normal scaling factor but for the 
+  float FZoom;                         // The current zoom factor. It's a normal scaling factor but for the 
                                        // whole view (scales everything).
   float FOffsetX;                      // A translation offset to enable scrolling (panning) in a zoomed scene.
   float FOffsetY;                      // The vertical translation offset.
+  float FPrintOffsetX;                 // Translation offsets for printing.
+  float FPrintOffsetY;
   float FWorkspaceWidth;               // Width and height of the virtual workspace.
   float FWorkspaceHeight;
   float FNearPlane;                    // The cut off plane that is closest to the eye.
@@ -137,69 +140,71 @@ private:
   CFloater* FExposeFloater;            // A floater marking the original display area.
 protected:
   void activate(void);
-  void applyTransformations(bool DoProjection);
-  void handleChange(CGCBase* origin, TGCChangeReason reason);
-  bool handleMouseDown(TMouseButton button, int modifiers, int windowX, int windowY, TVertex& viewCoords);
-  bool handleMouseMove(int modifiers, int windowX, int windowY, TVertex& viewCoords);
-  bool handleMouseUp(TMouseButton button, int modifiers, int windowX, int windowY, TVertex& viewCoords);
-  void render(void);
-  void updateCache(void);
+  void applyTransformations(void);
+	bool handleMouseDown(TMouseButton button, int modifiers, int windowX, int windowY, TVertex& viewCoords);
+	bool handleMouseMove(int modifiers, int windowX, int windowY, TVertex& viewCoords);
+	bool handleMouseUp(TMouseButton button, int modifiers, int windowX, int windowY, TVertex& viewCoords);
+	void internalRender(float zoom, bool renderGrid, bool renderConnections, bool renderFeedback, const TBoundingBox& visibleBounds);
+	void render(void);
+  void renderToMemory(TGCColorFormat format, float zoom, TGCViewport& bounds, unsigned char* memory);
+	void updateCache(void);
 public:
-  CGCView(CGenericCanvas* canvas, string name);
-  virtual ~CGCView(void);
+	CGCView(CGenericCanvas* canvas, string name);
+	virtual ~CGCView(void);
 
-  virtual void __cdecl addLayer(CLayer* layer);
-  virtual void __cdecl addToSelection(CFigureInstance* instance);
-  virtual void __cdecl clearContent(bool removeLayers);
-  virtual void __cdecl clearPrimitives(void);
-  virtual void __cdecl clearSelection(void);
-  virtual void __cdecl color(GLfloat* newColor);
-  virtual void __cdecl color(float red, float green, float blue, float alpha);
-  virtual bool __cdecl contains(CLayer* layer);
-  virtual CPrimitive* __cdecl createCircle(const TVertex& center, float ouiterRadius, float innerRadius, 
-    GLfloat* color, bool filled);
-  virtual CConnectionInstance* __cdecl createConnectionInstance(CConnection* connection, CFigureInstance* endPoint1, 
-    CFigureInstance* endPoint2);
-  virtual CFloater* __cdecl createFloater(const TBoundingBox& bounds, GLfloat* color1, GLfloat* color2, 
-    TFloaterDecoration decoration);
-  virtual CPrimitive* __cdecl createLine(const TVertex& start, const TVertex& end, GLfloat* color, GLfloat width, 
-    GLushort stipple);
-  virtual CPrimitive* __cdecl createRectangle(const TBoundingBox& bounds, GLfloat* color, bool filled);
-  virtual TFeedbackInfo __cdecl getFeedbackInfo(int windowX, int windowY);
-  virtual CFigureInstanceEnumerator* __cdecl getFigureInstanceEnumerator(void);
-  virtual CHitResults* __cdecl getHitTestInfoAt(TVertex point, bool singleHit);
-  virtual void __cdecl getLastRubberBounds(TBoundingBox& box);
-  virtual void __cdecl getOrigin(float* x, float* y, float* z);
-  virtual CSelectionEnumerator* __cdecl getSelectionEnumerator(void);
-  virtual void __cdecl getWorkspace(float* width, float* height);
-  virtual CGridLayer* __cdecl grid(void) { return FGrid; };
+	virtual void __cdecl addLayer(CLayer* layer);
+	virtual void __cdecl addToSelection(CFigureInstance* instance);
+	virtual void __cdecl clearContent(bool removeLayers);
+	virtual void __cdecl clearPrimitives(void);
+	virtual void __cdecl clearSelection(void);
+	virtual void __cdecl color(float red, float green, float blue, float alpha);
+	virtual void __cdecl colorV(GLfloat* newColor);
+	virtual bool __cdecl contains(CLayer* layer);
+	virtual CPrimitive* __cdecl createCircle(const TVertex& center, float ouiterRadius, float innerRadius,
+		GLfloat* color, bool filled);
+	virtual CConnectionInstance* __cdecl createConnectionInstance(CConnection* connection, CFigureInstance* endPoint1,
+		CFigureInstance* endPoint2);
+	virtual CFloater* __cdecl createFloater(const TBoundingBox& bounds, GLfloat* color1, GLfloat* color2,
+		TFloaterDecoration decoration);
+	virtual CPrimitive* __cdecl createLine(const TVertex& start, const TVertex& end, GLfloat* color, GLfloat width,
+		GLushort stipple);
+	virtual CPrimitive* __cdecl createRectangle(const TBoundingBox& bounds, GLfloat* color, bool filled);
+	virtual TFeedbackInfo __cdecl getFeedbackInfo(int windowX, int windowY);
+	virtual CFigureInstanceEnumerator* __cdecl getFigureInstanceEnumerator(void);
+	virtual CHitResults* __cdecl getHitTestInfoAt(TVertex point, bool singleHit);
+	virtual void __cdecl getLastRubberBounds(TBoundingBox& box);
+	virtual void __cdecl getOrigin(float* x, float* y, float* z);
+	virtual CSelectionEnumerator* __cdecl getSelectionEnumerator(void);
+	virtual void __cdecl getWorkspace(float* width, float* height);
+	virtual CGridLayer* __cdecl grid(void) { return FGrid; };
+  void handleChange(CGCBase* origin, TGCChangeReason reason);
   virtual bool __cdecl handleMouseInput(TMouseEvent event, TMouseButton button, int modifiers, int x, int y);
-  virtual void __cdecl jitter(float value);
-  virtual float __cdecl jitter(void);
-  virtual void __cdecl offsetX(float value);
-  virtual float __cdecl offsetX(void);
-  virtual void __cdecl offsetY(float value);
-  virtual float __cdecl offsetY(void);
+  virtual float __cdecl jitterGet(void);
+  virtual void __cdecl jitterSet(float value);
+  virtual float __cdecl offsetXGet(void);
+  virtual void __cdecl offsetXSet(float value);
+  virtual float __cdecl offsetYGet(void);
+  virtual void __cdecl offsetYSet(float value);
   virtual bool __cdecl overviewActive(void);
   virtual void __cdecl overviewStart(void);
   virtual void __cdecl overviewStop(bool returnToZoomed);
-  virtual TGCVariant __cdecl property(const char* name, unsigned int index);
-  virtual void __cdecl property(const char* name, unsigned int index, TGCVariant value);
+  virtual TGCVariant __cdecl propertyGet(const char* name, unsigned int index);
+  virtual void __cdecl propertySet(const char* name, unsigned int index, TGCVariant value);
   virtual void __cdecl removeFromSelection(CFigureInstance* instance);
   virtual void __cdecl removeLayer(CLayer* layer);
   virtual void __cdecl rubberBandStart(TRubberBandStyle style, const TVertex& coords);
   virtual void __cdecl rubberBandStop(void);
   virtual void __cdecl rubberRectStart(TRubberRectStyle style, const TVertex& coords, bool removeSelection);
   virtual void __cdecl rubberRectStop(void);
-  virtual bool __cdecl saveAsPNG(const TGCViewport& viewport, const char *filename);
   virtual void __cdecl setOrigin(float x, float y, float z);
   virtual void __cdecl setWorkspace(const float width, const float height);
   virtual void __cdecl showSelection(bool Visible);
-  virtual void __cdecl viewport(const TGCViewport& newViewport);
-  TGCViewport viewport(void) { return FViewport; };
+  TGCViewport viewportGet(void) { return FViewport; };
+  virtual void __cdecl viewportSet(const TGCViewport& newViewport);
+  virtual void __cdecl viewToWindow(const TVertex& coords, int& x, int& y);
   virtual void __cdecl windowToView(int x, int y, TVertex& coords);
-  virtual void __cdecl zoom(float value);
-  virtual float __cdecl zoom(void);
+  virtual float __cdecl zoomGet(void);
+  virtual void __cdecl zoomSet(float value);
 };
 
 //----------------------------------------------------------------------------------------------------------------------
