@@ -43,7 +43,7 @@ CGCBase::~CGCBase(void)
   FDestroying = true;
   for (CGCListeners::reverse_iterator iterator = FListeners.rbegin(); iterator != FListeners.rend(); ++iterator)
   {
-    CGCListener* listener = *iterator;
+    CGCListenerInterface* listener = *iterator;
     listener->onDestroy(this);
   };
   FListeners.clear();
@@ -55,10 +55,9 @@ CGCBase::~CGCBase(void)
  * Triggers the onAction event of all registered listeners to notfied them about a particular action.
  *
  * @param origin The object where the action was triggered.
- * @param actionType [in, out] Which action is about to be executed? Can be set to none by a listener to prevent it
- *                   from happening.
+ * @param action [in, out] The action which is about to be executed? Can be changed by a listener.
  */
-void CGCBase::action(CGCBase* origin, TActionType& actionType)
+void CGCBase::action(CGCBase* origin, TAction** action)
 {
   if (FDoEvents)
   {
@@ -68,8 +67,8 @@ void CGCBase::action(CGCBase* origin, TActionType& actionType)
       CGCListenerIterator next = iterator; ++next;
       try
       {
-        (*iterator)->onAction(this, origin, actionType);
-        if (actionType == GC_ACTION_NONE)
+        (*iterator)->onAction(this, origin, action);
+        if (action == NULL)
           break;
       }
       catch(...)
@@ -90,7 +89,7 @@ void CGCBase::action(CGCBase* origin, TActionType& actionType)
  *
  * @param listener The new listener to add to the internal list. A listener is only added once.
  */
-void CGCBase::addListener(CGCListener* listener) 
+void CGCBase::addListener(CGCListenerInterface* listener) 
 { 
   if (!FDestroying)
     if (FListeners.find(listener) == FListeners.end()) 
@@ -235,7 +234,7 @@ void CGCBase::error(CGCBase* origin, const char* message)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CGCBase::removeListener(CGCListener* listener)
+void CGCBase::removeListener(CGCListenerInterface* listener)
 {
   if (!FDestroying)
     FListeners.erase(listener);
