@@ -20,22 +20,24 @@
  * 
  */
 
+
 #include "myx_gc_canvas.h"
 #include "myx_gc_gl_helper.h"
+#include "gl2ps/gl2ps.h"
 
 //----------------------------------------------------------------------------------------------------------------------
 
 // Factory function for a canvas.
 GENERIC_CANVAS_API CGenericCanvas* CreateGenericCanvas(GCContext Context, char* name)
 {
-	return new CGenericCanvas(Context, utf8ToUtf16(name));
+  return new CGenericCanvas(Context, utf8ToUtf16(name));
 }
 
 //----------------- CCanvasListener ------------------------------------------------------------------------------------
 
-void CGenericCanvas::CCanvasListener::onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType)
+void CGenericCanvas::CCanvasListener::onAction(CGCBase* sender, CGCBase* origin, TAction** action)
 {
-  canvas->action(origin, actionType);
+  canvas->action(origin, action);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -47,12 +49,6 @@ void CGenericCanvas::CCanvasListener::onChange(CGCBase* sender, CGCBase* origin,
 
 //----------------------------------------------------------------------------------------------------------------------
 
-void CGenericCanvas::CCanvasListener::onDestroy(CGCBase* object)
-{
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 void CGenericCanvas::CCanvasListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
 {
   canvas->error(origin, message);
@@ -78,7 +74,11 @@ CGenericCanvas::CGenericCanvas(GCContext Context, wstring name): CGCBase(NULL)
   FListener.canvas = this;
   FModel = new CGCModel(this);
   FModel->addListener(&FListener);
+
   determineExtensions();
+  initializeOpenGLExtensions();
+
+  glEnable(GL_NORMALIZE);
 
   // Set a new lock on the font manager (and create it by the way if not yet done).
   lockFontManager();
@@ -129,6 +129,7 @@ void CGenericCanvas::determineExtensions(void)
   FExtensions.push_back(false); // GC_OE_CONVOLUTION
   FExtensions.push_back(false); // GC_OE_HISTOGRAM
   FExtensions.push_back(false); // GC_OE_TEXTURE3D
+  FExtensions.push_back(false); // GC_OE_FRAME_BUFFER_OBJECTS
 
   const GLubyte* extString = glGetString(GL_EXTENSIONS);
   if (extString != NULL)
@@ -141,6 +142,8 @@ void CGenericCanvas::determineExtensions(void)
       FExtensions[GC_OE_TEXTURE3D] = true;
     if (strstr((char*) extString, "GL_ARB_multisample") != NULL)
       FExtensions[GC_OE_MULTISAMPLING] = true;
+    if (strstr((char*) extString, "GL_EXT_framebuffer_object") != NULL)
+      FExtensions[GC_OE_FRAME_BUFFER_OBJECTS] = true;
   };
 }
 
@@ -539,7 +542,7 @@ CGCView* CGenericCanvas::createView(const char* name)
   FViews.push_back(View);
   View->addListener(&FListener);
   if (FCurrentView == NULL)
-    currentView(View);
+    currentViewSet(View);
 
   change(View, GC_CHANGE_CANVAS_ADD_VIEW);
   return View;
@@ -547,38 +550,38 @@ CGCView* CGenericCanvas::createView(const char* name)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Sets the currently active view.
- *
- * @param View The new view to activate.
- */
-void CGenericCanvas::currentView(CGCView* View)
-{
-  if (FCurrentView != View)
-  {
-    FCurrentView = View;
-    if (FCurrentView != NULL)
-      FStates |= GC_STATE_PENDING_ACTIVATION;
-    else
-      FStates &= ~GC_STATE_PENDING_ACTIVATION;
-    change(FCurrentView, GC_CHANGE_CANVAS_SWITCH_VIEW);
-  };
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Returns the currently active view.
  *
  * @return The currently active view.
  */
-CGCView* CGenericCanvas::currentView(void)
+CGCView* CGenericCanvas::currentViewGet(void)
 {
   return FCurrentView;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Sets the currently active view.
+ *
+ * @param View The new view to activate.
+ */
+void CGenericCanvas::currentViewSet(CGCView* View)
+{
+  if (FCurrentView != View)
+  {
+	FCurrentView = View;
+	if (FCurrentView != NULL)
+	  FStates |= GC_STATE_PENDING_ACTIVATION;
+	else
+	  FStates &= ~GC_STATE_PENDING_ACTIVATION;
+	change(FCurrentView, GC_CHANGE_CANVAS_SWITCH_VIEW);
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Returns the first layer with the given name from the layers collection.
  *
@@ -613,7 +616,7 @@ CLayer* CGenericCanvas::layerByName(const char* name)
  * @param index If the property is a list then this is the index into that list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
+TGCVariant CGenericCanvas::propertyGet(const char* name, unsigned int index)
 {
   // TODO: check if all properties are properly handled.
 
@@ -652,7 +655,7 @@ TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
     case GC_CONTAINER_LAYERS:
       {
         if (index < FLayers.size())
-          result = FLayers[index];
+		  result = FLayers[index];
         break;
       };
     case GC_CONTAINER_VIEWS:
@@ -680,15 +683,15 @@ TGCVariant CGenericCanvas::property(const char* name, unsigned int index)
  * @param index If the property is a list then this is the index into that list.
  * @param value The new value of the property. Automatic conversion is performed where possible.
  */
-void CGenericCanvas::property(const char* name, unsigned int index, TGCVariant value)
+void CGenericCanvas::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   switch (getPropertyID(name))
   {
-    case GC_PROPERTY_NAME:
-      {
-        FName = utf8ToUtf16(value);
-        break;
-      };
+	case GC_PROPERTY_NAME:
+	  {
+		FName = utf8ToUtf16(value);
+		break;
+	  };
   };
 }
 
@@ -803,6 +806,152 @@ void CGenericCanvas::render(void)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Renders the current scene to a file in the format given.
+ *
+ * @param filename The name of the target file. It must already contain the correct extension and must be encoded in UTF-8.
+ * @param format The format of the file to render. Supported are PNG, PDF, PS (postscript) and EPS (encapsulated postscript).
+ * @param title The titel for the document. Must be ANSI encoded for now.
+ * @param software A string describing the producer of the document. Must be ANSI encoded for now.
+ * @param zoom The zoom at which render the file.
+ * @param bounds Position and size of the area to store in the file (currently only for PNG).
+ * @return True if successful otherwise false.
+ */
+bool CGenericCanvas::renderToFile(const char* filename, TGCFileFormat format, const char* title, const char* software,
+                                  float zoom, TGCViewport& bounds)
+{
+  bool result = false;
+
+  if (!updating())
+  {
+    beginUpdate();
+    try
+    {
+      if ((FStates & GC_STATE_PENDING_ACTIVATION) != 0)
+      {
+        // A pending activation state always means there is a valid current view.
+        FStates &= ~GC_STATE_PENDING_ACTIVATION;
+        FCurrentView->activate();
+      };
+
+      switch (format)
+      {
+        case GC_FILE_FORMAT_PDF:
+        case GC_FILE_FORMAT_PS:
+        case GC_FILE_FORMAT_EPS:
+          {
+            if (FCurrentView != NULL)
+            {
+              const int fileTypeMapper[4] = {GL2PS_PDF, GL2PS_PS, GL2PS_EPS, GL2PS_TEX};
+
+              FILE *file = openFile(filename, "wb");
+
+              GLint bufferSize = 0;
+              GLint state = GL2PS_OVERFLOW;
+              
+              char *oldlocale = setlocale(LC_NUMERIC, "C");
+              while (state == GL2PS_OVERFLOW)
+              {
+                bufferSize += 1024 * 1024;
+                gl2psBeginPage(title, software, NULL, fileTypeMapper[format], GL2PS_NO_SORT, GL2PS_DRAW_BACKGROUND | GL2PS_USE_CURRENT_VIEWPORT |
+                  GL2PS_COMPRESS, GL_RGBA, 0, NULL, 0, 0, 0, bufferSize, file, filename);
+                gl2psEnable(GL2PS_BLEND);
+                gl2psBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+                clearBuffers();
+                FCurrentView->render();
+
+                state = gl2psEndPage();
+              };
+              setlocale(LC_NUMERIC, oldlocale);
+              fclose(file);
+
+              result = true;
+            };
+            break;
+          };
+        case GC_FILE_FORMAT_PNG:
+          {
+            if (FCurrentView != NULL)
+            {
+              TImage image;
+
+              float workspaceWidth;
+              float workspaceHeight;
+              FCurrentView->getWorkspace(&workspaceWidth, &workspaceHeight);
+              image.width = bounds.width;
+              if (image.width < 0)
+                image.width = (int) workspaceWidth;
+              image.height = bounds.height;
+              if (image.height < 0)
+                image.height = (int) workspaceHeight;
+
+              // If the frame buffer extension is not supported then there is no sense
+              // to allocate a buffer larger than the current viewport,
+              // because we cannot render more than this area in this case.
+              if (!supportsExtension(GC_OE_FRAME_BUFFER_OBJECTS))
+              {
+				if (image.height > FCurrentView->viewportGet().height)
+				  image.height = FCurrentView->viewportGet().height;
+				if (image.width > FCurrentView->viewportGet().width)
+                  image.width = FCurrentView->viewportGet().width;
+              };
+
+              image.colorType = COLOR_TYPE_RGB_ALPHA;
+              image.data = (unsigned char*) malloc(image.width * image.height * 4);
+              if (image.data != NULL)
+              {
+                FCurrentView->renderToMemory(GC_COLOR_FORMAT_RGBA, zoom, bounds, image.data);
+                image.width = bounds.width;
+                image.height = bounds.height;
+                result = savePNG(utf8ToUtf16(filename), &image, true, title, software);
+                free(image.data);
+              };
+            };
+            break;
+          };
+      };
+      endUpdate();
+    }
+    catch(...)
+    {
+      endUpdate();
+      throw;
+    };
+
+    checkError();
+  };
+
+  return result;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Renders the content of the current view into the given memory using the RGBA color layout.
+ *
+ * @param memory A pointer to already allocated memory. This is the target where data is rendered to and must be at least
+ *               bounds.width * bounds.height * 4 bytes in size.
+ * @param format Determines the order of the color components in the result.
+ * @param zoom The zoom level at which to render the scene.
+ * @param bounds [in/out] The coordinates of the area that should be rendered (in view space). When traditional rendering
+ *                        is used then it might be necessary to implicitly restrict this area to the current viewport.
+ *                        In this case bounds are used to return actual area to the caller.
+ * @param result True if rendering went fine, otherwise false.
+ */
+bool CGenericCanvas::renderToMemory(unsigned char* memory, TGCColorFormat format, float zoom, TGCViewport& bounds)
+{
+  if (FCurrentView != NULL)
+  {
+    FCurrentView->renderToMemory(format, zoom, bounds, memory);
+    return true;
+  }
+  else
+    return false;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Sets the path to be used by the texture manager for loading textures.
  *
@@ -852,4 +1001,3 @@ CGCView* CGenericCanvas::viewByName(const char* name)
 }
 
 //----------------------------------------------------------------------------------------------------------------------
-
