@@ -25,12 +25,6 @@
 
 //----------------- CFigureListener ------------------------------------------------------------------------------------
 
-void CConnection::CFigureListener::onChange(CGCBase* sender, CGCBase* origin, TGCChangeReason reason)
-{
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 void CConnection::CFigureListener::onDestroy(CGCBase* object)
 {
   connection->figureDestroyed((CFigure*) object);
@@ -94,7 +88,7 @@ CConnection::~CConnection(void)
  * @param data A list of CFigure instances that are to be used as decoration for the connection.
  *             Some or all entries can be NULL indicating so there should be no decoration for the corresponding part.
  */
-void CConnection::decorations(const TConnectionDecorations& data)
+void CConnection::decorationsSet(const TConnectionDecorations& data)
 {
   if (FDecorations.end1Decoration != NULL)
     FDecorations.end1Decoration->removeListener(&FListener);
@@ -179,7 +173,7 @@ void CConnection::figureDestroyed(CFigure* figure)
  * @param index The index of the sub property to return if it is located in a list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CConnection::property(const char* name, unsigned int index)
+TGCVariant CConnection::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -212,7 +206,7 @@ TGCVariant CConnection::property(const char* name, unsigned int index)
  * @param index The index of the sub property to return if it is located in a list.
  * @param value The new value of the property. Automatic conversion is performed where possible.
  */
-void CConnection::property(const char* name, unsigned int index, TGCVariant value)
+void CConnection::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   // There are currently no properties that could be changed.
 }
@@ -242,14 +236,7 @@ void CConnectionInstance::CConnectionListener::onDestroy(CGCBase* object)
   delete instance;
 }
 
-//----------------------------------------------------------------------------------------------------------------------
-
-void CConnectionInstance::CConnectionListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
-{
-  // Let the model report any errors up the chain. This is not our business.
-}
-
-//----------------------------------------------------------------------------------------------------------------------
+//----------------- CConnectionInstance --------------------------------------------------------------------------------
 
 /**
  * Constructor of the class.
@@ -291,7 +278,12 @@ CConnectionInstance::CConnectionInstance(CConnectionLayer* layer, CConnection* c
   FCenterLabel = NULL;
   FEnd2Decoration = NULL;
   FEnd2Label = NULL;
-  FCenter = 0.5f;
+
+  // If this connection is self referencing then move the center closer to the (one) end point otherwise center it.
+  if (FEnd1 == FEnd2)
+    FCenter = 0.01f;
+  else
+    FCenter = 0.5f;
 
   updateDecorations();
 }
@@ -396,41 +388,75 @@ float CConnectionInstance::computeCenterPosition(void)
 {
   float result = 0;
 
-  switch (FDirection1)
+  if (FEnd1 == FEnd2)
   {
-    case GC_DIR_SOUTH:
-    case GC_DIR_NORTH:
-    {
-      float offset1 = 0;
-      float offset2 = 0;
-      if (FEnd1Decoration != NULL)
-        offset1 = FEnd1Decoration->height();
-      if (FEnd2Decoration != NULL)
-        offset2 = FEnd2Decoration->height();
-      if (FEndVertex1.y <= FEndVertex2.y)
-        result = FEndVertex1.y + offset1 + FCenter * (FEndVertex2.y - offset2 - FEndVertex1.y - offset1);
-      else
-        result = FEndVertex2.y + offset2 + FCenter * (FEndVertex1.y - offset1 - FEndVertex2.y - offset2);
+    // Self referencing, means this connections starts and stops at the same end point.
+    float fullWidth;
+    float fullHeight;
 
-      break;
-    }
-    default:
-    {
-      // GC_DIR_EAST/GC_DIR_WEST
-      float offset1 = 0;
-      float offset2 = 0;
-      if (FEnd1Decoration != NULL)
-        offset1 = FEnd1Decoration->width();
-      if (FEnd2Decoration != NULL)
-        offset2 = FEnd2Decoration->width();
-      if (FEndVertex1.x <= FEndVertex2.x)
-        result = FEndVertex1.x + offset1 + FCenter * (FEndVertex2.x - offset2 - FEndVertex1.x - offset1);
-      else
-        result = FEndVertex2.x + offset2 + FCenter * (FEndVertex1.x - offset1 - FEndVertex2.x - offset2);
+    // Compute the center as percentage of the entire width/height.
+    canvas()->currentViewGet()->getWorkspace(&fullWidth, &fullHeight);
 
-      break;
+    switch (FDirection1)
+    {
+      case GC_DIR_SOUTH:
+      case GC_DIR_NORTH:
+      {
+        float offset = (FEnd1Decoration != NULL) ? FEnd1Decoration->height() : 0;
+        if ((FEnd2Decoration != NULL) && FEnd2Decoration->height() > offset)
+          offset = FEnd2Decoration->height();
+
+        if (FDirection1 == GC_DIR_NORTH)
+          result = FEndVertex1.y - offset - FCenter * fullHeight;
+        else
+          result = FEndVertex1.y + offset + FCenter * fullHeight;
+        
+        break;
+      }
+      default:
+      {
+        // GC_DIR_EAST/GC_DIR_WEST
+        float offset = (FEnd1Decoration != NULL) ? FEnd1Decoration->width() : 0;
+        if ((FEnd2Decoration != NULL) && FEnd2Decoration->width() > offset)
+          offset = FEnd2Decoration->width();
+
+        if (FDirection1 == GC_DIR_WEST)
+          result = FEndVertex1.x - offset - FCenter * fullWidth;
+        else
+          result = FEndVertex1.x + offset + FCenter * fullWidth;
+
+        break;
+      };
+    };
+  }
+  else
+    switch (FDirection1)
+    {
+      case GC_DIR_SOUTH:
+      case GC_DIR_NORTH:
+      {
+        float offset1 = (FEnd1Decoration != NULL) ? FEnd1Decoration->height() : 0;
+        float offset2 = (FEnd2Decoration != NULL) ? FEnd2Decoration->height() : 0;
+        if (FEndVertex1.y <= FEndVertex2.y)
+          result = FEndVertex1.y + offset1 + FCenter * (FEndVertex2.y - offset2 - FEndVertex1.y - offset1);
+        else
+          result = FEndVertex2.y + offset2 + FCenter * (FEndVertex1.y - offset1 - FEndVertex2.y - offset2);
+
+        break;
+      }
+      default:
+      {
+        // GC_DIR_EAST/GC_DIR_WEST
+        float offset1 = (FEnd1Decoration != NULL) ? FEnd1Decoration->width() : 0;
+        float offset2 = (FEnd2Decoration != NULL) ? FEnd2Decoration->width() : 0;
+        if (FEndVertex1.x <= FEndVertex2.x)
+          result = FEndVertex1.x + offset1 + FCenter * (FEndVertex2.x - offset2 - FEndVertex1.x - offset1);
+        else
+          result = FEndVertex2.x + offset2 + FCenter * (FEndVertex1.x - offset1 - FEndVertex2.x - offset2);
+
+        break;
+      };
     };
-  };
 
   return result;
 }
@@ -458,7 +484,7 @@ TConnectionDirection CConnectionInstance::determineCenterDirection(void)
         // then the center points east. If there is not enough distance to make up a meaningful horizontal
         // direction then point the center upwards. Finally if the second point is placed to the left of the
         // first point the center points to west. Similar for all other cases.
-        if (fabsf(dX) < centerWidth)
+        if (fabs(dX) < centerWidth)
           result = GC_DIR_NORTH;
         else
           if (dX > 0)
@@ -469,7 +495,7 @@ TConnectionDirection CConnectionInstance::determineCenterDirection(void)
       };
     case GC_DIR_EAST:
       {
-        if (fabsf(dY) < centerWidth)
+        if (fabs(dY) < centerWidth)
           result = GC_DIR_EAST;
         else
           if (dY > 0)
@@ -480,7 +506,7 @@ TConnectionDirection CConnectionInstance::determineCenterDirection(void)
       };
     case GC_DIR_SOUTH:
       {
-        if (fabsf(dX) < centerWidth)
+        if (fabs(dX) < centerWidth)
           result = GC_DIR_SOUTH;
         else
           if (dX > 0)
@@ -491,7 +517,7 @@ TConnectionDirection CConnectionInstance::determineCenterDirection(void)
       };
     case GC_DIR_WEST:
       {
-        if (fabsf(dY) < centerWidth)
+        if (fabs(dY) < centerWidth)
           result = GC_DIR_WEST;
         else
           if (dY > 0)
@@ -664,9 +690,9 @@ void CConnectionInstance::updateDecoration(TConnectionPart part)
       {
         case GC_DIR_NORTH:
           {
-            float dX1 = fabsf(position->x - decorationBounds.upper.x);
-            float dX2 = fabsf(decorationBounds.lower.x - position->x);
-            float dY = fabsf(position->y - decorationBounds.upper.y);
+            float dX1 = fabs(position->x - decorationBounds.upper.x);
+            float dX2 = fabs(decorationBounds.lower.x - position->x);
+            float dY = fabs(position->y - decorationBounds.upper.y);
             float x = (dX1 <= dX2) ? decorationBounds.upper.x : decorationBounds.lower.x;
             if (dX1 < dY || dX2 < dY)
             {
@@ -687,9 +713,9 @@ void CConnectionInstance::updateDecoration(TConnectionPart part)
           };
         case GC_DIR_EAST:
           {
-            float dY1 = fabsf(decorationBounds.upper.y - position->y);
-            float dY2 = fabsf(position->y - decorationBounds.lower.y);
-            float dX = fabsf(decorationBounds.lower.x - position->x);
+            float dY1 = fabs(decorationBounds.upper.y - position->y);
+            float dY2 = fabs(position->y - decorationBounds.lower.y);
+            float dX = fabs(decorationBounds.lower.x - position->x);
             float y = (dY1 <= dY2) ? decorationBounds.upper.y : decorationBounds.lower.y;
             if (dY1 < dX || dY2 < dX)
             {
@@ -710,9 +736,9 @@ void CConnectionInstance::updateDecoration(TConnectionPart part)
           };
         case GC_DIR_SOUTH:
           {
-            float dX1 = fabsf(position->x - decorationBounds.upper.x);
-            float dX2 = fabsf(decorationBounds.lower.x - position->x);
-            float dY = fabsf(decorationBounds.lower.y - position->y);
+            float dX1 = fabs(position->x - decorationBounds.upper.x);
+            float dX2 = fabs(decorationBounds.lower.x - position->x);
+            float dY = fabs(decorationBounds.lower.y - position->y);
             float x = (dX1 <= dX2) ? decorationBounds.upper.x : decorationBounds.lower.x;
             if (dX1 < dY || dX2 < dY)
             {
@@ -733,9 +759,9 @@ void CConnectionInstance::updateDecoration(TConnectionPart part)
           };
         case GC_DIR_WEST:
           {
-            float dY1 = fabsf(decorationBounds.upper.y - position->y);
-            float dY2 = fabsf(position->y - decorationBounds.lower.y);
-            float dX = fabsf(position->x - decorationBounds.upper.x);
+            float dY1 = fabs(decorationBounds.upper.y - position->y);
+            float dY2 = fabs(position->y - decorationBounds.lower.y);
+            float dX = fabs(position->x - decorationBounds.upper.x);
             float y = (dY1 <= dY2) ? decorationBounds.upper.y : decorationBounds.lower.y;
             if (dY1 < dX || dY2 < dX)
             {
@@ -827,16 +853,18 @@ bool CConnectionInstance::containsPoint(const float x, const float y)
  * @param point One of the end points of this connection.
  * @param direction Indicates the edge where the connection leaves the end point.
  * @param slot A distribution offset in the range (0..1) that controls the relative position along the edge.
+ * @param alternative Indicates to to use the opposite end point instead what @point would refer to. 
+ *                    Used, e.g., for self references.
  */
-void CConnectionInstance::computeCoordinates(CFigureInstance* point, TConnectionDirection direction, float slot)
+void CConnectionInstance::computeCoordinates(CFigureInstance* point, TConnectionDirection direction, float slot, 
+                                             bool alternative)
 {
   TVertex coordinates;
 
-  if (point == FEnd1)
+  if ((point == FEnd1) != alternative)
     FDirection1 = direction;
   else
-    if (point == FEnd2)
-      FDirection2 = direction;
+    FDirection2 = direction;
 
   TBoundingBox bounds = point->bounds();
   switch (direction)
@@ -866,7 +894,7 @@ void CConnectionInstance::computeCoordinates(CFigureInstance* point, TConnection
         break;
       };
   };
-  if (point == FEnd1)
+  if ((point == FEnd1) != alternative)
     FEndVertex1 = coordinates;
   else
     FEndVertex2 = coordinates;
@@ -901,25 +929,31 @@ TConnectionDirection CConnectionInstance::getDirection(CFigureInstance* point)
 
   if (from != NULL && to != NULL)
   {
-    TBoundingBox fromBounds = from->bounds();
-    TBoundingBox toBounds = to->bounds();
+    if (from == to)
+      // The connection starts and ends on the same end point.
+      result = GC_DIR_SELF;
+    else
+    {
+      TBoundingBox fromBounds = from->bounds();
+      TBoundingBox toBounds = to->bounds();
 
-    result = GC_DIR_NORTH;
-    float distance = fromBounds.upper.y - toBounds.lower.y;
-    if (toBounds.upper.x - fromBounds.lower.x > distance)
-    {
-      result = GC_DIR_EAST;
-      distance = toBounds.upper.x - fromBounds.lower.x;
-    };
-    if (toBounds.upper.y - fromBounds.lower.y > distance)
-    {
-      result = GC_DIR_SOUTH;
-      distance = toBounds.upper.y - fromBounds.lower.y;
-    };
-    if (fromBounds.upper.x - toBounds.lower.x > distance)
-    {
-      result = GC_DIR_WEST;
-      distance = toBounds.lower.x - fromBounds.upper.x;
+      result = GC_DIR_NORTH;
+      float distance = fromBounds.upper.y - toBounds.lower.y;
+      if (toBounds.upper.x - fromBounds.lower.x > distance)
+      {
+        result = GC_DIR_EAST;
+        distance = toBounds.upper.x - fromBounds.lower.x;
+      };
+      if (toBounds.upper.y - fromBounds.lower.y > distance)
+      {
+        result = GC_DIR_SOUTH;
+        distance = toBounds.upper.y - fromBounds.lower.y;
+      };
+      if (fromBounds.upper.x - toBounds.lower.x > distance)
+      {
+        result = GC_DIR_WEST;
+        distance = toBounds.lower.x - fromBounds.upper.x;
+      };
     };
   };
 
@@ -954,7 +988,7 @@ TFeedbackInfo CConnectionInstance::getFeedbackInfo(const float x, const float y)
     {
       float minX = MIN(FEndVertex1.x, FEndVertex2.x);
       float maxX = MAX(FEndVertex1.x, FEndVertex2.x);
-      if (fabsf(centerPoint - y) <= distance && (minX <= x) && (x <= maxX))
+      if (fabs(centerPoint - y) <= distance && (minX <= x) && (x <= maxX))
         // Close to the center line.
         result = GC_FI_HLINE_MOVE;
       else
@@ -964,14 +998,14 @@ TFeedbackInfo CConnectionInstance::getFeedbackInfo(const float x, const float y)
         {
           // Above the center.
           float actualX = (FEndVertex1.y <= centerPoint) ? FEndVertex1.x : FEndVertex2.x;
-          if (fabsf(actualX - x) <= distance)
+          if (fabs(actualX - x) <= distance)
             result = GC_FI_CONNECTION;
         }
         else
         {
           // Below the center
           float actualX = (FEndVertex1.y > centerPoint) ? FEndVertex1.x : FEndVertex2.x;
-          if (fabsf(actualX - x) <= distance)
+          if (fabs(actualX - x) <= distance)
             result = GC_FI_CONNECTION;
         };
       };
@@ -980,7 +1014,7 @@ TFeedbackInfo CConnectionInstance::getFeedbackInfo(const float x, const float y)
     {
       float minY = MIN(FEndVertex1.y, FEndVertex2.y);
       float maxY = MAX(FEndVertex1.y, FEndVertex2.y);
-      if (fabsf(centerPoint - x) <= distance && (minY <= y) && (y <= maxY))
+      if (fabs(centerPoint - x) <= distance && (minY <= y) && (y <= maxY))
         // Close to the center line.
         result = GC_FI_VLINE_MOVE;
       else
@@ -990,14 +1024,14 @@ TFeedbackInfo CConnectionInstance::getFeedbackInfo(const float x, const float y)
         {
           // Left to the center.
           float actualY = (FEndVertex1.x <= centerPoint) ? FEndVertex1.y : FEndVertex2.y;
-          if (fabsf(actualY - y) <= distance)
+          if (fabs(actualY - y) <= distance)
             result = GC_FI_CONNECTION;
         }
         else
         {
           // Right to the center
           float actualY = (FEndVertex1.x > centerPoint) ? FEndVertex1.y : FEndVertex2.y;
-          if (fabsf(actualY - y) <= distance)
+          if (fabs(actualY - y) <= distance)
             result = GC_FI_CONNECTION;
         };
       };
@@ -1038,12 +1072,12 @@ TConnectionPart CConnectionInstance::labelPartFromPoint(float x, float y)
  *
  * @param lineStyle The new line style to apply.
  */
-void CConnectionInstance::lineStyle(TConnectionLineStyle newStyle)
+void CConnectionInstance::lineStyleSet(TConnectionLineStyle newStyle)
 {
   if (FLineStyle != newStyle)
   {
-    FLineStyle = newStyle;
-    makeDirty(GC_CHANGE_CONNECTION_INSTANCE);
+	FLineStyle = newStyle;
+	makeDirty(GC_CHANGE_CONNECTION_INSTANCE);
   };
 }
 
@@ -1057,39 +1091,77 @@ void CConnectionInstance::lineStyle(TConnectionLineStyle newStyle)
  */
 void CConnectionInstance::moveCenterTo(TVertex position)
 {
-  switch (FDirection1)
+  if (FEnd1 == FEnd2)
   {
-    case GC_DIR_SOUTH:
-    case GC_DIR_NORTH:
+    // Self referencing, means this connections starts and stops at the same end point.
+    float fullWidth;
+    float fullHeight;
+
+    // Compute the center as percentage of the entire width/height.
+    canvas()->currentViewGet()->getWorkspace(&fullWidth, &fullHeight);
+
+    switch (FDirection1)
     {
-      if (FEndVertex1.y <= FEndVertex2.y)
-        FCenter = (position.y - FEndVertex1.y) / (FEndVertex2.y - FEndVertex1.y);
-      else
-        FCenter = (position.y - FEndVertex2.y) / (FEndVertex1.y - FEndVertex2.y);
+      case GC_DIR_SOUTH:
+      case GC_DIR_NORTH:
+      {
+        float offset = (FEnd1Decoration != NULL) ? FEnd1Decoration->height() : 0;
+        if ((FEnd2Decoration != NULL) && FEnd2Decoration->height() > offset)
+          offset = FEnd2Decoration->height();
 
-      if (FCenter < 0)
-        FCenter = 0;
-      if (FCenter > 1)
-        FCenter = 1;
+        if (FDirection1 == GC_DIR_NORTH)
+          FCenter = (FEndVertex1.y - offset - position.y) / fullHeight;
+        else
+          FCenter = (position.y - FEndVertex1.y - offset) / fullHeight;
+        
+        break;
+      }
+      default:
+      {
+        // GC_DIR_EAST/GC_DIR_WEST
+        float offset = (FEnd1Decoration != NULL) ? FEnd1Decoration->width() : 0;
+        if ((FEnd2Decoration != NULL) && FEnd2Decoration->width() > offset)
+          offset = FEnd2Decoration->width();
 
-      break;
-    }
-    default:
-    {
-      // GC_DIR_EAST/GC_DIR_WEST
-      if (FEndVertex1.x <= FEndVertex2.x)
-        FCenter = (position.x - FEndVertex1.x) / (FEndVertex2.x - FEndVertex1.x);
-      else
-        FCenter = (position.x - FEndVertex2.x) / (FEndVertex1.x - FEndVertex2.x);
+        if (FDirection1 == GC_DIR_WEST)
+          FCenter = (FEndVertex1.x - offset - position.x) / fullWidth;
+        else
+          FCenter = (position.x - FEndVertex1.x - offset) / fullWidth;
 
-      if (FCenter < 0)
-        FCenter = 0;
-      if (FCenter > 1)
-        FCenter = 1;
-
-      break;
+        break;
+      };
     };
-  };
+  }
+  else
+    switch (FDirection1)
+    {
+      case GC_DIR_SOUTH:
+      case GC_DIR_NORTH:
+      {
+        if (FEndVertex1.y <= FEndVertex2.y)
+          FCenter = (position.y - FEndVertex1.y) / (FEndVertex2.y - FEndVertex1.y);
+        else
+          FCenter = (position.y - FEndVertex2.y) / (FEndVertex1.y - FEndVertex2.y);
+
+        break;
+      }
+      default:
+      {
+        // GC_DIR_EAST/GC_DIR_WEST
+        if (FEndVertex1.x <= FEndVertex2.x)
+          FCenter = (position.x - FEndVertex1.x) / (FEndVertex2.x - FEndVertex1.x);
+        else
+          FCenter = (position.x - FEndVertex2.x) / (FEndVertex1.x - FEndVertex2.x);
+
+        break;
+      };
+    };
+
+  if (FCenter < 0)
+    FCenter = 0;
+  if (FCenter > 1)
+    FCenter = 1;
+
   makeDirty(GC_CHANGE_CONNECTION_INSTANCE);
 }
 
@@ -1139,7 +1211,7 @@ void CConnectionInstance::moveLabel(TConnectionPart part, float dX, float dY)
  * @param index The index of the sub property to return if it is located in a list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CConnectionInstance::property(const char* name, unsigned int index)
+TGCVariant CConnectionInstance::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -1154,7 +1226,7 @@ TGCVariant CConnectionInstance::property(const char* name, unsigned int index)
             result = FCenter;
             break;
           };
-          case GC_PROPERTY_LABEL1_OFFSET_X:
+		  case GC_PROPERTY_LABEL1_OFFSET_X:
           {
             result = FLabel1Offset.x;
             break;
@@ -1193,7 +1265,7 @@ TGCVariant CConnectionInstance::property(const char* name, unsigned int index)
         if (index < 4)
         {
           result = FColor[index];
-        };
+		};
         break;
       };
     case GC_CONTAINER_HOT_COLOR:
@@ -1232,7 +1304,7 @@ TGCVariant CConnectionInstance::property(const char* name, unsigned int index)
     case GC_CONTAINER_END2_DECORATION:
       {
         result = FEnd2Decoration;
-        break;
+		break;
       };
     case GC_CONTAINER_END2_LABEL:
       {
@@ -1263,15 +1335,15 @@ TGCVariant CConnectionInstance::property(const char* name, unsigned int index)
  * @param index The index of the sub property to return if it is located in a list.
  * @param value The new value of the property. Automatic conversion is performed where possible.
  */
-void CConnectionInstance::property(const char* name, unsigned int index, TGCVariant value)
+void CConnectionInstance::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   switch (getContainerID(name))
   {
-    case GC_CONTAINER_UNKNOWN:
-      {
-        switch (getPropertyID(name))
-        {
-          case GC_PROPERTY_CENTER:
+	case GC_CONTAINER_UNKNOWN:
+	  {
+		switch (getPropertyID(name))
+		{
+		  case GC_PROPERTY_CENTER:
           {
             FCenter = value;
             makeDirty(GC_CHANGE_CONNECTION_INSTANCE);
@@ -1425,7 +1497,7 @@ void CConnectionInstance::render(float currentZoom, bool isHot)
  */
 void CConnectionInstance::updateDecorations(void)
 {
-  TConnectionDecorations decorations = FConnection->decorations();
+  TConnectionDecorations decorations = FConnection->decorationsGet();
 
   updateFigureInstance(&FEnd1Decoration, decorations.end1Decoration);
   updateFigureInstance(&FEnd1Label, decorations.end1Label);
@@ -1446,8 +1518,14 @@ void CConnectionInstance::validate(void)
   {
     CGraphicElement::validate();
 
-    FLayer->validateEndPoint(FEnd1);
-    FLayer->validateEndPoint(FEnd2);
+    FLayer->validateEndPoint(FEnd1, true);
+    FLayer->validateEndPoint(FEnd2, false);
+    FEnd1Decoration->validate();
+    FEnd1Label->validate();
+    FCenterDecoration->validate();
+    FCenterLabel->validate();
+    FEnd2Decoration->validate();
+    FEnd2Label->validate();
 
     if (FDisplayList == 0)
       FDisplayList = glGenLists(1);
@@ -1989,15 +2067,17 @@ void CConnectionLayer::removeInstance(CConnectionInstance* instance)
  * Computes the coordinates of all connection instances touching the given point.
  *
  * @param point The endpoint to validate.
+ * @param handleSelfReferences If true then also consider connections that start and end at the same point.
  */
-void CConnectionLayer::validateEndPoint(CFigureInstance* point)
+void CConnectionLayer::validateEndPoint(CFigureInstance* point, bool handleSelfReferences)
 {
-  typedef vector<CConnectionInstance*> CConnectionList;
+  typedef vector<CConnectionInstance*> CLocalConnectionList;
 
-  CConnectionList northConnections;
-  CConnectionList eastConnections;
-  CConnectionList southConnections;
-  CConnectionList westConnections;
+  CLocalConnectionList northConnections;
+  CLocalConnectionList eastConnections;
+  CLocalConnectionList southConnections;
+  CLocalConnectionList westConnections;
+  CLocalConnectionList selfConnections;
 
   // In order to validate a connection instance both end points must be validated, that is, all
   // connections attached to these end points must be validated and the coordinates of the connection lines
@@ -2011,6 +2091,12 @@ void CConnectionLayer::validateEndPoint(CFigureInstance* point)
   {
     switch ((*iterator)->getDirection(point))
     {
+      case GC_DIR_SELF:
+        {
+          if (handleSelfReferences)
+            selfConnections.push_back(*iterator);
+          break;
+        };
       case GC_DIR_NORTH:
         {
           northConnections.push_back(*iterator);
@@ -2035,36 +2121,65 @@ void CConnectionLayer::validateEndPoint(CFigureInstance* point)
   };
 
   // Sort each direction list so connections do not overlap and compute for each direction the connection coordinates.
-  int index = 1;
   sort(northConnections.begin(), northConnections.end(), moreLeft);
-  for (CConnectionList::iterator iterator = northConnections.begin(); iterator != northConnections.end(); 
-    ++iterator, ++index)
-  {
-    (*iterator)->computeCoordinates(point, GC_DIR_NORTH, (float) index / (northConnections.size() + 1));
-  };
-
-  index = 1;
   sort(eastConnections.begin(), eastConnections.end(), moreTop);
-  for (CConnectionList ::iterator iterator = eastConnections.begin(); iterator != eastConnections.end(); 
-    ++iterator, ++index)
-  {
-    (*iterator)->computeCoordinates(point, GC_DIR_EAST, (float) index / (eastConnections.size() + 1));
-  };
-
-  index = 1;
   sort(southConnections.begin(), southConnections.end(), moreLeft);
-  for (CConnectionList ::iterator iterator = southConnections.begin(); iterator != southConnections.end(); 
-    ++iterator, ++index)
+  sort(westConnections.begin(), westConnections.end(), moreTop);
+
+  CLocalConnectionList::iterator instanceIterator;
+
+  // Distribute all self references so that all sides are equally filled.
+  for (instanceIterator = selfConnections.begin(); instanceIterator != selfConnections.end(); ++instanceIterator)
   {
-    (*iterator)->computeCoordinates(point, GC_DIR_SOUTH, (float) index / (southConnections.size() + 1));
+    CLocalConnectionList* targetList = &northConnections;
+
+    if (eastConnections.size() < targetList->size())
+      targetList = &eastConnections;
+    if (southConnections.size() < targetList->size())
+      targetList = &southConnections;
+    if (westConnections.size() < targetList->size())
+      targetList = &westConnections;
+
+    // Add the point twice. We need room for both ends.
+    targetList->insert(targetList->begin() + targetList->size() / 2, *instanceIterator);
+    targetList->insert(targetList->begin() + targetList->size() / 2, *instanceIterator);
   };
 
-  index = 1;
-  sort(westConnections.begin(), westConnections.end(), moreTop);
-  for (CConnectionList ::iterator iterator = westConnections.begin(); iterator != westConnections.end(); 
-    ++iterator, ++index)
+  int index;
+  CConnectionInstance* lastInstance = NULL;
+  for (instanceIterator = northConnections.begin(), index = 1; instanceIterator != northConnections.end(); 
+    ++instanceIterator, ++index)
   {
-    (*iterator)->computeCoordinates(point, GC_DIR_WEST, (float) index / (westConnections.size() + 1));
+    bool isSelfReference = lastInstance == *instanceIterator;
+    lastInstance = *instanceIterator;
+    lastInstance->computeCoordinates(point, GC_DIR_NORTH, (float) index / (northConnections.size() + 1), isSelfReference);
+  };
+
+  lastInstance = NULL;
+  for (instanceIterator = eastConnections.begin(), index = 1; instanceIterator != eastConnections.end(); 
+    ++instanceIterator, ++index)
+  {
+    bool isSelfReference = lastInstance == *instanceIterator;
+    lastInstance = *instanceIterator;
+    lastInstance->computeCoordinates(point, GC_DIR_EAST, (float) index / (eastConnections.size() + 1), isSelfReference);
+  };
+
+  lastInstance = NULL;
+  for (instanceIterator = southConnections.begin(), index = 1; instanceIterator != southConnections.end(); 
+    ++instanceIterator, ++index)
+  {
+    bool isSelfReference = lastInstance == *instanceIterator;
+    lastInstance = *instanceIterator;
+    lastInstance->computeCoordinates(point, GC_DIR_SOUTH, (float) index / (southConnections.size() + 1), isSelfReference);
+  };
+
+  lastInstance = NULL;
+  for (instanceIterator = westConnections.begin(), index = 1; instanceIterator != westConnections.end(); 
+    ++instanceIterator, ++index)
+  {
+    bool isSelfReference = lastInstance == *instanceIterator;
+    lastInstance = *instanceIterator;
+    lastInstance->computeCoordinates(point, GC_DIR_WEST, (float) index / (westConnections.size() + 1), isSelfReference);
   };
 }
 
