@@ -195,7 +195,7 @@ void CFloater::move(float dX, float dY)
  * @param index If the property is a list then this is the index into that list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CFloater::property(const char* name, unsigned int index)
+TGCVariant CFloater::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -226,7 +226,7 @@ TGCVariant CFloater::property(const char* name, unsigned int index)
  * @param index If the property is a list then this is the index into that list.
  * @param Value The new value of the property. Automatic conversion is performed where possible.
  */
-void CFloater::property(const char* name, unsigned int index, TGCVariant value)
+void CFloater::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   switch (getContainerID(name))
   {
@@ -500,13 +500,6 @@ void CFeedbackLayer::CFeedbackInstanceListener::onDestroy(CGCBase* object)
     layer->removeFromSelection((CFigureInstance*) object);
 }
 
-//----------------------------------------------------------------------------------------------------------------------
-
-void CFeedbackLayer::CFeedbackInstanceListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
-{
-  // Let the owner layer report any errors up the chain. This is not our business.
-}
-
 //----------------- CFeedbackLayer ------------------------------------------------------------------------------------
 
 CFeedbackLayer::CFeedbackLayer(CGCView* view): CLayer("feedback", view->canvas())
@@ -897,6 +890,12 @@ TFeedbackInfo CFeedbackLayer::getFeedbackInfo(const TVertex& coords, float zoom,
                       result = GC_FI_RESIZE_SOUTH_EAST;
               };
 
+          if (result == GC_FI_SELECTION_BODY)
+          {
+            // We are somewhere within a figure's bounds. Ask it for a more specific feedback.
+            result = (*instance)->getFeedbackInfo(coords);
+          };
+
           break;
         };
       };
