@@ -24,14 +24,11 @@
 #include "myx_gc_gl_helper.h"
 
 static map<string, TFigureElementResize> resizeLookup;
+static map<wstring, TFeedbackInfo> resizeDirectionLookup;
 static map<string, TFigureElementLayout> layoutLookup;
 static map<string, TAlignment> alignmentLookup;
 
-#ifdef __GNUC__
-  static map<wstring, TActionType> actionLookup;
-#else
-  static hash_map<wstring, TActionType> actionLookup;
-#endif
+static map<wstring, TActionType> actionLookup;
 
 //----------------------------------------------------------------------------------------------------------------------
 
@@ -86,6 +83,7 @@ void CFigureParser::checkLookupTables(void)
     actionLookup[L"toggle"] = GC_ACTION_TOGGLE;
     actionLookup[L"collapse"] = GC_ACTION_COLLAPSE;
     actionLookup[L"expand"] = GC_ACTION_EXPAND;
+    actionLookup[L"resizeFigure"] = GC_ACTION_RESIZE;
     actionLookup[L"changeStyle"] = GC_ACTION_CHANGE_STYLE;
     actionLookup[L"dragFigure"] = GC_ACTION_DRAG_FIGURE;
     actionLookup[L"do"] = GC_ACTION_APPLICATION;
@@ -99,6 +97,18 @@ void CFigureParser::checkLookupTables(void)
     resizeLookup["all"] = GC_RESIZE_ALL;
   };
 
+  if (resizeDirectionLookup.size() == 0)
+  {
+    resizeDirectionLookup[L"north"] = GC_FI_RESIZE_NORTH;
+    resizeDirectionLookup[L"north-east"] = GC_FI_RESIZE_NORTH_EAST;
+    resizeDirectionLookup[L"east"] = GC_FI_RESIZE_EAST;
+    resizeDirectionLookup[L"south-east"] = GC_FI_RESIZE_SOUTH_EAST;
+    resizeDirectionLookup[L"south"] = GC_FI_RESIZE_SOUTH;
+    resizeDirectionLookup[L"south-west"] = GC_FI_RESIZE_SOUTH_WEST;
+    resizeDirectionLookup[L"west"] = GC_FI_RESIZE_WEST;
+    resizeDirectionLookup[L"north-west"] = GC_FI_RESIZE_NORTH_WEST;
+  };
+
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -270,6 +280,10 @@ void CFigureParser::parseActions(string source, CFigureElementTemplate* template
               parameter = parameter.substr(1, parameter.length() - 2);
 
             action.parameters.push_back(utf8ToUtf16(parameter.substr(head, tail)));
+
+            // Special case resize action. Convert the direction parameter to a feedback info tag.
+            if (action.type == GC_ACTION_RESIZE)
+              action.feedbackInfo = resizeDirectionLookup[action.parameters[0]];
           };
         };
         template_->addAction(action);
@@ -282,24 +296,6 @@ void CFigureParser::parseActions(string source, CFigureElementTemplate* template
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Retrieves the value of the property given by path. The path syntax is must be something like (here expressed as regex)
- * (container)*(property), where container is a slash and the name of a container class (e.g. layers, figures) and
- * property is the name of a simple property of that container.
- *
- * @param name The name of the property to return.
- * @param index If the property is a list then this is the index into that list.
- * @return A description of the property value and, if the property is simple, the actual value.
- */
-TGCVariant CFigureParser::property(const char* name, unsigned int index)
-{
-  TGCVariant variant;
-
-  return variant;
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Parses a single layout definition and creates a figure template from it.
  *
@@ -332,3 +328,22 @@ void CFigureParser::parseLayoutDefinition(xmlNodePtr definition, CGCModel* model
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Retrieves the value of the property given by path. The path syntax is must be something like (here expressed as regex)
+ * (container)*(property), where container is a slash and the name of a container class (e.g. layers, figures) and
+ * property is the name of a simple property of that container.
+ *
+ * @param name The name of the property to return.
+ * @param index If the property is a list then this is the index into that list.
+ * @return A description of the property value and, if the property is simple, the actual value.
+ */
+TGCVariant CFigureParser::propertyGet(const char* name, unsigned int index)
+{
+  TGCVariant variant;
+
+  return variant;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+
