@@ -28,6 +28,7 @@
 
 //#define FONTCLASS FTGLPixmapFont
 #define FONTCLASS FTGLBitmapFont
+//#define FONTCLASS FTGLTextureFont
 
 //----------------------------------------------------------------------------------------------------------------------
 
@@ -240,9 +241,21 @@ string CFontManager::getFontFile(string family, string style, int weight)
   string filename;
 
 #ifdef USE_FONTCONFIG
-  FcPattern *pattern= FcPatternCreate();
+  static std::map<std::string,std::string> fileCache;
+  typedef struct {
+    char *family;
+    char *aliases[10];
+  } FamilyAlias;
+  static FamilyAlias fontFamilyAliases[]= {
+    {"Tahoma", {"Bitstream Vera Sans", "Bitstream Vera", "Nimbus Sans L", "Arial", NULL}},
+    {"Tahoma-Bold", {"hlv", "Bitstream Vera Sans", "Bitstream Vera", "Nimbus Sans L", "Arial", NULL}},
+    {NULL, {NULL}}
+  };
+
   FcResult result;
   int slant;
+  int f, i;
+  const char *tryFamily= family.c_str();
   
   if (style == "italic")
     slant= FC_SLANT_ITALIC;
@@ -250,41 +263,45 @@ string CFontManager::getFontFile(string family, string style, int weight)
     slant= FC_SLANT_OBLIQUE;
   else
     slant= FC_SLANT_ROMAN;
-  
-  FcPatternAddString(pattern, FC_FAMILY, (FcChar8*)family.c_str());
-  FcPatternAddInteger(pattern, FC_SLANT, slant);
-  FcPatternAddInteger(pattern, FC_WEIGHT, weight);
 
-  if (!FcConfigSubstitute(FcConfigGetCurrent(), pattern, FcMatchPattern))
+  for (f= 0; fontFamilyAliases[f].family!=NULL; f++)
+    if (strcasecmp(fontFamilyAliases[f].family, family.c_str())==0)
+  	break;
+  if (fontFamilyAliases[f].family == NULL) f= -1;
+
+  i= -1;
+  do 
   {
-    g_warning("Error calling FcConfigSubstitute() on font '%s %s'",
-              family.c_str(), style.c_str());
-    FcPatternDestroy(pattern);
-  }
-  else
-  {
-    FcPattern *fpat;
-    
-    FcDefaultSubstitute(pattern);
-  
-    fpat= FcFontMatch(FcConfigGetCurrent(), pattern, &result);
-    if (result != FcResultMatch)
-      g_warning("Error looking up for font '%s %s'",
+    FcPattern *pattern= FcPatternCreate();
+    FcPatternAddString(pattern, FC_FAMILY, (FcChar8*)tryFamily);
+    FcPatternAddInteger(pattern, FC_SLANT, slant);
+    FcPatternAddInteger(pattern, FC_WEIGHT, weight);
+
+    if (!FcConfigSubstitute(FcConfigGetCurrent(), pattern, FcMatchPattern))
+    {
+      g_warning("Error calling FcConfigSubstitute() on font '%s %s'",
                 family.c_str(), style.c_str());
+      FcPatternDestroy(pattern);
+    }
     else
     {
+      FcPattern *fpat;
       FcValue value;
-      
-      if (FcPatternGet(fpat, FC_FILE, 0, &value))
+
+      FcDefaultSubstitute(pattern);
+  
+      fpat= FcFontMatch(FcConfigGetCurrent(), pattern, &result);
+      if (fpat)
       {
-        g_message("FILE --> %s", value.u.s);
-      
-        filename= (char*)value.u.s;
+        if (FcPatternGet(fpat, FC_FILE, 0, &value) == FcResultMatch)
+        {
+          filename= (char*)value.u.s;
+        }
+        FcPatternDestroy(fpat);
       }
-      FcPatternDestroy(fpat);
+      FcPatternDestroy(pattern);
     }
-    FcPatternDestroy(pattern);
-  }
+  } while (filename.empty() && f >= 0 && (tryFamily= fontFamilyAliases[f].aliases[++i])!=NULL);
 #elif defined(__APPLE__)
   FSSpec pathSpec;
   FSRef fsref;
