@@ -25,9 +25,9 @@
 
 //----------------- CInstanceListener ----------------------------------------------------------------------------------
 
-void CLayer::CInstanceListener::onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType)
+void CLayer::CInstanceListener::onAction(CGCBase* sender, CGCBase* origin, TAction** action)
 {
-  layer->action(origin, actionType);
+  layer->action(origin, action);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -108,22 +108,6 @@ void CLayer::checkError(void)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Marks the display list for this layer as invalid, hence it will be recreated next time validate is called.
- * If a list already exists then it is freed.
- */
-void CLayer::makeDirty(void)
-{
-  if (!FDirty)
-  {
-    FDirty = true;
-    if (!destroying() && !updating())
-      canvas()->refresh();
-  };
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /*
  * Helper method to determine the transformed vertices of the given figure instance. The layer applies its own
  * transformations and only renders the figure instance.
@@ -160,12 +144,15 @@ void CLayer::validate(void)
     FDirty = false;
 
     FMinimalScale = FScaling[0] < FScaling[1] ? FScaling[0] : FScaling[1];
+        
     validateLayerContent();
-
+    
     // Give the instances the opportunity to trigger validation of their associated figures.
     for (CFigureInstances::iterator iterator = FInstances.begin(); iterator != FInstances.end(); ++iterator)
+    {
       (*iterator)->validate();
-
+      checkError();
+    }
     checkError();
   };
 }
@@ -208,7 +195,7 @@ void CLayer::bringToFront(CFigureInstance* instance, CFigureInstance* relativeTo
         else
           FInstances.insert(relIterator, instance); // insert before the next one
         canvas()->refresh();
-        return;
+		return;
       }
     }
     FInstances.push_back(instance);
@@ -311,31 +298,43 @@ void CLayer::getHitTestInfoAt(CHitResults* hits, TVertex point, bool singleHit)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Sets the layer's enabled state.
+ * Returns the current enabled state.
  *
- * @param IsEnabled Set it to true if you want the layer to be visible.
+ * @return The current enabled state.
  */
-void CLayer::enabled(bool IsEnabled)
+bool CLayer::enabledGet(void)
 {
-  if (FEnabled != IsEnabled)
-  {
-    FEnabled = IsEnabled;
-    makeDirty();
-  };
+  return FEnabled;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Returns the current enabled state.
+ * Sets the layer's enabled state.
  *
- * @return The current enabled state.
+ * @param IsEnabled Set it to true if you want the layer to be visible.
  */
-bool CLayer::enabled(void)
+void CLayer::enabledSet(bool IsEnabled)
 {
-  return FEnabled;
+  if (FEnabled != IsEnabled)
+  {
+	FEnabled = IsEnabled;
+	makeDirty();
+  };
 }
 
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Return internal list of instances.
+ */
+CFigureInstances CLayer::instancesGet()
+{
+  return FInstances;
+}
+
+
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
@@ -355,6 +354,22 @@ TVertex CLayer::layerToView(TVertex coords)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Marks the display list for this layer as invalid, hence it will be recreated next time validate is called.
+ * If a list already exists then it is freed.
+ */
+void CLayer::makeDirty(void)
+{
+  if (!FDirty)
+  {
+    FDirty = true;
+    if (!destroying() && !updating())
+      canvas()->refresh();
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Retrieves the value of the property given by path. The path syntax is must be something like (here expressed as regex)
  * (container)*(property), where container is a slash and the name of a container class (e.g. layers, figures) and
@@ -364,7 +379,7 @@ TVertex CLayer::layerToView(TVertex coords)
  * @param index If the property is a list then this is the index into that list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CLayer::property(const char* name, unsigned int index)
+TGCVariant CLayer::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -376,7 +391,7 @@ TGCVariant CLayer::property(const char* name, unsigned int index)
         {
           case GC_PROPERTY_NAME:
             {
-              result = utf16ToUtf8(FName);
+			  result = utf16ToUtf8(FName);
               break;
             };
           case GC_PROPERTY_DESCRIPTION:
@@ -403,7 +418,7 @@ TGCVariant CLayer::property(const char* name, unsigned int index)
         break;
       };
     case GC_CONTAINER_FIGURE_INSTANCES:
-      {
+	  {
         if (index < FInstances.size())
           result = FInstances[index];
         break;
@@ -418,7 +433,7 @@ TGCVariant CLayer::property(const char* name, unsigned int index)
     case GC_CONTAINER_SCALING:
       {
         if (index < 3)
-          result = FScaling[index];
+		  result = FScaling[index];
         break;
       };
     case GC_CONTAINER_TRANSLATION:
@@ -441,13 +456,13 @@ TGCVariant CLayer::property(const char* name, unsigned int index)
  * @param index If the property is a list then this is the index into that list.
  * @param Value The new value of the property. Automatic conversion is performed where possible.
  */
-void CLayer::property(const char* name, unsigned int index, TGCVariant value)
+void CLayer::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   switch (getContainerID(name))
   {
     case GC_CONTAINER_UNKNOWN:
       {
-        switch (getPropertyID(name))
+		switch (getPropertyID(name))
         {
           case GC_PROPERTY_NAME:
             {
@@ -486,7 +501,7 @@ void CLayer::property(const char* name, unsigned int index, TGCVariant value)
     case GC_CONTAINER_TRANSLATION:
       {
         if (index < 3)
-        {
+		{
           FTranslation[index] = value;
           change(this, GC_CHANGE_LAYER_PROPERTY);
         };
@@ -589,26 +604,26 @@ void CLayer::scale(float Sx, float Sy, float Sz, bool Accumulative)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/** 
+/**
  * Scales the layer by the amount given in Factor. If Accumulative is true then the new scale factors are multiplied
  * with the existing values. This version of scale uses an array of values in the parameter list.
  *
  * @param Factor An array of 3 scale values, one for each axis.
  * @param Accumulative If true then the given values are added to any existing values otherwiese they are used as given.
  */
-void CLayer::scale(const float Factor[3], bool Accumulative)
+void CLayer::scaleV(const float Factor[3], bool Accumulative)
 {
   if (Accumulative)
   {
-    FScaling[0] += Factor[0];
-    FScaling[1] += Factor[1];
-    FScaling[2] += Factor[2];
+	FScaling[0] += Factor[0];
+	FScaling[1] += Factor[1];
+	FScaling[2] += Factor[2];
   }
   else
   {
-    FScaling[0] = Factor[0];
-    FScaling[1] = Factor[1];
-    FScaling[2] = Factor[2];
+	FScaling[0] = Factor[0];
+	FScaling[1] = Factor[1];
+	FScaling[2] = Factor[2];
   };
   change(this, GC_CHANGE_LAYER_PROPERTY);
   makeDirty();
@@ -721,29 +736,29 @@ TVertex CLayer::viewToLayer(TVertex coords)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Sets the layer's visibility state.
+ * Returns the visibility state.
  *
- * @param IsVisible Set it to true if you want the layer to be visible.
+ * @return The current visibility state.
  */
-void CLayer::visible(bool IsVisible)
+bool CLayer::visibleGet(void)
 {
-  if (FVisible != IsVisible)
-  {
-    FVisible = IsVisible;
-    change(this, GC_CHANGE_LAYER_VISIBILITY);
-  };
+  return FVisible;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Returns the visibility state.
+ * Sets the layer's visibility state.
  *
- * @return The current visibility state.
+ * @param IsVisible Set it to true if you want the layer to be visible.
  */
-bool CLayer::visible(void)
+void CLayer::visibleSet(bool IsVisible)
 {
-  return FVisible;
+  if (FVisible != IsVisible)
+  {
+	FVisible = IsVisible;
+	change(this, GC_CHANGE_LAYER_VISIBILITY);
+  };
 }
 
 //----------------- CViewListener --------------------------------------------------------------------------------------
@@ -753,20 +768,6 @@ void CGridLayer::CViewListener::onChange(CGCBase* sender, CGCBase* origin, TGCCh
   layer->makeDirty();
 }
 
-//----------------------------------------------------------------------------------------------------------------------
-
-void CGridLayer::CViewListener::onDestroy(CGCBase* object)
-{
-  // This event will never occur because the viewer frees its grid layer before it is destroyed.
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-void CGridLayer::CViewListener::onError(CGCBase* sender, CGCBase* origin, const char* message)
-{
-  // Let the owner layer report any errors up the chain. This is not our business.
-}
-
 //----------------- CGridLayer -----------------------------------------------------------------------------------------
 
 CGridLayer::CGridLayer(CGCView* view): CLayer("grid", view->canvas())
