@@ -98,29 +98,6 @@ void CGCModel::addFigure(CFigure* figure)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Removes the given figure from the internal figures list.
- *
- * @param figure The figure to be removed.
- */
-void CGCModel::removeFigure(CFigure* figure)
-{
-  if (!updating())
-  {
-    for (CFigureList::iterator iterator = FFigures.begin(); iterator != FFigures.end(); ++iterator)
-    {
-      if (*iterator == figure)
-      {
-        (*iterator)->removeListener(&FListener);
-        FFigures.erase(iterator);
-        break;
-      };
-    };
-  };
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Removes all connections.
  */
@@ -247,7 +224,7 @@ CFigureTemplate* CGCModel::createLayout(wstring type, wstring layoutClass)
 
   if (result == NULL)
   {
-    result = new CFigureTemplate(this, type, layoutClass);
+	result = new CFigureTemplate(this, type, layoutClass);
     FLayouts.insert(CLayoutPair(type, result));
   };
 
@@ -289,7 +266,7 @@ CGCTexture* CGCModel::createTexture(const TLODList& lodData, const wstring& id,
     mapper["nearest-mipmap-nearest"] = GL_NEAREST_MIPMAP_NEAREST;
     mapper["linear-mipmap-nearest"] = GL_LINEAR_MIPMAP_NEAREST;
     mapper["nearest-mipmap-linear"] = GL_NEAREST_MIPMAP_LINEAR;
-    mapper["linear-mipmap-linear"] = GL_LINEAR_MIPMAP_LINEAR;
+	mapper["linear-mipmap-linear"] = GL_LINEAR_MIPMAP_LINEAR;
     mapper["decal"] = GL_DECAL;
     mapper["modulate"] = GL_MODULATE;
     mapper["blend"] = GL_BLEND;
@@ -314,27 +291,27 @@ CGCTexture* CGCModel::createTexture(const TLODList& lodData, const wstring& id,
   GLenum magFilter = GL_NEAREST;
   iterator = mapper.find(magnificationFilterStr);
   if (iterator != mapper.end())
-    magFilter = iterator->second;
+	magFilter = iterator->second;
 
   GLenum textureMode = GL_DECAL;
   iterator = mapper.find(mode);
   if (iterator != mapper.end())
-    textureMode = iterator->second;
+	textureMode = iterator->second;
 
   TLODList prefixedLodData;
   for (TLODList::const_iterator path= lodData.begin(); path != lodData.end(); ++path)
   {
-    wstring finalPath;
-    if (path->substr(0, 2) == L"./")
-      finalPath = FTexturePath + path->substr(2);
-    else 
-      // If an absolute path is given then do not apply the texture image path.
-      if ((*path)[0] == '/' || (path->size() > 2 && (*path)[1] == ':'))
-        finalPath = *path;
-      else
-        finalPath = FTexturePath + *path;
+	wstring finalPath;
+	if (path->substr(0, 2) == L"./")
+	  finalPath = FTexturePath + path->substr(2);
+	else
+	  // If an absolute path is given then do not apply the texture image path.
+	  if ((*path)[0] == '/' || (path->size() > 2 && (*path)[1] == ':'))
+		finalPath = *path;
+	  else
+		finalPath = FTexturePath + *path;
 
-    prefixedLodData.push_back(finalPath);
+	prefixedLodData.push_back(finalPath);
   }
   CGCTexture* result = new CGCTexture(prefixedLodData, id, wrapModeS, wrapModeT, minFilter, magFilter, dimensions, textureMode);
   FTextures[id] = result;
@@ -383,9 +360,9 @@ CFigureTemplate* CGCModel::layout(wstring type, wstring layoutClass)
       {
         result = start->second;
         break;
-      };
-      ++start;
-    };
+	  };
+	  ++start;
+	};
 
   return result;
 }
@@ -401,7 +378,7 @@ CFigureTemplate* CGCModel::layout(wstring type, wstring layoutClass)
  * @param index If the property is a list then this is the index into that list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CGCModel::property(const char* name, unsigned int index)
+TGCVariant CGCModel::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -415,7 +392,7 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
             {
               result = "canvas model";
               break;
-            };
+			};
           case GC_PROPERTY_DESCRIPTION:
             {
               result = "The central model of the canvas containing all styles, layouts and figures.";
@@ -452,12 +429,12 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
 
         break;
       };
-    case GC_CONTAINER_LAYOUTS:
+	case GC_CONTAINER_LAYOUTS:
       {
         if (index < FLayouts.size())
         {
           CLayoutList::iterator iterator = FLayouts.begin();
-          while (index > 0)
+		  while (index > 0)
           {
             ++iterator;
             --index;
@@ -481,7 +458,7 @@ TGCVariant CGCModel::property(const char* name, unsigned int index)
  * @param index If the property is a list then this is the index into that list.
  * @param Value The new value of the property. Automatic conversion is performed where possible.
  */
-void CGCModel::property(const char* name, unsigned int index, TGCVariant value)
+void CGCModel::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   // No properties to set at this point.
 }
@@ -497,13 +474,36 @@ void CGCModel::removeConnection(CConnection* connection)
 {
   if (!destroying())
   {
-    for (CConnectionList::iterator iterator = FConnections.begin(); iterator != FConnections.end(); ++iterator)
-      if (*iterator == connection)
+	for (CConnectionList::iterator iterator = FConnections.begin(); iterator != FConnections.end(); ++iterator)
+	  if (*iterator == connection)
+	  {
+		connection->removeListener(&FListener);
+		FConnections.erase(iterator);
+		break;
+	  };
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Removes the given figure from the internal figures list.
+ *
+ * @param figure The figure to be removed.
+ */
+void CGCModel::removeFigure(CFigure* figure)
+{
+  if (!updating())
+  {
+    for (CFigureList::iterator iterator = FFigures.begin(); iterator != FFigures.end(); ++iterator)
+    {
+      if (*iterator == figure)
       {
-        connection->removeListener(&FListener);
-        FConnections.erase(iterator);
+        (*iterator)->removeListener(&FListener);
+        FFigures.erase(iterator);
         break;
       };
+    };
   };
 }
 
@@ -529,13 +529,13 @@ void CGCModel::removeStyle(wstring ID)
 void CGCModel::setTexturePath(const string& path)
 {
   if (path.size() == 0)
-    FTexturePath.clear();
+	FTexturePath.clear();
   else
   {
-    // Make sure the path ends with a slash.
-    FTexturePath = utf8ToUtf16(path);
-    if (FTexturePath[FTexturePath.size() - 1] != '/')
-      FTexturePath += '/';
+	// Make sure the path ends with a slash.
+	FTexturePath = utf8ToUtf16(path);
+	if (FTexturePath[FTexturePath.size() - 1] != '/')
+	  FTexturePath += '/';
   };
 }
 
