@@ -50,7 +50,7 @@ CGCStyle::~CGCStyle(void)
  * @param index The index of the sub property to return if it is located in a list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CGCStyle::property(const char* name, unsigned int index)
+TGCVariant CGCStyle::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -88,7 +88,7 @@ TGCVariant CGCStyle::property(const char* name, unsigned int index)
  * @param index The index of the sub property to return if it is located in a list.
  * @param value The new value of the property. Automatic conversion is performed where possible.
  */
-void CGCStyle::property(const char* name, unsigned int index, TGCVariant value)
+void CGCStyle::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   // There are currently no properties that could be changed. The name is a unique identifier and must not be changed.
 }
