@@ -279,6 +279,9 @@ GLuint CSVGParser::parseElement(xmlNodePtr svg, bool doFill, GLubyte* fillColor,
           };
       };
 
+      if (texture != NULL)
+        texture->validateHandle();
+
       resultList = glGenLists(1);
       // We need the display lists executed while being compiled to have the modelview matrix updated correctly.
       // This is needed to compute the bounding box correctly.
@@ -361,17 +364,13 @@ GLuint CSVGParser::parseElement(xmlNodePtr svg, bool doFill, GLubyte* fillColor,
 GLuint CSVGParser::parseGroup(xmlNodePtr svg, bool doFill, GLubyte* fillColor, bool doStroke, GLubyte* strokeColor, 
                               float strokeWidth, float masterAlpha)
 {
-  xmlNodePtr child = svg->children;
   vector<GLuint> lists;
-  while (child != NULL)
-  {
+  for (xmlNodePtr child = svg->children; child != NULL; child = child->next)
     if (child->type == XML_ELEMENT_NODE)
     {
       GLuint List = parseElement(child, doFill, fillColor, doStroke, strokeColor, strokeWidth, masterAlpha);
       lists.push_back(List);
     };
-    child = child->next;
-  };
 
   GLuint resultList = glGenLists(1);
   glNewList(resultList, GL_COMPILE);
@@ -407,6 +406,8 @@ void CSVGParser::parseImage(xmlNodePtr svg, bool render)
       texture = FModel->createTexture(lodData, id, "clamp-to-edge", "clamp-to-edge", "linear-mipmap-linear", 
         "linear", 2, "modulate");
     };
+    // Make sure the handle of the texture is created before a display list is created.
+    texture->validateHandle();
   }
   else
   {
@@ -474,7 +475,9 @@ void CSVGParser::parseLine(xmlNodePtr svg, bool doStroke, GLubyte* strokeColor,
 
     glBegin(GL_LINES);
       glVertex2f(X1, Y1);
+    //  glNormal3f(0, 0, 1);
       glVertex2f(X2, Y2);
+    //  glNormal3f(0, 0, 1);
     glEnd();
   };
 }
@@ -640,10 +643,10 @@ GLuint CSVGParser::parseText(xmlNodePtr svg, bool doFill, GLubyte* fillColor, bo
       glNewList(localList, GL_COMPILE_AND_EXECUTE);
       if (fillColor != NULL)
         glColor4ubv(fillColor);
-        // FTGL uses the lower left corner as origin, we however have the upper left corner for this task.
-        // Adjust the vertical coordinate accordingly.
-        glRasterPos2f(0, box.lower.y);
-        fontManager()->textOut(output, fontFamily, fontStyle, weight, fontSize);
+      // FTGL uses the lower left corner as origin, we however have the upper left corner for this task.
+      // Adjust the vertical coordinate accordingly.
+      glRasterPos2f(0, box.lower.y);
+      fontManager()->textOut(output, fontFamily, fontStyle, weight, fontSize);
       glEndList();
       lists.push_back(localList);
     };
@@ -676,41 +679,41 @@ GLuint CSVGParser::parseText(xmlNodePtr svg, bool doFill, GLubyte* fillColor, bo
  * Parsers the given string and interprets the content as one or more transformation of the form:
  *   translate(x, y, z) scale(x, y, z) etc.
  *
- * @param Transformation The textual representation of the transformation to parse and perform.
+ * @param transformation The textual representation of the transformation to parse and perform.
  */
-void CSVGParser::parseTransformation(char* Transformation)
+void CSVGParser::parseTransformation(char* transformation)
 {
-  string Delimiters(" ,()\t\n");
-  StringTokenizer tokenizer(Transformation, Delimiters);
+  string delimiters(" ,()\t\n");
+  StringTokenizer tokenizer(transformation, delimiters);
 
   // Each run in the outer loop is one transformation.
   while (tokenizer.hasMoreTokens())
   {
     // Skip leading white spaces.
-    string Token = tokenizer.nextToken();
+    string token = tokenizer.nextToken();
 
-    int Type = 0;
-    if (Token == "matrix")
-      Type = 1;
+    int type = 0;
+    if (token == "matrix")
+      type = 1;
     else
-      if (Token == "translate")
-        Type = 2;
+      if (token == "translate")
+        type = 2;
       else
-        if (Token == "scale")
-          Type = 3;
+        if (token == "scale")
+          type = 3;
         else
-          if (Token == "rotate")
-            Type = 4;
+          if (token == "rotate")
+            type = 4;
           else
-            if (Token == "skewX")
-              Type = 5;
+            if (token == "skewX")
+              type = 5;
             else
-              if (Token == "skewY")
-                Type = 6;
+              if (token == "skewY")
+                type = 6;
               else
                 break;
 
-    switch (Type)
+    switch (type)
     {
       case 1: // matrix
         {
@@ -726,9 +729,9 @@ void CSVGParser::parseTransformation(char* Transformation)
             0, 0, 0, 1
           };
 
-          for (int J = 0; J < 2; J++)
+          for (int j = 0; j < 2; j++)
             for (int i = 0; i < 2; i++)
-              matrix[J * 4 + i] = tokenizer.nextTokenAsFloat();
+              matrix[j * 4 + i] = tokenizer.nextTokenAsFloat();
           matrix[12] = tokenizer.nextTokenAsFloat();
           matrix[13] = tokenizer.nextTokenAsFloat();
           glMultMatrixf(matrix);
@@ -738,22 +741,22 @@ void CSVGParser::parseTransformation(char* Transformation)
       case 2: // Translation
         {
           // One or two floats are possible here.
-          float TX = tokenizer.nextTokenAsFloat();
-          float TY = 0;
+          float tx = tokenizer.nextTokenAsFloat();
+          float ty = 0;
           if (tokenizer.hasMoreTokens())
-            TY = tokenizer.nextTokenAsFloat();
-          glTranslatef(TX, TY, 0);
+            ty = tokenizer.nextTokenAsFloat();
+          glTranslatef(tx, ty, 0);
 
           break;
         };
       case 3: // Scaling
         {
           // One or two floats are possible here.
-          float SX = tokenizer.nextTokenAsFloat();
-          float SY = SX;
+          float sx = tokenizer.nextTokenAsFloat();
+          float sy = sx;
           if (tokenizer.hasMoreTokens())
-            SY = tokenizer.nextTokenAsFloat();
-          glScalef(SX, SY, 0);
+            sy = tokenizer.nextTokenAsFloat();
+          glScalef(sx, sy, 0);
 
           break;
         };
@@ -786,7 +789,7 @@ void CSVGParser::parseTransformation(char* Transformation)
             0, 0, 0, 1
           };
 
-          int index = (Type == 5) ? 4 : 1;
+          int index = (type == 5) ? 4 : 1;
           matrix[index] = tan(tokenizer.nextTokenAsFloat() * M_PI / 180);
           glMultMatrixd(matrix);
           break;
@@ -829,6 +832,7 @@ void CSVGParser::renderVertices(bool doFill, GLubyte* fillColor, bool doStroke,
       if (TextureCoordinates != NULL)
         glTexCoord2d((*TextureCoordinates)[i].x, (*TextureCoordinates)[i].y);
       glVertex2f(vertices[i].x, vertices[i].y);
+    //  glNormal3f(0, 0, 1);
 
       // Update the accumulated bounding box.
       FBoundsComputer.include(matrix, vertices[i]);
@@ -844,7 +848,10 @@ void CSVGParser::renderVertices(bool doFill, GLubyte* fillColor, bool doStroke,
     {
       glBegin(closeShape ? GL_LINE_LOOP : GL_LINE_STRIP);
       for (int i = 0; i < (int) vertices.size(); i++)
+      {
         glVertex2f(vertices[i].x, vertices[i].y);
+     //   glNormal3f(0, 0, 1);
+      };
       glEnd();
     }
     else
@@ -870,16 +877,19 @@ void CSVGParser::renderVertices(bool doFill, GLubyte* fillColor, bool doStroke,
         dY = halfWidth * cos(Angle);
         glBegin(GL_POLYGON);
           glVertex2f(vertices[index].x - dX, vertices[index].y + dY);
-          // Update the accumulated bounding box.
+      //    glNormal3f(0, 0, 1);
           FBoundsComputer.include(matrix, vertices[index]);
 
           glVertex2f(vertices[NextIndex].x - dX, vertices[NextIndex].y + dY);
+       //   glNormal3f(0, 0, 1);
           FBoundsComputer.include(matrix, vertices[index]);
 
           glVertex2f(vertices[NextIndex].x + dX, vertices[NextIndex].y - dY);
+        //  glNormal3f(0, 0, 1);
           FBoundsComputer.include(matrix, vertices[index]);
 
           glVertex2f(vertices[index].x + dX, vertices[index].y - dY);
+        //  glNormal3f(0, 0, 1);
           FBoundsComputer.include(matrix, vertices[index]);
         glEnd();
 
@@ -901,16 +911,12 @@ void CSVGParser::renderVertices(bool doFill, GLubyte* fillColor, bool doStroke,
 void CSVGParser::convert(xmlNodePtr svg, GLuint displayList)
 {
   vector<GLuint> lists;
-  xmlNodePtr child = svg->children;
-  while (child != NULL)
-  {
+  for (xmlNodePtr child = svg->children; child != NULL; child = child->next)
     if (child->type == XML_ELEMENT_NODE)
     {
       GLuint localList = parseElement(child, false, NULL, false, NULL, 1, 1);
       lists.push_back(localList);
     };
-    child = child->next;
-  };
 
   // Not necessary for creating the list but used to create a clean starting point to transform bounding boxes.
   glLoadIdentity();
@@ -918,9 +924,13 @@ void CSVGParser::convert(xmlNodePtr svg, GLuint displayList)
   glNewList(displayList, GL_COMPILE);
 
   glColor4f(0, 0, 0, 1);
-
   for (vector<GLuint>::iterator iterator = lists.begin(); iterator != lists.end(); iterator++)
+  {
     glCallList(*iterator);
+    
+    //glRectf(FBoundsComputer.boundingBox().upper.x, FBoundsComputer.boundingBox().upper.y, FBoundsComputer.boundingBox().lower.x, FBoundsComputer.boundingBox().lower.y);
+  };
+
   glEndList();
 }
 
