@@ -350,12 +350,34 @@ string utf8ToANSI(const string& source)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * @brief User defined file handling callback for handling file i/o in png lib.
- *        We have to make it so because of the FILE allocation and png being in a DLL on Windows.
+ * User defined file handling callback for handling file i/o in png lib.
+ * We have to make it so because of the FILE allocation and png being in a DLL on Windows.
  */
-void pngRead(png_structp png_ptr, png_bytep data, png_size_t length)
+void pngRead(png_structp png, png_bytep data, png_size_t length)
 {
-  fread(data, length, 1, (FILE*) png_ptr->io_ptr);
+  fread(data, length, 1, (FILE*) png->io_ptr);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * User defined file handling callback for handling file i/o in png lib.
+ * We have to make it so because of the FILE allocation and png being in a DLL on Windows.
+ */
+void pngWrite(png_structp png, png_bytep data, png_size_t length)
+{
+  fwrite(data, length, 1, (FILE*) png->io_ptr);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * User defined file handling callback for handling file i/o in png lib.
+ * We have to make it so because of the FILE allocation and png being in a DLL on Windows.
+ */
+void pngFlush(png_structp png)
+{
+  fflush((FILE*) png->io_ptr);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -409,9 +431,8 @@ TImage* loadPNG(const wstring& filename)
         return NULL;
       };
 
-      png_set_read_fn(png, file, &pngRead);
-
       png_init_io(png, file);
+      png_set_read_fn(png, file, pngRead);
       png_set_sig_bytes(png, sizeof(magic));
 
       png_read_info(png, info);
@@ -493,98 +514,113 @@ TImage* loadPNG(const wstring& filename)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-
-
 static jmp_buf png_error_jmpbuf;
 
-static void png_error_handler(png_structp png_ptr, png_const_charp msg)
+static void png_error_handler(png_structp png, png_const_charp msg)
 {
   fprintf(stderr, "png write error: %s\n", msg);
-  
   longjmp(png_error_jmpbuf, 1);
 }
 
+// -----------------------------------------------------------------------------
+
 /**
 * @brief Saves a png file.
  *
  * @param filename An ANSI encoded file name (can contain path info) to the png file.
- * @return Returns a pointer to a TImage structure containing the image data.
- * @note The return memory must be freed using freeImage().
+ * @param image An image structure containing the actual pixel data.
+ * @param flipped If true then the image will be stored bottom-up otherwise top-down.
+ * @param title The titel for the document. Must be ANSI encoded for now.
+ * @param software A string describing the producer of the document. Must be ANSI encoded for now.
+ * @return Returns true if the image could be saved otherwise false.
  */
-bool savePNG(const wstring& filename, TImage *image, bool flipped)
+bool savePNG(const wstring& filename, TImage *image, bool flipped, const char* title, const char* software)
 {
-  FILE *f;
-  png_structp  png_ptr;
-  png_infop  info_ptr;
+  FILE *file;
+  png_structp  png;
+  png_infop  info;
   png_time  modtime;
-  char *line;
+  png_text comments[3];
   
-  f= fopen(utf16ToANSI(filename).c_str(), "w+");
-  if (!f)
+  file = openFile(filename, L"wb");
+  if (!file)
     return false;
   
-  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL,
-                                    png_error_handler, NULL);
-  if (!png_ptr)
-    goto error;
-  
-  info_ptr = png_create_info_struct(png_ptr);
-  if (!info_ptr) {
-    png_destroy_write_struct(&png_ptr, NULL);
-    goto error;
-  }
-  
-  if (setjmp(png_error_jmpbuf)) {
-    png_destroy_write_struct(&png_ptr, &info_ptr);
-    goto error;
-  }
-  
-  png_init_io(png_ptr, f);
-  
-  //    png_set_compression_level(png_ptr, Z_BEST_COMPRESSION);
-  
-  png_set_IHDR(png_ptr, info_ptr, image->width, image->height,
-               8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE,
-               PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
-  
-  png_convert_from_time_t(&modtime, time(NULL));
-  png_set_tIME(png_ptr, info_ptr, &modtime);
-  
-  png_write_info(png_ptr, info_ptr);
-  
-  png_set_packing(png_ptr);
-
-  line= (char*)malloc(image->width * 3);
-  for (unsigned int i= 0; i < image->height; i++)
+  png = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, png_error_handler, NULL);
+  if (png)
   {
-    char *data= !flipped ? (char*)image->data+i*image->width*3 : (char*)image->data+(image->height-i-1)*image->width*3;
-    char *ptr= line;
-    
-    for (unsigned int xx= 0; xx < image->width; xx++)
-    {
-      *ptr++= *data++;
-      *ptr++= *data++;
-      *ptr++= *data++;
-    }
-    png_write_row(png_ptr, (png_byte*)line);
-  }
-  free(line);
-  
-  png_write_end(png_ptr, NULL);
-  
-  if (png_ptr && info_ptr)
-    png_destroy_write_struct(&png_ptr, &info_ptr);
-  
-  fclose(f);
-  
-  return true;
-  
-error:
-    if (png_ptr && info_ptr)
-      png_destroy_write_struct(&png_ptr, &info_ptr);
-  
-  fclose(f);
-  
+	  info = png_create_info_struct(png);
+	  if (!info)
+			png_destroy_write_struct(&png, NULL);
+		else
+		{
+			if (setjmp(png_error_jmpbuf))
+			{
+			  png_destroy_write_struct(&png, &info);
+			}
+			else
+			{
+				png_set_write_fn(png, file, &pngWrite, &pngFlush);
+				png_init_io(png, file);
+				png_set_IHDR(png, info, image->width, image->height, 8, image->colorType, PNG_INTERLACE_NONE,
+				PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
+				png_convert_from_time_t(&modtime, time(NULL));
+				png_set_tIME(png, info, &modtime);
+				png_write_info(png, info);
+				png_set_packing(png);
+
+				// A few words to the user.
+				comments[0].key = "Software";
+				comments[0].text = (char*) software;
+				comments[0].compression = PNG_TEXT_COMPRESSION_NONE;
+				comments[1].key = "Title";
+				comments[1].text = (char*) title;
+				comments[1].compression = PNG_TEXT_COMPRESSION_NONE;
+				comments[2].key = "Author";
+				comments[2].text = "MySQL GUI tools";
+				comments[2].compression = PNG_TEXT_COMPRESSION_NONE;
+				png_set_text(png, info, comments, 3);
+
+				int bytesPerPixel = 1;
+				if (image->colorType == COLOR_TYPE_RGB)
+				bytesPerPixel = 3;
+				else
+				if (image->colorType == COLOR_TYPE_RGB_ALPHA)
+					bytesPerPixel = 4;
+
+				unsigned char *data = NULL;
+				int offset = 0;
+				if (flipped)
+				{
+				data = image->data + (image->height - 1) * image->width * bytesPerPixel;
+				offset = -image->width * bytesPerPixel;
+				}
+				else
+				{
+				data = image->data;
+				offset = image->width * bytesPerPixel;
+				};
+				for (int i= 0; i < image->height; i++)
+				{
+				png_write_row(png, (png_byte*) data);
+				data += offset;
+				};
+				png_write_end(png, info);
+
+				if (png && info)
+				png_destroy_write_struct(&png, &info);
+
+				fclose(file);
+
+				return true;
+			};
+		};
+	};
+  if (png && info)
+	  png_destroy_write_struct(&png, &info);
+
+  fclose(file);
+
   return false;
 }
 
@@ -790,11 +826,7 @@ TContainerID getContainerID(const string& container)
 {
   TContainerID result = GC_CONTAINER_UNKNOWN;
 
-#ifdef __GNUC__
   static map<string, TContainerID> containers;
-#else
-  static hash_map<string, TContainerID> containers;
-#endif
 
   // Initialize container mapping if not yet done.
   if (containers.size() == 0)
@@ -848,11 +880,7 @@ TContainerID getContainerID(const string& container)
  */
 TPropertyID getPropertyID(const string& property)
 {
-#ifdef __GNUC__
   static map<string, TPropertyID> properties;
-#else
-  static hash_map<string, TPropertyID> properties;
-#endif
   TPropertyID result = GC_PROPERTY_UNKNOWN;
 
   // Fill property mapping first if not yet done.
@@ -963,7 +991,7 @@ string colorToString(GLfloat* color)
     snprintf(buffer, sizeof(buffer), "#%.2X%.2X%.2X", ROUND(color[0] * 255), ROUND(color[1] * 255), ROUND(color[2] * 255));
   #endif // #ifdef _WINDOWS
 
-  return buffer;
+  return string(buffer);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -986,7 +1014,7 @@ string colorToString(GLubyte* color)
     snprintf(buffer, sizeof(buffer), "#%.2X%.2X%.2X", color[0], color[1], color[2]);
   #endif // #ifdef _WINDOWS
 
-  return buffer;
+  return string(buffer);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -1773,12 +1801,12 @@ void CBoundingBoxComputer::include(TMatrix matrix, const TBoundingBox& newBox)
     URZ = newBox.upper.z;
   };
 
-  TVertex lower(LLX, LLY, 0, 1);
-  if (matrix != NULL)
-    lower = matrixTransform(matrix, lower);
-  includeVertex(lower);
+	TVertex lower(LLX, LLY, LLZ, 1);
+	if (matrix != NULL)
+		lower = matrixTransform(matrix, lower);
+	includeVertex(lower);
 
-  TVertex upper(URX, URY, 0, 1);
+  TVertex upper(URX, URY, URZ, 1);
   if (matrix != NULL)
     upper = matrixTransform(matrix, upper);
   includeVertex(upper);
@@ -1835,11 +1863,13 @@ void CBoundingBoxComputer::include(const TVertex& offset, const TBoundingBox& ne
 
   TVertex v;
   v.x = LLX + offset.x;
-  v.y = LLY + offset.y;
+	v.y = LLY + offset.y;
+	v.z = LLZ + offset.z;
   includeVertex(v);
 
   v.x = URX + offset.y;
-  v.y = URY + offset.y;
+	v.y = URY + offset.y;
+	v.y = URZ + offset.z;
   includeVertex(v);
 }
 
