@@ -22,6 +22,10 @@
 
 #include "myx_gc_canvas.h"
 #include "myx_gc_feedback.h"
+#include "gc_glext.h"
+#include "myx_gc_gl_helper.h"
+
+extern const TMatrix identity;
 
 //----------------- CFigureInstanceEnumerator --------------------------------------------------------------------------
 
@@ -120,9 +124,9 @@ void CFigureInstanceEnumerator::reset(void)
 
 //----------------- CLayerListener -----------------------------------------------------------------------------
 
-void CGCView::CLayerListener::onAction(CGCBase* sender, CGCBase* origin, TActionType& actionType)
+void CGCView::CLayerListener::onAction(CGCBase* sender, CGCBase* origin, TAction** action)
 {
-  view->action(origin, actionType);
+  view->action(origin, action);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -161,8 +165,8 @@ CGCView::CGCView(CGenericCanvas* canvas, string name): CGCBase(canvas)
   FOffsetY = 0;
   FWorkspaceWidth = 1000;
   FWorkspaceHeight = 1000;
-  FNearPlane = -100;
-  FFarPlane = 100;
+  FNearPlane = -1000;
+  FFarPlane = 1000;
   FJitter = 0;
   FListener.view = this;
   FStates = 0;
@@ -206,8 +210,7 @@ void CGCView::activate(void)
   glClearColor(FColor[0], FColor[1], FColor[2], FColor[3]);
 
   // TODO: make this dynamically configurable (where applicable).
-  glFrontFace(GL_CW);
-  glDisable(GL_AUTO_NORMAL);
+  glFrontFace(GL_CCW);
   glDisable(GL_CULL_FACE);
   glDisable(GL_DITHER);
   glDisable(GL_DEPTH_TEST);
@@ -226,12 +229,13 @@ void CGCView::activate(void)
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
+  glEnable(GL_AUTO_NORMAL);
   glDisable(GL_FOG);
-  glDisable(GL_LIGHTING);
   glDisable(GL_LOGIC_OP);
   glDisable(GL_STENCIL_TEST);
   glDisable(GL_TEXTURE_1D);
   glDisable(GL_TEXTURE_2D);
+  glDisable(GL_LIGHTING);
   
   // Disable extensions that could slow down glDrawPixels.
   if (canvas()->supportsExtension(GC_OE_CONVOLUTION))
@@ -273,53 +277,19 @@ void CGCView::activate(void)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Sets up the current projection and modelview matrices.
- *
- * @param doProjection A flag telling if also the project matrix should be set. Useful to avoid unnecessary applications.
+ * Sets up the current projection and modelview matrices for normal rendering.
  */
-void CGCView::applyTransformations(bool doProjection)
+void CGCView::applyTransformations(void)
 {
-  if (doProjection)
-  {
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    glOrtho(-FJitter, FViewport.width - FJitter, FViewport.height - FJitter, -FJitter, FNearPlane, FFarPlane);
-  };
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  glOrtho(-FOffsetX / FZoom - FJitter, (FViewport.width - FOffsetX) / FZoom - FJitter, 
+    (FViewport.height - FOffsetY) / FZoom - FJitter, -FOffsetY / FZoom - FJitter, FNearPlane, FFarPlane);
 
   glViewport(FViewport.left, FViewport.top, FViewport.width, FViewport.height);
 
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
-  glTranslated(FOffsetX, FOffsetY, 0);
-  glScaled(FZoom, FZoom, 1);
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-/**
- * Takes care for certain change events that must be considered special.
- */
-void CGCView::handleChange(CGCBase* origin, TGCChangeReason reason)
-{
-  if (!destroying())
-  {
-    switch (reason)
-    {
-      case GC_CHANGE_LAYER_ADD_INSTANCE:
-      case GC_CHANGE_LAYER_CLEAR:
-      case GC_CHANGE_LAYER_REMOVE_INSTANCE:
-      case GC_CHANGE_LAYER_ADD_GROUP:
-      case GC_CHANGE_LAYER_REMOVE_GROUP:
-        {
-          if (FCache != NULL)
-            FCache->invalidate();
-
-          break;
-        };
-      default:
-        change(origin, reason);
-    };
-  };
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -337,7 +307,9 @@ void CGCView::handleChange(CGCBase* origin, TGCChangeReason reason)
 bool CGCView::handleMouseDown(TMouseButton button, int modifiers, int windowX, int windowY, TVertex& viewCoords)
 {
   bool result = false;
-  TActionType lastAction = GC_ACTION_NONE;
+  TAction defaultAction;
+  defaultAction.type = GC_ACTION_NONE;
+  TAction* lastAction = &defaultAction;
 
   switch (button)
   {
@@ -361,19 +333,18 @@ bool CGCView::handleMouseDown(TMouseButton button, int modifiers, int windowX, i
   if (button == GC_MOUSE_BUTTON_LEFT)
   {
     // First check special layers with interactive objects.
-    CFigureInstance* instance;
+    CFigureInstance* instance = NULL;
+    bool keepHitInfo = false;
+
     TFeedbackInfo feedbackInfo = FFeedbackLayer->getFeedbackInfo(viewCoords, FZoom, &instance);
     if (feedbackInfo == GC_FI_FLOATER)
     {
-      lastAction = GC_ACTION_FLOATER_DRAG;
-      action(this, lastAction);
-      if (lastAction == GC_ACTION_FLOATER_DRAG)
+      lastAction->type = GC_ACTION_FLOATER_DRAG;
+      action(this, &lastAction);
+      if (lastAction != NULL && lastAction->type == GC_ACTION_FLOATER_DRAG)
       {
         FStates |= GC_STATE_OVERVIEW_DRAG;
-        FLastWindowX = windowX;
-        FLastWindowY= windowY;
-        FLastViewX = viewCoords.x;
-        FLastViewY = viewCoords.y;
+        keepHitInfo = true;
         result = true;
       };
     };
@@ -391,44 +362,50 @@ bool CGCView::handleMouseDown(TMouseButton button, int modifiers, int windowX, i
       if (hit != NULL && feedbackInfo == GC_FI_NONE)
         feedbackInfo = GC_FI_ON_OBJECT;
 
+      if (feedbackInfo != GC_FI_NONE)
+      {
+        // Since there is a small gap between object and selection decoration
+        // hit can be NULL.
+        if (hit != NULL)
+        {
+          instance = (CFigureInstance*) hit;
+          TAction* action = instance->executeAssociatedActions(viewCoords);
+          if (action != NULL)
+          {
+            lastAction = action;
+            result = true;
+            if (action->type == GC_ACTION_DRAG_FIGURE)
+            {
+              FStates |= GC_STATE_DRAG_PENDING;
+              if (modifiers == 0)
+                FStates |= GC_STATE_CLEAR_PENDING;
+              else
+                if ((modifiers & GC_MODIFIER_ADD) != 0)
+                  FFeedbackLayer->addToSelection(instance);
+                else
+                  if ((modifiers & GC_MODIFIER_TOGGLE) != 0)
+                    if (instance->selected())
+                      FFeedbackLayer->removeFromSelection(instance);
+                    else
+                      FFeedbackLayer->addToSelection(instance);
+
+              // If the hit is not yet selected then drag only this one instance,
+              // otherwise the entire selection will be dragged.
+              FDragSelection = instance->selected();
+              keepHitInfo = true;
+            }
+            else
+              if (action->type == GC_ACTION_RESIZE)
+              {
+                feedbackInfo = action->feedbackInfo;
+                keepHitInfo = true;
+              };
+          };
+        };
+      };
+
       switch (feedbackInfo)
       {
-        case GC_FI_SELECTION_BODY:
-        case GC_FI_ON_OBJECT:
-          {
-            // Since there is a small gap between object and selection decoration
-            // hit can be NULL.
-            if (hit != NULL)
-            {
-              instance = (CFigureInstance*) hit;
-              lastAction = instance->executeAssociatedActions(viewCoords);
-              if (lastAction == GC_ACTION_DRAG_FIGURE)
-              {
-                FStates |= GC_STATE_DRAG_PENDING;
-                if (modifiers == 0)
-                  FStates |= GC_STATE_CLEAR_PENDING;
-                else
-                  if ((modifiers & GC_MODIFIER_ADD) != 0)
-                    FFeedbackLayer->addToSelection(instance);
-                  else
-                    if ((modifiers & GC_MODIFIER_TOGGLE) != 0)
-                      if (instance->selected())
-                        FFeedbackLayer->removeFromSelection(instance);
-                      else
-                        FFeedbackLayer->addToSelection(instance);
-
-                // If the hit is not yet selected then drag only this one instance,
-                // otherwise the entire selection will be dragged.
-                FDragSelection = instance->selected();
-                FLastHit = instance;
-                FLastWindowX = windowX;
-                FLastWindowY= windowY;
-                FLastViewX = viewCoords.x;
-                FLastViewY = viewCoords.y;
-              };
-            };
-            break;
-          };
         case GC_FI_RESIZE_NORTH:
         case GC_FI_RESIZE_NORTH_EAST:
         case GC_FI_RESIZE_EAST:
@@ -438,29 +415,34 @@ bool CGCView::handleMouseDown(TMouseButton button, int modifiers, int windowX, i
         case GC_FI_RESIZE_WEST:
         case GC_FI_RESIZE_NORTH_WEST:
           {
-            lastAction = GC_ACTION_RESIZE;
-            action(this, lastAction);
-            if (lastAction == GC_ACTION_RESIZE)
+            lastAction->type = GC_ACTION_RESIZE;
+            action(this, &lastAction);
+            if (lastAction != NULL && lastAction->type == GC_ACTION_RESIZE)
             {
               FStates |= GC_STATE_RESIZING;
               FResizeInfo = feedbackInfo;
-              FLastHit = instance;
-              FLastWindowX = windowX;
-              FLastWindowY= windowY;
-              FLastViewX = viewCoords.x;
-              FLastViewY = viewCoords.y;
+              keepHitInfo = true;
             };
             break;
           };
       };
     };
 
-    if (!result && lastAction == GC_ACTION_NONE)
+    if (keepHitInfo)
+    {
+      FLastHit = instance;
+      FLastWindowX = windowX;
+      FLastWindowY= windowY;
+      FLastViewX = viewCoords.x;
+      FLastViewY = viewCoords.y;
+    };
+
+    if (!result && lastAction->type == GC_ACTION_NONE)
     {
       // Nothing happend so far so start the rubber rectangle if allowed.
-      lastAction = GC_ACTION_RUBBER_RECT;
-      action(this, lastAction);
-      if (lastAction = GC_ACTION_RUBBER_RECT)
+      lastAction->type = GC_ACTION_RUBBER_RECT;
+      action(this, &lastAction);
+      if (lastAction != NULL && lastAction->type == GC_ACTION_RUBBER_RECT)
       {
         rubberRectStart(GC_RRSTYLE_BLENDED_DIAGONALS, viewCoords, modifiers == 0);
         FStates |= GC_STATE_SELECTION_RECTANGLE;
@@ -518,6 +500,7 @@ bool CGCView::handleMouseMove(int modifiers, int windowX, int windowY, TVertex&
     {
       FStates &= ~(GC_STATE_CLEAR_PENDING | GC_STATE_DRAG_PENDING);
       FStates |= GC_STATE_DRAGGING;
+      change(this, GC_CHANGE_VIEW_DRAGGING_START);
     };
   };
 
@@ -533,14 +516,18 @@ bool CGCView::handleMouseMove(int modifiers, int windowX, int windowY, TVertex&
       FFeedbackLayer->moveSelectedInstances(dX, dY, 0, true);
     else
       FLastHit->translate(dX, dY, 0, true);
-    FCache->invalidate();
+    #if USE_BSP_TREE
+      FCache->invalidate();
+    #endif
     result = true;
   }
   else
     if ((FStates & GC_STATE_RESIZING) != 0)
     {
       FLastHit->resize(dX, dY, FResizeInfo);
-      FCache->invalidate();
+      #if USE_BSP_TREE
+        FCache->invalidate();
+      #endif
       result = true;
     }
     else
@@ -597,8 +584,14 @@ bool CGCView::handleMouseUp(TMouseButton button, int modifiers, int windowX, int
 
   FLastHit = NULL;
 
+  if ((FStates & GC_STATE_DRAGGING) != 0)
+  {
+    FStates &= ~GC_STATE_DRAGGING;
+    change(this, GC_CHANGE_VIEW_DRAGGING_STOP);
+  };
+
   // Remove all states which can be stopped without further action.
-  FStates &= ~(GC_STATE_DRAG_PENDING | GC_STATE_DRAGGING | GC_STATE_RESIZING | GC_STATE_OVERVIEW_DRAG);
+  FStates &= ~(GC_STATE_DRAG_PENDING | GC_STATE_RESIZING | GC_STATE_OVERVIEW_DRAG);
 
   switch (button)
   {
@@ -625,40 +618,251 @@ bool CGCView::handleMouseUp(TMouseButton button, int modifiers, int windowX, int
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * This is the main paint routine. It is called by the canvas if this view is the current view.
+ * Internal render routine used by different other functions.
+ *
+ * @param zoom The zoom value to use for rendering (depends on rendering purpose).
+ * @param renderGrid If true then the grid is rendered.
+ * @param renderConnections If true then the connection layer is rendered.
+ * @param renderFeedback If true then the feedback layer is rendered.
+ * @param visibleBounds The visible area that needs to be rendered. Anything outside can be omitted.
  */
-void CGCView::render(void)
+void CGCView::internalRender(float zoom, bool renderGrid, bool renderConnections, bool renderFeedback, 
+                             const TBoundingBox& visibleBounds)
 {
-  applyTransformations(true);
-
-  // Transform viewport coordinates into view space. They are used for occlusion culling.
-  TBoundingBox visibleBounds;
-  visibleBounds.upper.x = (FViewport.left - FOffsetX) / FZoom - FOrigin[0];
-  visibleBounds.upper.y = (FViewport.top - FOffsetY) / FZoom - FOrigin[1];
-  visibleBounds.lower.x = (FViewport.left + FViewport.width - FOffsetX) / FZoom - FOrigin[0];
-  visibleBounds.lower.y = (FViewport.top + FViewport.height - FOffsetY) / FZoom - FOrigin[1];
-
-  // TODO: complete origin implementation.
-  glTranslated(FOrigin[0], FOrigin[1], FOrigin[2]);
-
-  FGrid->render(FZoom, visibleBounds);
-
-  for (CLayers::iterator iterator = FLayers.begin(); iterator != FLayers.end(); ++iterator)
-    (*iterator)->render(FZoom, visibleBounds);
-
-  FPrimitiveLayer->render(FZoom, visibleBounds);
+  if (renderGrid)
+    FGrid->render(zoom, visibleBounds);
 
   // A simple frame marks the virtual workspace size.
   glColor3ub(0, 0, 0);
   glBegin(GL_LINE_LOOP); 
     glVertex2f(0, 0);
-    glVertex2f(FWorkspaceWidth, 0); 
-    glVertex2f(FWorkspaceWidth, FWorkspaceHeight);
-    glVertex2f(0, FWorkspaceHeight); 
+    glVertex2f(FWorkspaceWidth - 1, 0); 
+    glVertex2f(FWorkspaceWidth - 1, FWorkspaceHeight - 1);
+    glVertex2f(0, FWorkspaceHeight - 1); 
   glEnd();
 
-  FConnectionLayer->render(FZoom, visibleBounds);
-  FFeedbackLayer->render(FZoom, visibleBounds);
+  for (CLayers::iterator iterator = FLayers.begin(); iterator != FLayers.end(); ++iterator)
+    (*iterator)->render(zoom, visibleBounds);
+
+  FPrimitiveLayer->render(zoom, visibleBounds);
+
+  if (renderConnections)
+    FConnectionLayer->render(zoom, visibleBounds);
+
+  if (renderFeedback)
+    FFeedbackLayer->render(zoom, visibleBounds);
+ };
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * This is the main paint routine. It is called by the canvas if this view is the current view.
+ */
+void CGCView::render(void)
+{
+  applyTransformations();
+
+  // Transform viewport coordinates into view space. They are used for occlusion culling.
+  TBoundingBox visibleBounds;
+  visibleBounds.upper.x = -FOffsetX / FZoom;
+  visibleBounds.upper.y = -FOffsetY / FZoom;
+  visibleBounds.lower.x = (FViewport.width - FOffsetX) / FZoom;
+  visibleBounds.lower.y = (FViewport.height - FOffsetY) / FZoom;
+
+  internalRender(FZoom, true, true, true, visibleBounds);
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+/**
+ * Renders the entire content of the view to memory. The function tries to use an OpenGL extension (frame buffer objects)
+ * which may not be available on all graphics boards. If this is the case or the set up of this extension fails then
+ * a fall back is use, rendering the tradional (but slow and inaccurate) way.
+ *
+ * @param format Determines the order of the color components in the result.
+ * @param zoom The zoom level to be used for the output.
+ * @param bounds [in/out] The coordinates of the area that should be rendered (in view space). When traditional rendering
+ *                        is used then it might be necessary to implicitly restrict this area to the current viewport.
+ *                        In this case bounds are used to return actual area to the caller.
+ * @param memory The memory into which the view is to be rendered.
+ */
+void CGCView::renderToMemory(TGCColorFormat format, float zoom, TGCViewport& bounds, unsigned char* memory)
+{
+  GLenum glFormat = GL_RGBA;
+  if (format == GC_COLOR_FORMAT_BGRA)
+    glFormat = GL_BGRA;
+
+  bool canUseFBOs = canvas()->supportsExtension(GC_OE_FRAME_BUFFER_OBJECTS);
+  if (canUseFBOs)
+  {
+    // Make sure we do not allocate more buffer space than supported.
+    GLint maxBufferSize = 0x7FFFFFFF;
+    glGetIntegerv(GL_MAX_RENDERBUFFER_SIZE_EXT, &maxBufferSize);
+
+    GLint maxViewport[2];
+    glGetIntegerv(GL_MAX_VIEWPORT_DIMS, maxViewport);
+
+    // Set maximum possible viewport here (restore old one on exit).
+    int bufferWidth = (maxViewport[0] < maxBufferSize) ? maxViewport[0] : maxBufferSize;
+    int bufferHeight = (maxViewport[1] < maxBufferSize) ? maxViewport[1] : maxBufferSize;
+
+    // Get current draw buffer for later restoration.
+    GLint currentDrawBuffer = 0;
+    glGetIntegerv(GL_DRAW_BUFFER, &currentDrawBuffer);
+
+    // Determine area to render.
+    float left = (float) bounds.left;
+    float top = (float) bounds.top;
+    float width = (float) bounds.width;
+    if (width < 0)
+      width = FWorkspaceWidth;
+    if (width < bufferWidth)
+      bufferWidth = (int) width;
+    float height = (float) bounds.height;
+    if (height < 0)
+      height = FWorkspaceHeight;
+    if (height < bufferHeight)
+      bufferHeight = (int) height;
+
+    // Now that we know the dimensions we can set the viewport.
+    glViewport(0, 0, bufferWidth, bufferHeight);
+
+    GLuint frameBuffer = 0;
+    GLuint renderBuffer = 0;
+
+    // Frame buffer objects allow for hardware accelerated off-screen rendering.
+    glGenFramebuffersEXT(1, &frameBuffer);
+    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, frameBuffer);
+    glGenRenderbuffersEXT(1, &renderBuffer);
+    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, renderBuffer);
+    glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_RENDERBUFFER_EXT, renderBuffer);
+
+    glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_RGBA8, bufferWidth, bufferHeight); 
+    // Check validity of the frame buffer and go back to traditional rendering if something failed.
+    GLenum status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
+    if (status != GL_FRAMEBUFFER_COMPLETE_EXT)
+      canUseFBOs = false;
+    
+    if (canUseFBOs)
+    {
+      glPixelStoref(GL_PACK_ROW_LENGTH, width);
+
+      // Render the entire workspace by splitting it in pieces of the render buffer size.
+      for (float y = 0; y < height; y += bufferHeight)
+      {
+        for (float x = 0; x < width; x += bufferWidth)
+        {
+          int currentWidth = (width - x < bufferWidth) ? int(width - x) : bufferWidth;
+          int currentHeight = (height - y < bufferHeight) ? int(height - y) : bufferHeight;
+          
+          float verticalOffset = height - y - currentHeight;
+          int offset = int(verticalOffset * width + x) * 4; // * 4 because we have 4 components per pixel.
+
+          glMatrixMode(GL_PROJECTION);
+          glLoadIdentity();
+
+          // Set an orthogonal (parallel) projection, mapping the current view area in the entire workspace to the 
+          // viewport coordinates.
+          glOrtho(x + left, x + left + bufferWidth, y + top + bufferHeight, y + top, FNearPlane, FFarPlane);
+
+          glMatrixMode(GL_MODELVIEW);
+          glLoadIdentity();
+          canvas()->clearBuffers();
+
+          // Transform viewport coordinates into view space. They are used for occlusion culling.
+          TBoundingBox visibleBounds;
+          visibleBounds.upper.x = x + left;
+          visibleBounds.upper.y = y + top;
+          visibleBounds.lower.x = x + left + currentWidth;
+          visibleBounds.lower.y = y + top + currentHeight;
+
+          internalRender(zoom, true, true, false, visibleBounds);
+          glReadPixels(0, bufferHeight - currentHeight, currentWidth, currentHeight, glFormat, GL_UNSIGNED_BYTE, memory + offset);
+        };
+      };
+      glPixelStoref(GL_PACK_ROW_LENGTH, 0);
+    };
+
+    // Release frame buffer binding to enable normal rendering again.
+    glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0);
+    glDeleteRenderbuffersEXT(1, &renderBuffer);
+    glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+    glDeleteFramebuffersEXT(1, &frameBuffer);
+
+    glViewport(FViewport.left, FViewport.top, FViewport.width, FViewport.height);
+
+    // Return the actually used bounds to the caller. Particularly width and height are important to
+    // handle following interpretation of the rendered data.
+    bounds.left = (int) left;
+    bounds.top = (int) top;
+    bounds.width = (int) width;
+    bounds.height = (int) height;
+  };
+
+  // Fallback in case either the frame buffer objects extension is not supported or could not properly be set up.
+  if (!canUseFBOs)
+  {
+    // Determine area to render. We can here only render what's currently in the viewport, so limit the area to that.
+    // For validation we need the coords in window space, though.
+    int left;
+    int top;
+    viewToWindow(TVertex(bounds.left, bounds.top, 0), left, top);
+
+    int right;
+    int bottom;
+
+    int width = bounds.width;
+    if (width < 0)
+      width = (int) FWorkspaceWidth;
+    int height = bounds.height;
+    if (height < 0)
+      height = (int) FWorkspaceHeight;
+    viewToWindow(TVertex(bounds.left + width, bounds.top + height, 0), right, bottom);
+    if (left < 0)
+      left = 0;
+    if (top < 0)
+      top = 0;
+    if (right > FViewport.width)
+      right = FViewport.width;
+    if (bottom > FViewport.height)
+      bottom = FViewport.height;
+    width = right - left;
+    height = bottom - top;
+
+    glPixelStorei(GL_PACK_ROW_LENGTH, width);
+    glReadBuffer(GL_BACK);
+
+    glMatrixMode(GL_PROJECTION);
+    glLoadIdentity();
+
+    glOrtho(-FOffsetX / zoom - FJitter, (FViewport.width - FOffsetX) / zoom - FJitter, 
+      (FViewport.height - FOffsetY) / zoom - FJitter, -FOffsetY / zoom - FJitter, FNearPlane, FFarPlane);
+
+    glMatrixMode(GL_MODELVIEW);
+    glLoadIdentity();
+    canvas()->clearBuffers();
+
+    // Transform viewport coordinates into view space. They are used for occlusion culling.
+    TBoundingBox visibleBounds;
+    visibleBounds.upper.x = -FOffsetX / zoom;
+    visibleBounds.upper.y = -FOffsetY / zoom;
+    visibleBounds.lower.x = (FViewport.width - FOffsetX) / zoom;
+    visibleBounds.lower.y = (FViewport.height - FOffsetY) / zoom;
+
+    internalRender(zoom, true, true, false, visibleBounds);
+
+    glReadPixels(left, FViewport.height - top - height, width, height, glFormat, GL_UNSIGNED_BYTE, memory);
+    glPixelStoref(GL_PACK_ROW_LENGTH, 0);
+
+    // Return the actually used bounds to the caller. Particularly width and height are important to
+    // handle following interpretation of the rendered data.
+    bounds.left = left;
+    bounds.top = top;
+    bounds.width = width;
+    bounds.height = height;
+  };
+
+  canvas()->checkError();
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -798,7 +1002,7 @@ void CGCView::color(float red, float green, float blue, float alpha)
  *
  * @param newColor The new color to use.
  */
-void CGCView::color(GLfloat* newColor)
+void CGCView::colorV(GLfloat* newColor)
 {
   FColor[0] = newColor[0];
   FColor[1] = newColor[1];
@@ -1051,6 +1255,34 @@ void CGCView::getWorkspace(float* width, float* height)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Takes care for certain change events that must be considered special.
+ */
+void CGCView::handleChange(CGCBase* origin, TGCChangeReason reason)
+{
+  if (!destroying())
+  {
+    switch (reason)
+    {
+      case GC_CHANGE_LAYER_ADD_INSTANCE:
+      case GC_CHANGE_LAYER_CLEAR:
+      case GC_CHANGE_LAYER_REMOVE_INSTANCE:
+      case GC_CHANGE_LAYER_ADD_GROUP:
+      case GC_CHANGE_LAYER_REMOVE_GROUP:
+        {
+          if (FCache != NULL)
+            FCache->invalidate();
+
+          break;
+        };
+      default:
+        change(origin, reason);
+    };
+  };
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Called by the viewer to let the current view handle user input with the mouse.
  *
@@ -1094,27 +1326,12 @@ bool CGCView::handleMouseInput(TMouseEvent event, TMouseButton button, int modif
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Jittering the viewport a little bit sometimes improves display quality (e.g. for thin lines). This function
- * sets this value for this view.
- *
- * @param value The new jitter value.
- */
-void CGCView::jitter(float value)
-{
-  FJitter = value;
-  change(this, GC_CHANGE_VIEW_PROPERTY);
-  canvas()->refresh();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
 /**
  * Returns the current jitter value.
  *
  * @return The current jitter value.
  */
-float CGCView::jitter(void)
+float CGCView::jitterGet(void)
 {
   return FJitter;
 }
@@ -1122,13 +1339,15 @@ float CGCView::jitter(void)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Sets the horizontal offset of the view. The offset is value about which the content of the view is moved.
+ * Jittering the viewport a little bit sometimes improves display quality (e.g. for thin lines). This function
+ * sets this value for this view.
  *
- * @param value The new horizontal offset.
+ * @param value The new jitter value.
  */
-void CGCView::offsetX(float value)
+void CGCView::jitterSet(float value)
 {
-  FOffsetX = value;
+  FJitter = value;
+  change(this, GC_CHANGE_VIEW_PROPERTY);
   canvas()->refresh();
 }
 
@@ -1139,7 +1358,7 @@ void CGCView::offsetX(float value)
  *
  * @return The current horizontal offset.
  */
-float CGCView::offsetX(void)
+float CGCView::offsetXGet(void)
 {
   return FOffsetX;
 }
@@ -1147,13 +1366,13 @@ float CGCView::offsetX(void)
 //----------------------------------------------------------------------------------------------------------------------
 
 /**
- * Sets the new vertical offset.
+ * Sets the horizontal offset of the view. The offset is value about which the content of the view is moved.
  *
- * @param value The new vertical offset.
+ * @param value The new horizontal offset.
  */
-void CGCView::offsetY(float value)
+void CGCView::offsetXSet(float value)
 {
-  FOffsetY = value;
+  FOffsetX = value;
   canvas()->refresh();
 }
 
@@ -1164,13 +1383,26 @@ void CGCView::offsetY(float value)
  *
  * @return The curretn vertical offset.
  */
-float CGCView::offsetY(void)
+float CGCView::offsetYGet(void)
 {
   return FOffsetY;
 }
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Sets the new vertical offset.
+ *
+ * @param value The new vertical offset.
+ */
+void CGCView::offsetYSet(float value)
+{
+  FOffsetY = value;
+  canvas()->refresh();
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Tells the caller if this view is currently in overview mode.
  *
@@ -1259,7 +1491,7 @@ void CGCView::overviewStop(bool returnToZoomed)
  * @param index If the property is a list then this parameter gives the index into that list.
  * @return A description of the property value and, if the property is simple, the actual value.
  */
-TGCVariant CGCView::property(const char* name, unsigned int index)
+TGCVariant CGCView::propertyGet(const char* name, unsigned int index)
 {
   TGCVariant result;
 
@@ -1298,7 +1530,7 @@ TGCVariant CGCView::property(const char* name, unsigned int index)
             {
               result = FOffsetX;
               break;
-            };
+			};
           case GC_PROPERTY_Y:
             {
               result = FOffsetY;
@@ -1332,12 +1564,12 @@ TGCVariant CGCView::property(const char* name, unsigned int index)
  * @param index If the property is a list then this parameter gives the index into that list.
  * @param value The new value of the property. Automatic conversion is performed where possible.
  */
-void CGCView::property(const char* name, unsigned int index, TGCVariant value)
+void CGCView::propertySet(const char* name, unsigned int index, TGCVariant value)
 {
   switch (getPropertyID(name))
   {
-    case GC_PROPERTY_COLOR:
-      {
+	case GC_PROPERTY_COLOR:
+	  {
         stringToColor(value, FColor);
         FColor[3] = 1;
         change(this, GC_CHANGE_VIEW_PROPERTY);
@@ -1431,6 +1663,7 @@ void CGCView::rubberBandStart(TRubberBandStyle style, const TVertex& coords)
   // Implizitely stops any active rubber band.
   FFeedbackLayer->rubberBandStart(style, coords);
   FStates |= GC_STATE_RUBBER_BAND;
+  change(this, GC_CHANGE_VIEW_RUBBERBAND_START);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -1442,6 +1675,7 @@ void CGCView::rubberBandStop(void)
 {
   FStates &= ~GC_STATE_RUBBER_BAND;
   FFeedbackLayer->rubberBandStop();
+  change(this, GC_CHANGE_VIEW_RUBBERBAND_STOP);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -1455,9 +1689,10 @@ void CGCView::rubberBandStop(void)
  */
 void CGCView::rubberRectStart(TRubberRectStyle style, const TVertex& coords, bool removeSelection)
 {
-  // Implizitely stops any active rubber rectangle.
+  // Implicitely stops any active rubber rectangle.
   FFeedbackLayer->rubberRectStart(style, coords, removeSelection);
   FStates |= GC_STATE_RUBBER_RECTANGLE;
+  change(this, GC_CHANGE_VIEW_RUBBERRECT_START);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -1469,34 +1704,7 @@ void CGCView::rubberRectStop(void)
 {
   FStates &= ~GC_STATE_RUBBER_RECTANGLE;
   FFeedbackLayer->rubberRectStop();
-}
-
-//----------------------------------------------------------------------------------------------------------------------
-
-bool CGCView::saveAsPNG(const TGCViewport& viewport, const char *filename)
-{
-  unsigned char *pixels;
-  
-  pixels= (unsigned char*)malloc(viewport.width * viewport.height * 3);
-  if (pixels)
-  {
-    TImage image;
-    bool rc;
-    
-    glReadPixels(viewport.left, viewport.top, viewport.width, viewport.height,
-                 GL_RGB, GL_UNSIGNED_BYTE, pixels);
-    
-    image.width= viewport.width;
-    image.height= viewport.height;
-    image.data= pixels;
-    
-    rc= savePNG(utf8ToUtf16(filename), &image, true);
-    
-    free(pixels);
-
-    return rc;
-  }
-  return false;
+  change(this, GC_CHANGE_VIEW_RUBBERRECT_STOP);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -1548,7 +1756,7 @@ void CGCView::setWorkspace(const float width, const float height)
  */
 void CGCView::showSelection(bool visible)
 {
-  FFeedbackLayer->visible(visible);
+  FFeedbackLayer->visibleSet(visible);
 }
 
 //----------------------------------------------------------------------------------------------------------------------
@@ -1558,7 +1766,7 @@ void CGCView::showSelection(bool visible)
  *
  * @param NewViewport The new viewport to be used.
  */
-void CGCView::viewport(const TGCViewport& newViewport)
+void CGCView::viewportSet(const TGCViewport& newViewport)
 {
   FViewport = newViewport;
   glViewport(FViewport.left, FViewport.top, FViewport.width, FViewport.height);
@@ -1568,6 +1776,36 @@ void CGCView::viewport(const TGCViewport& newViewport)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Converts the given view coordinates into window coordinates
+ * @note This function resets the current projection and view matrices.
+ *
+ * @param coords A vertex containing view coordiantes.
+ * @param x [out] Horizontal window coordinate in pixels.
+ * @param y [out] Vertical window coordinate in pixels.
+ */
+void CGCView::viewToWindow(const TVertex& coords, int& x, int& y)
+{
+  applyTransformations();
+
+  GLint viewport[4];
+  glGetIntegerv(GL_VIEWPORT, viewport);
+
+  GLdouble modelviewMatrix[16];
+  glGetDoublev(GL_MODELVIEW_MATRIX, modelviewMatrix);
+
+  GLdouble projectionMatrix[16];
+  glGetDoublev(GL_PROJECTION_MATRIX, projectionMatrix);
+
+  double winX, winY, winZ;
+  gluProject(coords.x, coords.y, coords.z, modelviewMatrix, projectionMatrix, viewport, &winX, &winY, &winZ);
+
+  x = int(winX);
+  y = int(viewport[3] - winY); 
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Converts the given window (viewer) coordinates into local (view) space.
  * @note This function resets the current projection and view matrices.
@@ -1578,7 +1816,7 @@ void CGCView::viewport(const TGCViewport& newViewport)
  */
 void CGCView::windowToView(int x, int y, TVertex& coords)
 {
-  applyTransformations(true);
+  applyTransformations();
 
   GLint viewport[4];
   glGetIntegerv(GL_VIEWPORT, viewport);
@@ -1597,12 +1835,24 @@ void CGCView::windowToView(int x, int y, TVertex& coords)
 
 //----------------------------------------------------------------------------------------------------------------------
 
+/**
+ * Returns the current zoom factor.
+ *
+ * @return The current zoom factor.
+ */
+float CGCView::zoomGet(void)
+{
+  return FZoom;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * Sets a new zoom factor.
  *
  * @param value The new zoom factor.
  */
-void CGCView::zoom(float value)
+void CGCView::zoomSet(float value)
 {
   FZoom = value;
   change(this, GC_CHANGE_VIEW_PROPERTY);
@@ -1611,15 +1861,4 @@ void CGCView::zoom(float value)
 
 //----------------------------------------------------------------------------------------------------------------------
 
-/**
- * Returns the current zoom factor.
- *
- * @return The current zoom factor.
- */
-float CGCView::zoom(void)
-{
-  return FZoom;
-}
-
-//----------------------------------------------------------------------------------------------------------------------
 
