@@ -1,23 +1,25 @@
-/* Copyright (C) 2004 MySQL AB
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
+/* Copyright (c) 2004, 2005 MySQL AB
+  
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+  
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+  
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the
+   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+ */
 
 #ifndef __MYX_GRT_H__
 #define __MYX_GRT_H__
 
-#define GRT_VERSION "2.0.11 beta"
+#define GRT_VERSION "2.0.12"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 # define __LCC__
@@ -26,10 +28,7 @@
 # define MYX_SOCKET int
 #endif
 
-#include <myx_public_interface.h>
-#ifdef ENABLE_LUA_MODULES
-#include <lua.h>
-#endif
+#include <myx_util_public_interface.h>
 #ifdef ENABLE_JAVA_MODULES
 # ifdef __APPLE__
 #  include <JavaVM/jni.h>
@@ -62,7 +61,8 @@ extern "C" {
 /// [SCRIPT::LibInterfaceMapper] -public_interface "libmysqlgrt"
 #define libmysqlgrt_PUBLIC_INTERFACE_VERSION 10000
 
-/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_to_uses "myx_util_public_interface, myx_public_interface"
+/// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library_util\include\myx_util_public_interface.h"
 /// [SCRIPT::LibInterfaceMapper] -add_datatypes_from "..\..\..\mysql-gui-common\library\include\myx_public_interface.h"
 
 
@@ -96,7 +96,11 @@ typedef enum
 
   MYX_GRT_JAVA_NOT_FOUND,
   MYX_GRT_JAVA_REGISTRY_CORRUPTED,
-  MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED
+  MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED,
+
+  MYX_GRT_SHELL_UNSUPPORTED,
+  MYX_GRT_SHELL_ALREADY_LOADED,
+  MYX_GRT_SHELL_INIT_ERROR
 
 } MYX_GRT_ERROR;
 
@@ -160,13 +164,19 @@ typedef enum
 typedef struct MYX_GRT_MODULE_LOADER MYX_GRT_MODULE_LOADER;
 typedef struct MYX_GRT MYX_GRT;
 
+typedef enum MYX_GRT_SHELL_INTERFACE
+{
+  MYX_GRT_SHELL_LUA,
+  MYX_GRT_SHELL_PYTHON
+} MYX_GRT_SHELL_INTERFACE;
+  
 typedef enum MYX_GRT_SHELL_COMMAND
 {
   MYX_GRT_SHELL_COMMAND_UNKNOWN = -1,
   MYX_GRT_SHELL_COMMAND_EXIT = 0,
   MYX_GRT_SHELL_COMMAND_ALL,
   MYX_GRT_SHELL_COMMAND_ERROR,
-  MYX_GRT_SHELL_COMMAND_LUA,
+  MYX_GRT_SHELL_COMMAND_STATEMENT, // was _LUA
   MYX_GRT_SHELL_COMMAND_HELP,
   MYX_GRT_SHELL_COMMAND_LS,
   MYX_GRT_SHELL_COMMAND_CD,
@@ -276,6 +286,18 @@ typedef struct
   MYX_GRT_DICT_ITEM *items; // this must be in strcmp() order of name
 } MYX_GRT_DICT;
 
+typedef struct MYX_GRT_VALUE_EXTENDED
+{
+  MYX_GRT_MODULE *bridge_module; // if set, callbacks to the bridge are made if the value is changed
+  void *bridge_data_object; // data set by the bridge so it is possible to associate bridged objects with GRT values
+  void *bridge_data_owner; // data set by the bridge so it is possible to associate bridged objects with GRT values
+  char *bridge_dict_key; // if this is a dict item, store the key name so the bridge can identify the property associated with the value
+  unsigned int bridge_list_index; // if this is a list item, store the index so the bridge can identify the item associated with the value
+
+  unsigned int listeners_num;
+  struct MYX_GRT_VALUE_LISTENER **listeners;
+} MYX_GRT_VALUE_EXTENDED;
+
 typedef struct MYX_GRT_VALUE 
 {
   MYX_GRT_VALUE_TYPE type;
@@ -288,15 +310,7 @@ typedef struct MYX_GRT_VALUE
     MYX_GRT_DICT *d;
   } value;
 
-  MYX_GRT_MODULE *bridge_module; // if set, callbacks to the bridge are made if the value is changed
-  void *bridge_data_object; // data set by the bridge so it is possible to associate bridged objects with GRT values
-  void *bridge_data_owner; // data set by the bridge so it is possible to associate bridged objects with GRT values
-  char *bridge_dict_key; // if this is a dict item, store the key name so the bridge can identify the property associated with the value
-  unsigned int bridge_list_index; // if this is a list item, store the index so the bridge can identify the item associated with the value
-
-  unsigned int listeners_num;
-  struct MYX_GRT_VALUE_LISTENER **listeners;
-
+  MYX_GRT_VALUE_EXTENDED *extended;
 } MYX_GRT_VALUE;  
 
 typedef struct MYX_GRT_VALUE_LISTENER
@@ -345,7 +359,8 @@ typedef enum
     MYX_PYTHON_MODULE_TYPE,
     MYX_PHP_MODULE_TYPE,
     MYX_DELPHI_MODULE_TYPE,
-    MYX_OBJC_MODULE_TYPE
+    MYX_OBJC_MODULE_TYPE,
+    MYX_CPP_MODULE_TYPE
 //    MYX_PERL_MODULE_TYPE,
 } MYX_GRT_MODULE_TYPE;
 
@@ -369,7 +384,7 @@ typedef int (*MYX_GRT_STATUS_QUERY_CALLBACK)(void *user_data);
 typedef void (*MYX_GRT_LOG_CALLBACK)(MYX_GRT *, int, const char *, const char*);
 typedef int (*MYX_GRT_VALUE_LISTENER_CALLBACK)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data);
 
-MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
+MYX_PUBLIC_FUNC int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist);
 
 MYX_PUBLIC_FUNC MYX_GRT * myx_grt_initialize(int options);
 MYX_PUBLIC_FUNC void myx_grt_finalize(MYX_GRT *grt);
@@ -407,6 +422,7 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_get_root(MYX_GRT *grt);
 MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root);
 
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id);
+MYX_PUBLIC_FUNC void myx_grt_reference_cache_add(MYX_GRT *grt, MYX_GRT_VALUE *obj);
 MYX_PUBLIC_FUNC void myx_grt_reference_cache_clear(MYX_GRT *grt);
 MYX_PUBLIC_FUNC void myx_grt_reference_cache_rescan(MYX_GRT *grt);
 
@@ -417,7 +433,7 @@ MYX_PUBLIC_FUNC void myx_grt_messages_free(MYX_GRT_MSGS *msgs);
 
 MYX_PUBLIC_FUNC void myx_grt_messages_stack_add(MYX_GRT *grt, int msg_type, const char *message,
                                           MYX_STRINGLIST *details, int copy_details, int progress);
-MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_message(MYX_GRT *grt, const char *message, 
+MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_message(MYX_GRT *grt, const char *message,
                                           MYX_STRINGLIST *details, int copy_details, ...);
 MYX_PUBLIC_FUNC void myx_grt_messages_stack_add_error(MYX_GRT *grt, const char *message, 
                                           MYX_STRINGLIST *details, int copy_details, ...);
@@ -593,7 +609,7 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_retain(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_release(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_get_current_reference_count(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_dup(MYX_GRT_VALUE *value);
-  
+
 MYX_PUBLIC_FUNC void myx_grt_value_listener_add(MYX_GRT *grt, MYX_GRT_VALUE *value, void *user_data, 
                                                 int (*listener_callback)(MYX_GRT *grt, MYX_GRT_VALUE *value, MYX_GRT_VALUE_CALLBACK_REASON reason, void *user_data));
 MYX_PUBLIC_FUNC void myx_grt_value_listener_remove(MYX_GRT_VALUE *value, void *user_data, 
@@ -619,7 +635,9 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_grt_value_get_type(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC const char * myx_get_value_type_as_string(MYX_GRT_VALUE_TYPE value_type);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE_TYPE myx_get_value_type_from_string(const char *value_type_name, MYX_GRT_ERROR *error);
 MYX_PUBLIC_FUNC int myx_grt_value_is_simple_type(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_value_extend(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC int myx_grt_value_is_bridged(MYX_GRT_VALUE *value);
+MYX_PUBLIC_FUNC int myx_grt_value_has_listeners(MYX_GRT_VALUE *value);
 
 MYX_PUBLIC_FUNC int myx_grt_value_as_int(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC double myx_grt_value_as_real(MYX_GRT_VALUE *value);
@@ -810,28 +828,31 @@ MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_php_init_loader(MYX_GRT *grt, MYX_GR
 
 #ifdef ENABLE_LUA_MODULES
 MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, const char *lua_module_path);
-
-// Lua Shell
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_init_lua_shell(MYX_GRT *grt);
-MYX_PUBLIC_FUNC char * myx_grt_lua_shell_get_prompt(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_GRT_SHELL_COMMAND myx_grt_lua_shell_execute(MYX_GRT *grt, const char *linebuf);
-MYX_PUBLIC_FUNC int myx_grt_lua_shell_run_file(MYX_GRT *grt, const char *file_name, int interactive);
-MYX_PUBLIC_FUNC void myx_grt_shell_print_welcome(MYX_GRT *grt);
-MYX_PUBLIC_FUNC lua_State * myx_grt_lua_shell_get_lua(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_lua_shell_get_global_var(MYX_GRT *grt, const char *var_name);
-MYX_PUBLIC_FUNC void myx_grt_lua_shell_set_global_var(MYX_GRT *grt, const char *var_name, MYX_GRT_VALUE *value);
-
 #endif
 
-// Module messaging (with sockets)
-MYX_PUBLIC_FUNC int myx_grt_setup_module_messaging(MYX_GRT *grt); 
-MYX_PUBLIC_FUNC int myx_grt_cleanup_module_messaging(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_SOCKET myx_grt_get_messaging_fd(MYX_GRT *grt);
-MYX_PUBLIC_FUNC MYX_SOCKET myx_grt_get_messaging_module_fd(MYX_GRT *grt);
-MYX_PUBLIC_FUNC int myx_grt_check_module_connected(MYX_GRT *grt);
-MYX_PUBLIC_FUNC int myx_grt_check_module_message(MYX_GRT *grt, char **message, int timeout);
-MYX_PUBLIC_FUNC int myx_grt_send_module_message(MYX_GRT *grt, const char *message);
-  
+// ----------------------------------------------------------------------------------------------------------------------
+// Python module
+
+#ifdef ENABLE_PYTHON_MODULES
+MYX_PUBLIC_FUNC MYX_GRT_MODULE_LOADER * myx_python_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, const char *python_module_path);
+#endif
+
+// ----------------------------------------------------------------------------------------------------------------------
+// Shell Support  
+
+MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_grt_setup_shell(MYX_GRT *grt, MYX_GRT_SHELL_INTERFACE flavour);
+
+MYX_PUBLIC_FUNC int myx_grt_shell_init(MYX_GRT *grt);
+MYX_PUBLIC_FUNC char * myx_grt_shell_get_prompt(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_SHELL_COMMAND myx_grt_shell_execute(MYX_GRT *grt, const char *linebuf);
+MYX_PUBLIC_FUNC int myx_grt_shell_run_file(MYX_GRT *grt, const char *file_name, int interactive);
+MYX_PUBLIC_FUNC void myx_grt_shell_print_welcome(MYX_GRT *grt);
+MYX_PUBLIC_FUNC void * myx_grt_shell_get_interpreter_data(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_VALUE * myx_grt_shell_get_global_var(MYX_GRT *grt, const char *var_name);
+MYX_PUBLIC_FUNC int myx_grt_shell_set_global_var(MYX_GRT *grt, const char *var_name, MYX_GRT_VALUE *value);
+
+
+
 // Aux funcs
 MYX_PUBLIC_FUNC int _myx_grt_get_refcount(MYX_GRT_VALUE *value);
 MYX_PUBLIC_FUNC MYX_GRT_VALUE *make_return_value(MYX_GRT_VALUE *value);
@@ -839,6 +860,11 @@ MYX_PUBLIC_FUNC MYX_GRT_VALUE *make_return_value_error(const char *message, cons
 
 MYX_PUBLIC_FUNC char * myx_grt_get_guid();
 
+#ifdef ENABLE_LUA_MODULES
+#include <lua.h>
+// Text User Interface stuff
+MYX_PUBLIC_FUNC void myx_grt_init_textui(lua_State *l);
+#endif
 
 
 #ifdef __cplusplus
