@@ -1,18 +1,20 @@
-/* Copyright (C) 2004 MySQL AB
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
+/* Copyright (c) 2004, 2005 MySQL AB
+  
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+  
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+  
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the
+   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+ */
 
 /**
  * @file myx_grt.c 
@@ -26,8 +28,9 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 
-#include "myx_xml_aux_functions.h"
+#include "myx_xml_util_functions.h"
 #include "myx_grt_private.h"
+#include "myx_shared_util_functions.h"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 #include <objbase.h>
@@ -43,6 +46,9 @@ static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashT
 static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id);
 static xmlNodePtr serialize_to_xml(MYX_GRT *grt, xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
 
+static int refcache_callback(MYX_GRT* grt, MYX_GRT_VALUE* value, MYX_GRT_VALUE_CALLBACK_REASON reason, void* data);
+static void refcache_callback2(gpointer key, gpointer value, gpointer data);
+
 
 static void default_print(const char *msg, void *data)
 {
@@ -104,7 +110,10 @@ MYX_GRT *myx_grt_initialize(int options)
   grt->struct_icon_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, g_free);
 
-  grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+  grt->global_reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, NULL);
+
+  grt->direct_reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, NULL);
   
   myx_grt_register_module_loader(grt, myx_builtin_init_loader(grt));
@@ -131,6 +140,14 @@ MYX_GRT *myx_grt_initialize(int options)
 void myx_grt_finalize(MYX_GRT *grt)
 {
   //XXX TODO
+
+  // Remove all value listener registrations.
+  g_hash_table_foreach(grt->global_reference_cache, refcache_callback2, NULL); 
+  g_hash_table_destroy(grt->global_reference_cache);
+
+  // Remove all value listener registrations.
+  g_hash_table_foreach(grt->direct_reference_cache, refcache_callback2, NULL); 
+  g_hash_table_destroy(grt->direct_reference_cache);
 }
 
 
@@ -1554,6 +1571,22 @@ void myx_grt_messages_stack_flush(MYX_GRT *grt, unsigned int count)
 }
 
 
+/**
+ ****************************************************************************
+ * @brief Adds an object to the direct reference cache
+ *
+ * @param grt MYX_GRT pointer
+ * @param obj The object to add
+ ****************************************************************************
+ */
+void myx_grt_reference_cache_add(MYX_GRT *grt, MYX_GRT_VALUE *obj)
+{
+  // TODO Keep manually added values in a separate list, so they are re-added when the cache is rebuilt.
+  g_hash_table_insert(grt->direct_reference_cache, g_strdup(myx_grt_dict_id_item_as_string(obj)), obj);
+  myx_grt_value_listener_add(grt, obj, NULL, refcache_callback);
+}
+
+
 /**
  ****************************************************************************
  * @brief Called by the GRT when certain actions happen to a value, e.g. it is being deleted.
@@ -1570,7 +1603,9 @@ int refcache_callback(MYX_GRT* grt, MYX_GRT_VALUE* value, MYX_GRT_VALUE_CALLBACK
   if (reason == MYX_GVCR_DELETE)
   {
     const char *_id= myx_grt_dict_item_get_as_string(value, "_id");
-    g_hash_table_remove(grt->reference_cache, _id);
+    g_hash_table_remove(grt->global_reference_cache, _id);
+
+    g_hash_table_remove(grt->direct_reference_cache, _id);
   };
 
   return 0;
@@ -1587,7 +1622,7 @@ int refcache_callback(MYX_GRT* grt, MYX_GRT_VALUE* value, MYX_GRT_VALUE_CALLBACK
  */
 void refcache_callback2(gpointer key, gpointer value, gpointer data)
 {
-  MYX_GRT_VALUE* grtValue = value;
+//  MYX_GRT_VALUE* grtValue = value;
   myx_grt_value_listener_remove(value, data, refcache_callback);
 }
 
@@ -1602,19 +1637,18 @@ void myx_grt_reference_cache_clear(MYX_GRT *grt)
   GRT_ENTER(grt);
   
   // Remove all value listener registrations.
-  g_hash_table_foreach(grt->reference_cache, refcache_callback2, NULL); 
+  g_hash_table_foreach(grt->global_reference_cache, refcache_callback2, NULL); 
 
   // free current cache
-  g_hash_table_destroy(grt->reference_cache);
+  g_hash_table_destroy(grt->global_reference_cache);
 
   // reallocate cache
-  grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
+  grt->global_reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, NULL);
   
   GRT_LEAVE(grt);
 }
 
-
 static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *object_path)
 {
   unsigned int i;
@@ -1634,7 +1668,7 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
     {
       dict->value.d->grt= grt;
       dict->value.d->object_path= g_strdup(object_path);
-      g_hash_table_insert(grt->reference_cache, g_strdup(_id), dict);
+      g_hash_table_insert(grt->global_reference_cache, g_strdup(_id), dict);
       myx_grt_value_listener_add(grt, dict, NULL, refcache_callback);
     }
   }
@@ -1701,20 +1735,31 @@ MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id)
 {
   MYX_GRT_VALUE *retval= NULL;
   
+  if (!ref_id || !ref_id[0])
+    return NULL;
+
   GRT_ENTER(grt);
   if(ref_id)
   {
     // try to find the value in the cache
-    MYX_GRT_VALUE *value= g_hash_table_lookup(grt->reference_cache, ref_id);
+    MYX_GRT_VALUE *value= g_hash_table_lookup(grt->global_reference_cache, ref_id);
 
     if (value)
       retval= value;
     else
     {
-      // if it is not found, rescan and search again
-      myx_grt_reference_cache_rescan(grt);
+      // try direct cache
+      value= g_hash_table_lookup(grt->direct_reference_cache, ref_id);
 
-      retval= g_hash_table_lookup(grt->reference_cache, ref_id);
+      if (value)
+        retval= value;
+      else
+      {
+        // if it is not found, rescan and search again
+        myx_grt_reference_cache_rescan(grt);
+
+        retval= g_hash_table_lookup(grt->global_reference_cache, ref_id);
+      }
     }
   }
   else
@@ -1726,6 +1771,8 @@ MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id)
 
 char * myx_grt_get_guid()
 {
+  /* GUIDs must be no more than 50 chars */
+  
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   GUID guid;
   WCHAR guid_wstr[50];
@@ -1807,7 +1854,22 @@ const char *myx_grt_error_string(MYX_GRT_ERROR error)
     return "Java registry corrupted";
   case MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED:
     return "Java JRE cannot be loaded";
+
+  case MYX_GRT_SHELL_UNSUPPORTED:
+    return "Shell type is not supported";
+  case MYX_GRT_SHELL_ALREADY_LOADED:
+    return "Shell was already initialized";
+  case MYX_GRT_SHELL_INIT_ERROR:
+    return "Error initializing shell";
+
   default:
     return "Unknown error";
   }
 }
+
+#if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
+int myx_free_lib_stringlist(MYX_STRINGLIST *stringlist)
+{
+  return myx_free_stringlist(stringlist);
+}
+#endif
