@@ -18,7 +18,7 @@
 #ifdef ENABLE_JAVA_MODULES
 
 #include "myx_grt_java.h"
-#include "myx_shared_aux_functions.h"
+#include "myx_shared_util_functions.h"
 
 #ifdef __GNUC__
 #define JAVA_PATH_SEPARATOR ":"
@@ -68,7 +68,7 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
   MYX_GRT_MODULE_LOADER *loader= g_new0(MYX_GRT_MODULE_LOADER, 1);
   MYX_JAVA_LOADER *priv= g_new0(MYX_JAVA_LOADER, 1);
   JavaVMInitArgs vm_init_args;
-  JavaVMOption jvm_options[7];
+  JavaVMOption jvm_options[6];
   jclass grt_class;
   static char *file_extensions[]= {
     ".class"
@@ -113,7 +113,7 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
 #endif
 
   if (grt->options & MYX_GRT_REMOTE_DEBUG)
-    vm_init_args.nOptions = 6;
+    vm_init_args.nOptions = 5;
   else
     vm_init_args.nOptions = 2;
 
@@ -187,9 +187,10 @@ MYX_GRT_MODULE_LOADER *myx_java_init_loader(MYX_GRT *grt, const char *class_path
     jvm_options[2].optionString= strdup("-Xrunjdwp:transport=dt_socket,server=y,address=8000,suspend=n");
     jvm_options[3].optionString= "-Xnoagent";
     jvm_options[4].optionString= "-Djava.compiler=NONE";
-    jvm_options[5].optionString= "-Xss200m";
+    jvm_options[5].optionString= "-Xmx256m";
 
-    jvm_options[6].optionString= "-Xmx256m";
+    // do not modify the stack size
+    //jvm_options[5].optionString= "-Xss64m";
   }
   else
     jvm_options[1].optionString= "-Xmx256m";
@@ -1891,139 +1892,175 @@ JNIEXPORT jint JNICALL Java_com_mysql_grt_GrtCallbackNative_processStatusQuery
   return res;
 }
 
+JNIEXPORT void JNICALL Java_com_mysql_grt_GrtCallbackNative_setBridgeDataObject
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring objectPath, jobject dataObj)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
+  MYX_GRT_VALUE *value;
+  
+  GRT_ENTER(grt);
+  
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
+
+  if (dataObj == NULL)
+  {
+    dataObj= myx_grt_value_bridge_data_object_get(value);
+    (*env)->DeleteGlobalRef(env, dataObj);
+    dataObj= NULL;
+  }
+  else
+    dataObj= (*env)->NewGlobalRef(env, dataObj);
+
+  myx_grt_value_bridge_data_object_set(value, dataObj);
+
+  if (c_str_objectPath)
+    (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
+
+  GRT_LEAVE(grt);
+}
+
+JNIEXPORT jobject JNICALL Java_com_mysql_grt_GrtCallbackNative_getBridgeDataObject
+  (JNIEnv *env, jobject obj, jlong grtMyxGrtPointer, jstring objectPath)
+{
+  MYX_GRT *grt= LongToPtr(grtMyxGrtPointer);
+  const char *c_str_objectPath= (*env)->GetStringUTFChars(env, objectPath, 0);
+  MYX_GRT_VALUE *value;
+  jobject dataObj;
+  
+  GRT_ENTER(grt);
+  
+  value= myx_grt_dict_item_get_by_path(grt, myx_grt_get_root(grt), c_str_objectPath);
+
+  dataObj= myx_grt_value_bridge_data_object_get(value);
+
+  if (c_str_objectPath)
+    (*env)->ReleaseStringUTFChars(env, objectPath, c_str_objectPath);
+
+  GRT_RETURN(grt, dataObj, jobject);
+}
+
 // -------------------------------------------------------------------------------------------------
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 
-static int registry_get_value_as_string(HKEY key, const char *name, char *buffer, int buffer_size)
-{
-    DWORD reg_value_type;
-    DWORD reg_value_size;
-
-    if (((RegQueryValueEx(key, name, 0, &reg_value_type, 0, &reg_value_size) == 0) && 
-        (reg_value_type == REG_SZ) && (reg_value_size < (unsigned int)buffer_size))) 
-    {
-	    if (RegQueryValueEx(key, name, 0, 0, buffer, &reg_value_size) == 0) 
-	      return 1;
-    }
-
-    return 0;
-}
-
 #define MAX_KEY_LENGTH 255
 #define MAX_VALUE_NAME 16383
 
 static int get_java_runtimelib_filename(const char *version, char **runtimelib_filename)
 {
   HKEY java_key;
+  HKEY java_subkey;
   TCHAR subkey_name[MAX_KEY_LENGTH];   // buffer for subkey name
   DWORD subkey_length;
   DWORD cSubKeys=0;               // number of subkeys 
-  DWORD cbMaxSubKey;              // longest subkey size 
-  FILETIME ftLastWriteTime;      // last write time 
   DWORD i, retCode;
   char *best_hit= NULL;
-  int best_update= 0;
-  char buffer[MAX_VALUE_NAME];
+  int usedSubversion = -1;
+  int usedBuild = 0;
 
   // Find the Java key
-  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\JavaSoft\\Java Runtime Environment", 
-    0, KEY_READ, &java_key) != 0)
+  if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\\JavaSoft\\Java Runtime Environment", 0, KEY_READ, &java_key) != 0)
 	  return MYX_GRT_JAVA_NOT_FOUND;
 
   // Get the class name and the value count. 
-  retCode = RegQueryInfoKey(
-    java_key,                    // key handle 
-    NULL,                // buffer for class name 
-    NULL,           // size of class string 
-    NULL,                    // reserved 
-    &cSubKeys,               // number of subkeys 
-    &cbMaxSubKey,            // longest subkey size 
-    NULL,            // longest class string 
-    NULL,                // number of values for this key 
-    NULL,            // longest value name 
-    NULL,         // longest value data 
-    NULL,   // security descriptor 
-    &ftLastWriteTime);       // last write time
+  RegQueryInfoKey(java_key, NULL, NULL, NULL, &cSubKeys, NULL, NULL, NULL, NULL, NULL, NULL, NULL); 
 
   for (i= 0; i < cSubKeys; i++) 
   {
     subkey_length= MAX_KEY_LENGTH;
-
-    retCode = RegEnumKeyEx(java_key, i,
-      subkey_name, 
-      &subkey_length, 
-      NULL, 
-      NULL, 
-      NULL, 
-      &ftLastWriteTime);
-
+    retCode = RegEnumKeyEx(java_key, i, subkey_name, &subkey_length, NULL, NULL, NULL, NULL);
     if (retCode == ERROR_SUCCESS) 
     {
       char *better_hit= NULL;
 
       if (str_beginswith(subkey_name, version))
       {
-        if (subkey_length > 3)
+        DWORD reg_value_type;
+        BOOL fileExists = TRUE;
+        char buffer[MAX_VALUE_NAME];
+        int bufferSize = MAX_VALUE_NAME;
+
+        // Check if the jvm files exists and keep its name if it does, otherwise skip the entry.
+        RegOpenKeyEx(java_key, subkey_name, 0, KEY_READ, &java_subkey);
+        retCode = RegQueryValueEx(java_subkey, "RuntimeLib", NULL, &reg_value_type, buffer, &bufferSize) == 0;
+        RegCloseKey(java_subkey);
+        if (retCode == ERROR_SUCCESS);
         {
-          char subversion[2];
-          int better_update;
-
-          subversion[0]= subkey_name[4];
-          subversion[1]= 0;
-
-          better_update= atoi(subversion) * 100 + atoi(subkey_name + 6);
-
-          if (better_update > best_update)
+          // We got the RuntimeLib entry. Check whether it is a string value and points to an exisiting file.
+          if (reg_value_type == REG_SZ)
           {
-            best_update= better_update;
-            better_hit= g_strdup_printf("Software\\JavaSoft\\Java Runtime Environment\\%s", subkey_name);
-          }
-        }
-        else if (best_update == 0)
-        {
-          better_hit= g_strdup_printf("Software\\JavaSoft\\Java Runtime Environment\\%s", subkey_name);
-        }
-      }
+            TCHAR filename[MAX_PATH];
+            FILE* file;
 
-      if (better_hit)
-      {
-        g_free(best_hit);
-        best_hit= better_hit;
-      }
-    }
-  }
+            ExpandEnvironmentStrings(buffer, filename, MAX_PATH);
+            file = fopen(filename, "r");
+            if (file != NULL)
+            {
+              fileExists = TRUE;
+              fclose(file);
+              better_hit = g_strdup(filename);
+            };
+          };
 
+          if (fileExists)
+          {
+            // Now check the version info to find the highest installed version.
+            // Since the major and minor version numbers are given as parameters we only need 
+            // to check the sub version and the build numbers (e.g. as in 1.5.0_02).
+            gchar** parts = g_strsplit(subkey_name, ".", 3);
+            
+            // No need to process subversion or build if they aren't given (e.g. as in 1.5).
+            if (parts[1] != NULL && parts[2] != NULL)
+            {
+              int subVersion, build;
+              gchar** restParts = g_strsplit(parts[2], "_", 2);
+              subVersion = atoi(restParts[0]);
+              if (restParts[1] != NULL)
+                build = atoi(restParts[1]);
+              else
+                build = 0;
+              g_strfreev(restParts);
+              if (usedSubversion < 0)
+              {
+                // Nothing found yet, so use this entry.
+                best_hit = better_hit;
+                usedSubversion = subVersion;
+                usedBuild = build;
+              }
+              else
+                if ((usedSubversion < subVersion) || (usedSubversion == subVersion && usedBuild < build))
+                {
+                  usedSubversion = subVersion;
+                  usedBuild = build;
+                  g_free(best_hit);
+                  best_hit = better_hit;
+                };
+            }
+            else
+              if (usedSubversion < 0)
+              {
+                // Nothing found yet, so use this entry.
+                best_hit = better_hit;
+                usedSubversion = 0;
+                usedBuild = 0;
+              };
+
+            g_strfreev(parts);
+          };
+        };
+      };
+    };
+  };
   RegCloseKey(java_key);
 
-  // if a 1.4 hit was found, get the RuntimeLib value
   if (best_hit)
   {
-    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, best_hit, 
-      0, KEY_READ, &java_key) != 0)
-    {
-      g_free(best_hit);
-	    return MYX_GRT_JAVA_NOT_FOUND;
-    }
-
-    g_free(best_hit);
-
-    if (!registry_get_value_as_string(java_key, "RuntimeLib", buffer, sizeof(buffer))) 
-	    return MYX_GRT_JAVA_REGISTRY_CORRUPTED;
-
-    if (!buffer || !buffer[0])
-      return MYX_GRT_JAVA_NOT_FOUND;
-
-    *runtimelib_filename= g_strdup(buffer);
-
-    RegCloseKey(java_key);
-
+    *runtimelib_filename= best_hit;
     return MYX_GRT_NO_ERROR;
   }
   else
-  {
     return MYX_GRT_JAVA_NOT_FOUND;
-  }
 }
 
 #else // !windows
