@@ -59,6 +59,14 @@ static MYX_GRT *get_grt(lua_State *L)
 }
 
 
+static void module_add_listener(MYX_GRT *grt, char *name, void *param, void *userdata)
+{
+  lua_State *l= (lua_State*)userdata;
+
+  myx_lua_refresh(grt, l);
+} 
+
+
 MYX_GRT_MODULE_LOADER *myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, const char *lua_module_path)
 {
   MYX_GRT_MODULE_LOADER *loader= g_new0(MYX_GRT_MODULE_LOADER, 1);
@@ -67,6 +75,8 @@ MYX_GRT_MODULE_LOADER *myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, c
     ".lua"
   };
   char *lua_module_file;
+  GDir *dir;
+  const char *entry;
   
   GRT_ENTER(grt);
 
@@ -81,7 +91,7 @@ MYX_GRT_MODULE_LOADER *myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, c
   loader->extensions= file_extensions;
 
   priv->lua= lua_open();
-  
+
   // register a global __GRT variable 
   {
     MYX_GRT **userdata;
@@ -103,36 +113,54 @@ MYX_GRT_MODULE_LOADER *myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error, c
     }
   }
 
+  // register listener for module notifications
+  myx_grt_add_listener(grt, module_add_listener, GRT_MODULE_ADD_NOTIFICATION, priv->lua);
+  
   // register logging functions
   lua_register(priv->lua, "logerror", l_log_error);
   lua_register(priv->lua, "logwarning", l_log_warning);
   lua_register(priv->lua, "logmessage", l_log_message);
 
-  lua_module_file= g_build_filename(lua_module_path, "_library.lua", NULL);
-  if (check_file_exists(lua_module_file))
+  myx_lua_register_functions(grt, priv->lua);
+  
+  dir= g_dir_open(lua_module_path, 0, NULL);
+
+  if (dir)
   {
-    int status;
-    // load the _library
-    status= luaL_loadfile(priv->lua, lua_module_file);
-    if (status != 0)
+    while ((entry= g_dir_read_name(dir)) != NULL)
     {
-      if (getenv("GRT_VERBOSE"))
+      if (entry[0] == '_')
       {
-        g_warning("Could not load lua _library: %s", lua_tostring(priv->lua, -1));
-        lua_pop(priv->lua, 1);
+        lua_module_file= g_build_filename(lua_module_path, entry, NULL);
+        if (check_file_exists(lua_module_file))
+        {
+          int status;
+          // load the _library
+          status= luaL_loadfile(priv->lua, lua_module_file);
+          if (status != 0)
+          {
+            if (getenv("GRT_VERBOSE"))
+            {
+              g_warning("Could not load lua _library: %s", lua_tostring(priv->lua, -1));
+              lua_pop(priv->lua, 1);
+            }
+          }
+
+          // execute the _library, so that function declarations in it get executed
+          status= lua_pcall(priv->lua, 0, 0, 0);
+
+          if (status != 0)
+          {
+            g_warning("error executing lua _library: %s\n", lua_tostring(priv->lua, -1));
+            lua_pop(priv->lua, 1);
+          }
+        }
+        g_free(lua_module_file);
       }
     }
 
-    // execute the _library, so that function declarations in it get executed
-    status= lua_pcall(priv->lua, 0, 0, 0);
-
-    if (status != 0)
-    {
-      g_warning("error executing lua _library: %s\n", lua_tostring(priv->lua, -1));
-      lua_pop(priv->lua, 1);
-    }
+    g_dir_close(dir);
   }
-  g_free(lua_module_file);
 
   g_assert(lua_gettop(priv->lua) == 0);
   
