@@ -1,18 +1,20 @@
-/* Copyright (C) 2003,2004 MySQL AB
-
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
-
-   This program is distributed in the hope that it will be useful,
+/* Copyright (c) 2005 MySQL AB
+  
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Lesser General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+  
+   This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+  
+   You should have received a copy of the GNU Lesser General Public
+   License along with this library; if not, write to the
+   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+ */
 
 #ifdef ENABLE_LUA_MODULES
 
@@ -1389,6 +1391,15 @@ static MYX_GRT_SHELL_COMMAND_HELP_TEXT help_grtV[] =
       "               storedConnsFilename and assigns it to rdbmsMgmt.storedConns",
     NULL
   },
+  {
+    "lookupAdd",
+    "Adds the given object to the global object id cache.",
+    "grtV.lookupAdd (obj)",
+    "obj             The object to add to the cache.",
+    "  grtV.lookupAdd(conn)"NL
+    "               Adds the object conn to the cache.",
+    NULL
+  },  
   {
     "lookupId",
     "Looks up the id in the global object id cache and returns the value if it's found.",
