@@ -20,7 +20,7 @@
 //#define NSAPI 1
 
 #include "myx_grt_php.h"
-#include "myx_shared_aux_functions.h"
+#include "myx_shared_util_functions.h"
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
 #include <Basetsd.h>
