@@ -26,6 +26,20 @@ MYX_GRT_VALUE *get_schemata(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *reverse_engineer(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *get_character_sets(MYX_GRT_VALUE *param, void *data);
 
+static MYX_GRT_BUILTIN_FUNCTION functions_reverse_engineer[]= {
+  {"getVersion::", get_version },
+  {"getSchemata::", get_schemata },
+  {"reverseEngineer::", reverse_engineer },
+  {"getCharacterSets::", get_character_sets },
+};
+
+static MYX_GRT_BUILTIN_MODULE grt_module_reverse_engineer_mysql= {
+  "ReverseEngineeringMysql",
+  NULL,
+  sizeof(functions_reverse_engineer)/sizeof(MYX_GRT_BUILTIN_FUNCTION),
+  functions_reverse_engineer
+};
+
 #ifdef __cplusplus
 }
 #endif
