@@ -24,12 +24,36 @@ extern "C" {
 MYX_GRT_VALUE * generate_sql_create_statements(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE * execute_sql_statements(MYX_GRT_VALUE *param, void *data);
 
-MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *get_sql_script(MYX_GRT_VALUE *param, void *data);
 
 MYX_GRT_VALUE *get_sql_create(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *get_sql_drop(MYX_GRT_VALUE *param, void *data);
 MYX_GRT_VALUE *get_sql_merge(MYX_GRT_VALUE *param, void *data);
 
+MYX_GRT_VALUE *get_script_header(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *get_script_footer(MYX_GRT_VALUE *param, void *data);
+
+MYX_GRT_VALUE *split_sql_commands(MYX_GRT_VALUE *param, void *data);
+
+static MYX_GRT_BUILTIN_FUNCTION functions_transformation[]= {
+  {"generateSqlCreateStatements::", generate_sql_create_statements },
+  {"executeSqlStatements::", execute_sql_statements },
+  {"getSqlScript::", get_sql_script },
+  {"getSqlCreate::", get_sql_create },
+  {"getSqlDrop::", get_sql_drop },
+  {"getSqlMerge::", get_sql_merge },
+  {"getScriptHeader::", get_script_header },
+  {"getScriptFooter::", get_script_footer },
+  {"splitSqlCommands::", split_sql_commands },
+};
+
+static MYX_GRT_BUILTIN_MODULE grt_module_transformation_mysql= {
+  "TransformationMysql",
+  NULL,
+  sizeof(functions_transformation)/sizeof(MYX_GRT_BUILTIN_FUNCTION),
+  functions_transformation
+};
+
 #ifdef __cplusplus
 }
 #endif
