@@ -50,6 +50,13 @@ static MYX_GRT_VALUE *reverse_engineer_routine_params(MYSQL *mysql, MYX_GRT_VALU
                                                MYX_GRT_VALUE *schema, MYX_GRT_VALUE *sp);
 static void reverse_engineer_update_foreign_keys_references(MYX_GRT_VALUE *catalog);
 
+// --------------------------------------------------------------------------
+// module registration functions
+
+MYX_GRT_MODULE* myx_register_builtin_grt_module_reverse_engineer_mysql(MYX_GRT *grt)
+{
+  return myx_grt_module_register_builtin(grt, &grt_module_reverse_engineer_mysql, grt);
+}
 
 /**
  ****************************************************************************
@@ -902,10 +909,15 @@ static MYX_GRT_VALUE *reverse_engineer_table_columns(MYX_GRT *grt, MYSQL *mysql,
     {
       char *default_value;
       val= myx_convert_dbstr_utf8(mysql, row[fi[5]]);
-      default_value= g_strdup_printf("'%s'", val);
-      myx_grt_dict_item_set_value_from_string(column, "defaultValue", default_value);
+      if (val && (g_ascii_strcasecmp(val, "CURRENT_TIMESTAMP") == 0))
+        myx_grt_dict_item_set_value_from_string(column, "defaultValue", val);
+      else
+      {
+        default_value= g_strdup_printf("'%s'", val);
+        myx_grt_dict_item_set_value_from_string(column, "defaultValue", default_value);
+        g_free(default_value);
+      }
       g_free(val);
-      g_free(default_value);
 
       myx_grt_dict_item_set_value_from_int(column, "defaultValueIsNull", 0);
     }
@@ -930,7 +942,7 @@ static MYX_GRT_VALUE *reverse_engineer_table_columns(MYX_GRT *grt, MYSQL *mysql,
     myx_grt_value_release(column);
 
     // primary key
-    if ((row[fi[4]]) && (strcmp2(row[fi[4]], "PRI") == 0))
+    /*if ((row[fi[4]]) && (strcmp2(row[fi[4]], "PRI") == 0))
     {
       MYX_GRT_VALUE *pk= myx_grt_dict_item_get_value(table, "primaryKey");
       MYX_GRT_VALUE *pk_columns;
@@ -960,7 +972,7 @@ static MYX_GRT_VALUE *reverse_engineer_table_columns(MYX_GRT *grt, MYSQL *mysql,
       myx_grt_list_item_add_as_string(pk_columns, myx_grt_dict_item_get_as_string(column, "_id"));
 
       myx_grt_list_item_count(pk_columns);
-    }
+    }*/
   }
 
   mysql_free_result(res);
@@ -1033,8 +1045,8 @@ static MYX_GRT_VALUE *reverse_engineer_table_indices(MYSQL *mysql, MYX_GRT_VALUE
     MYX_GRT_VALUE *index_column;
     unsigned int i;
 
-    if (strcmp2(row[fi[2]], "PRIMARY") == 0)
-      continue;
+    /*if (strcmp2(row[fi[2]], "PRIMARY") == 0)
+      continue;*/
 
     // check if this is a new index (and not just the 2nd or 3rd column of the last index)
     // if it is a new index, create the GRT value for it
@@ -1060,7 +1072,7 @@ static MYX_GRT_VALUE *reverse_engineer_table_indices(MYSQL *mysql, MYX_GRT_VALUE
 
       // unique
       if (fi[1] > -1)
-        myx_grt_dict_item_set_value_from_int(index, "unique", strcmp2(row[fi[2]], "1") ? 1 : 0);
+        myx_grt_dict_item_set_value_from_int(index, "unique", strcmp2(row[fi[1]], "1") ? 1 : 0);
 
       // indexKind and indexType
       if (strcmp2(row[fi[2]], "PRIMARY") == 0)
@@ -1106,6 +1118,16 @@ static MYX_GRT_VALUE *reverse_engineer_table_indices(MYSQL *mysql, MYX_GRT_VALUE
 
       // add index to the table index list
       myx_grt_list_item_add(indices, index);
+
+      // if this is the primary key, set it in the table
+      if (strcmp2(row[fi[2]], "PRIMARY") == 0)
+      {
+        myx_grt_dict_item_set_value_from_int(index, "isPrimary", 1);
+
+        myx_grt_dict_item_set_value_from_string(table, "primaryKey", 
+          myx_grt_dict_item_get_as_string(index, "_id"));
+      }
+
       myx_grt_value_release(index);
     }
     
