@@ -18,12 +18,14 @@
 #include <myx_public_interface.h>
 #include <myx_grt_public_interface.h>
 #include <myx_grt_builtin_module_public_interface.h>
+#include <myx_sql_parser_public_interface.h>
 
 #include "myx_grt_mysql_transformation.h"
-#include "myx_aux_functions.h"
+#include "myx_util_functions.h"
 
 
-static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const char *assets_name, void *data);
+static void generate_sql_create_statements_asset(MYX_GRT_VALUE *asset, int include_drop, void *data);
+static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const char *assets_name, int include_drop, void *data);
 MYX_GRT_VALUE * execute_sql_statements(MYX_GRT_VALUE *param, void *data);
 
 static char *get_sql_create_script_assets(MYX_GRT_VALUE *schema, const char *assets_name);
@@ -40,22 +42,36 @@ static char * get_sql_drop_routine(MYX_GRT *grt, MYX_GRT_VALUE *routine);
 
 static char * comment_out(char *sql);
 
+static int process_split_sql_commands(const char *sql, void *user_data);
+
+
+// --------------------------------------------------------------------------
+// module registration functions
+
+MYX_GRT_MODULE* myx_register_builtin_grt_module_transformation_mysql(MYX_GRT *grt)
+{
+  return myx_grt_module_register_builtin(grt, &grt_module_transformation_mysql, grt);
+}
 
 
 MYX_GRT_VALUE *generate_sql_create_statements(MYX_GRT_VALUE *param, void *data)
 {
 //  MYX_GRT *grt= (MYX_GRT *) data;
   MYX_GRT_VALUE *catalog= NULL;
+  MYX_GRT_VALUE *options= NULL;
 
   if (!param)
     return myx_grt_function_create_error_result("You have to submit one object as parameter.", NULL);
 
-  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) == 1))
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) >= 1))
     catalog= myx_grt_list_item_get(param, 0);
   else if ( (myx_grt_value_get_type(param) == MYX_DICT_VALUE) &&
     (strcmp2(myx_grt_dict_struct_get_name(param), "db.mysql.Catalog") == 0) )
     catalog= param;
 
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) >= 2))
+    options= myx_grt_list_item_get(param, 1);
+
   if (!catalog)
     return myx_grt_function_create_error_result("The submitted parameter has to be a dict value of the struct db.mysql.catalog.", NULL);
   else
@@ -66,16 +82,20 @@ MYX_GRT_VALUE *generate_sql_create_statements(MYX_GRT_VALUE *param, void *data)
     for (i= 0; i < myx_grt_list_item_count(schemata); i++)
     {
       MYX_GRT_VALUE *schema= myx_grt_list_item_get(schemata, i);
-      MYX_GRT_VALUE *schema_create_string= 
-        myx_grt_function_extract_value_from_result(
-          get_sql_create(schema, data));
+      int include_drop= 0;
 
-      myx_grt_dict_item_set_value(schema, "sql", schema_create_string);
-      myx_grt_value_release(schema_create_string);
+      if (options && (strcmp2(myx_grt_dict_item_get_as_string(options, "KeepSchema"), "yes") != 0))
+        generate_sql_create_statements_asset(schema, 1, data);
+      else
+      {
+        generate_sql_create_statements_asset(schema, 0, data);
+        include_drop= 1;
+      }
 
-      generate_sql_create_statements_assets(schema, "tables", data);
-      generate_sql_create_statements_assets(schema, "views", data);
-      generate_sql_create_statements_assets(schema, "routines", data);
+      // get the schema assets
+      generate_sql_create_statements_assets(schema, "tables", include_drop, data);
+      generate_sql_create_statements_assets(schema, "views", include_drop, data);
+      generate_sql_create_statements_assets(schema, "routines", include_drop, data);
     }
   }
 
@@ -88,7 +108,7 @@ MYX_GRT_VALUE * execute_sql_statements(MYX_GRT_VALUE *param, void *data)
   return myx_grt_function_create_result(NULL);
 }
 
-static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const char *assets_name, void *data)
+static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const char *assets_name, int include_drop, void *data)
 {
   MYX_GRT_VALUE *assets= myx_grt_dict_item_get_value(schema, assets_name);
   unsigned int i;
@@ -98,60 +118,99 @@ static void generate_sql_create_statements_assets(MYX_GRT_VALUE *schema, const c
     for (i= 0; i < myx_grt_list_item_count(assets); i++)
     {
       MYX_GRT_VALUE *asset= myx_grt_list_item_get(assets, i);
-      MYX_GRT_VALUE *asset_create_string= 
-          myx_grt_function_extract_value_from_result(
-            get_sql_create(asset, data));
 
-      myx_grt_dict_item_set_value(asset, "sql", asset_create_string);
-      myx_grt_value_release(asset_create_string);
+      generate_sql_create_statements_asset(asset, include_drop, data);
     }
   }
 }
 
-MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
+static void generate_sql_create_statements_asset(MYX_GRT_VALUE *asset, int include_drop, void *data)
+{
+  MYX_GRT_VALUE *asset_create_string;      
+  char *sql= NULL;
+
+  if (include_drop)
+  {
+    MYX_GRT_VALUE *asset_drop_string= 
+      myx_grt_function_extract_value_from_result(
+        get_sql_drop(asset, data));
+
+    sql= g_strdup(myx_grt_value_as_string(asset_drop_string));
+    sql= str_g_append(sql, _br);
+
+    myx_grt_value_release(asset_drop_string);
+  }
+
+  asset_create_string= 
+      myx_grt_function_extract_value_from_result(
+        get_sql_create(asset, data));
+
+  sql= str_g_append(sql, myx_grt_value_as_string(asset_create_string));
+
+  myx_grt_dict_item_set_value_from_string(asset, "sql", sql);
+
+  g_free(sql);
+  
+  myx_grt_value_release(asset_create_string);
+}
+
+static char *get_sql_create_script_assets(MYX_GRT_VALUE *schema, const char *assets_name)
+{
+  char *sql= NULL;
+  MYX_GRT_VALUE *assets= myx_grt_dict_item_get_value(schema, assets_name);
+  unsigned int i;
+
+  for (i= 0; i < myx_grt_list_item_count(assets); i++)
+  {
+    MYX_GRT_VALUE *asset= myx_grt_list_item_get(assets, i);
+
+    sql= str_g_append(sql, myx_grt_dict_item_get_as_string(asset, "sql"));
+    sql= str_g_append(sql, _br _br);
+  }
+
+  return sql;
+}
+
+MYX_GRT_VALUE *get_sql_script(MYX_GRT_VALUE *param, void *data)
 {
 //  MYX_GRT *grt= (MYX_GRT *) data;
   MYX_GRT_VALUE *result= NULL;
   MYX_GRT_VALUE *catalog= NULL;
+  MYX_GRT_VALUE *options= NULL;
   char *sql;
 
   if (!param)
     return myx_grt_function_create_error_result("You have to submit one object as parameter.", NULL);
 
-  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) == 1))
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) >= 1))
     catalog= myx_grt_list_item_get(param, 0);
   else if ( (myx_grt_value_get_type(param) == MYX_DICT_VALUE) &&
     (strcmp2(myx_grt_dict_struct_get_name(param), "db.mysql.Catalog") == 0) )
     catalog= param;
 
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) >= 2))
+    options= myx_grt_list_item_get(param, 1);
+
   if (!catalog)
     return myx_grt_function_create_error_result("The submitted parameter has to be a dict value of the struct db.mysql.catalog.", NULL);
   else
   {
     MYX_GRT_VALUE *schemata= myx_grt_dict_item_get_value(catalog, "schemata");
     unsigned int i;
-
-    sql= g_strdup(
-        "-- ----------------------------------------------------------------------" _br
-		    "-- SQL create script" _br
-		    "-- ----------------------------------------------------------------------" _br 
-        _br
-        "SET FOREIGN_KEY_CHECKS = 0;" _br _br);
+    MYX_GRT_VALUE *header, *footer;
+    
+    // get header
+    header= myx_grt_function_extract_value_from_result(get_script_header(options, data));
+    sql= g_strdup(myx_grt_value_as_string(header));
+    myx_grt_value_release(header);
 
     for (i= 0; i < myx_grt_list_item_count(schemata); i++)
     {
       char *sql_part;
       MYX_GRT_VALUE *schema= myx_grt_list_item_get(schemata, i);
-      MYX_GRT_VALUE *schema_drop_string= get_sql_drop(schema, data);
-
-      // add drop schema
-      sql= str_g_append(sql, myx_grt_dict_item_get_as_string(schema_drop_string, "value"));
-      sql= str_g_append(sql, _br _br);
-
-      myx_grt_value_release(schema_drop_string);
 
       sql= str_g_append(sql, myx_grt_dict_item_get_as_string(schema, "sql"));
-      sql= str_g_append(sql, _br _br);
+      sql= str_g_append(sql, _br);
 
 
       // tables
@@ -194,7 +253,10 @@ MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
         g_free(sql_part);
     }
 
-    sql= str_g_append(sql, "SET FOREIGN_KEY_CHECKS = 1;" _br _br);
+    // get footer
+    footer= myx_grt_function_extract_value_from_result(get_script_footer(options, data));
+    sql= str_g_append(sql, myx_grt_value_as_string(footer));
+    myx_grt_value_release(footer);
   }
 
   result= myx_grt_function_create_result(myx_grt_value_from_string(sql));
@@ -203,23 +265,6 @@ MYX_GRT_VALUE *get_sql_create_script(MYX_GRT_VALUE *param, void *data)
   return result;
 }
 
-static char *get_sql_create_script_assets(MYX_GRT_VALUE *schema, const char *assets_name)
-{
-  char *sql= NULL;
-  MYX_GRT_VALUE *assets= myx_grt_dict_item_get_value(schema, assets_name);
-  unsigned int i;
-
-  for (i= 0; i < myx_grt_list_item_count(assets); i++)
-  {
-    MYX_GRT_VALUE *asset= myx_grt_list_item_get(assets, i);
-
-    sql= str_g_append(sql, myx_grt_dict_item_get_as_string(asset, "sql"));
-    sql= str_g_append(sql, _br _br);
-  }
-
-  return sql;
-}
-
 MYX_GRT_VALUE *get_sql_create(MYX_GRT_VALUE *param, void *data)
 {
   MYX_GRT *grt= (MYX_GRT *) data;
@@ -331,7 +376,7 @@ static char * get_sql_create_schema(MYX_GRT *grt, MYX_GRT_VALUE *schema)
   const char *collation= myx_grt_dict_item_get_as_string(schema, "defaultCollationName");
 
   char *sql= g_strdup_printf(
-      "CREATE DATABASE %s%s%s" _br,
+      "CREATE DATABASE IF NOT EXISTS %s%s%s" _br,
       quote_char, schema_name, quote_char);
 
   if ((charset) && (charset[0]))
@@ -399,7 +444,7 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
   const char *table_name= myx_grt_dict_item_get_as_string(table, "name");
   char *quoted_schema_name= get_owner_name(grt, table, quote_char);
   MYX_GRT_VALUE *columns= myx_grt_dict_item_get_value(table, "columns");
-  MYX_GRT_VALUE *pk= myx_grt_dict_item_get_value(table, "primaryKey");
+  MYX_GRT_VALUE *pk= myx_grt_dict_item_get_reference_value(grt, table, "primaryKey");
   MYX_GRT_VALUE *indices= myx_grt_dict_item_get_value(table, "indices");
   MYX_GRT_VALUE *fks= myx_grt_dict_item_get_value(table, "foreignKeys");
   char *sql= NULL;
@@ -427,6 +472,8 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
   const char *opt_avg_row_length= myx_grt_dict_item_get_as_string(table, "avgRowLength");
   const char *opt_min_rows= myx_grt_dict_item_get_as_string(table, "minRows");
   const char *opt_max_rows= myx_grt_dict_item_get_as_string(table, "maxRows");
+  int is_integer= 0;
+  int is_float= 0;
 
   // build create table sql
   sql= g_strdup_printf(
@@ -447,6 +494,7 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
     const char *column_collation= myx_grt_dict_item_get_as_string(column, "collationName");
     char *datatype_definition;
     const char *default_value;
+    int default_value_is_null;
     char *column_comment;
     char *column_length= myx_grt_dict_item_get_formated_as_string(column, "length");
     char *column_precision= myx_grt_dict_item_get_formated_as_string(column, "precision");
@@ -460,7 +508,9 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
     {
       // append length / precision / scale to datatype definition
       if ((strcmp2(datatype, "CHAR") == 0) ||
-         (strcmp2(datatype, "VARCHAR") == 0))
+         (strcmp2(datatype, "VARCHAR") == 0) ||
+         (strcmp2(datatype, "VARBINARY") == 0) ||
+         (strcmp2(datatype, "BINARY") == 0))
       {
         datatype_definition= g_strdup_printf("%s(%s)", 
           datatype, column_length);
@@ -472,6 +522,8 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
          (strcmp2(datatype, "INTEGER") == 0) ||
          (strcmp2(datatype, "BIGINT") == 0))
       {
+        is_integer= 1;
+
         if (strcmp2(column_precision, "0") != 0)
         {
           datatype_definition= g_strdup_printf("%s(%s)", 
@@ -487,10 +539,18 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
          (strcmp2(datatype, "DECIMAL") == 0) ||
          (strcmp2(datatype, "NUMERIC") == 0))
       {
-        datatype_definition= g_strdup_printf("%s(%s, %s)", 
-          datatype,
-          column_precision,
-          column_scale);
+        is_float= 1;
+
+        // if the scale holds a negative value like -1, do not use it
+        if (column_scale[0] != '-')
+          datatype_definition= g_strdup_printf("%s(%s, %s)", 
+            datatype,
+            column_precision,
+            column_scale);
+        else
+          datatype_definition= g_strdup_printf("%s(%s)", 
+            datatype,
+            column_precision);
       }
       else
         datatype_definition= g_strdup(datatype);
@@ -531,7 +591,9 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
 
     // add default value
     default_value= myx_grt_dict_item_get_as_string(column, "defaultValue");
-    if (default_value && default_value[0])
+    default_value_is_null= myx_grt_dict_item_get_as_int(column, "defaultValueIsNull");
+    if (default_value && default_value[0] && !default_value_is_null && 
+      !((is_integer || is_float) && (strcmp(default_value, "''") == 0)))
     {
       sql_cols= str_g_append_and_free(sql_cols, g_strdup_printf(" DEFAULT %s", default_value));
     }
@@ -566,7 +628,9 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
 
     for (i= 0; i < myx_grt_list_item_count(pk_columns); i++)
     {
-      MYX_GRT_VALUE *column= myx_grt_list_item_get_reference_value(grt, pk_columns, i);
+      MYX_GRT_VALUE *index_column= myx_grt_list_item_get(pk_columns, i);
+      MYX_GRT_VALUE *column= myx_grt_dict_item_get_reference_value(grt, index_column, "referedColumn");
+      int index_column_length= myx_grt_dict_item_get_as_int(index_column, "columnLength");
 
       if (sql_pk_cols)
         sql_pk_cols= str_g_append(sql_pk_cols, ", ");
@@ -575,6 +639,10 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
           quote_char,
           myx_grt_dict_item_get_as_string(column, "name"),
           quote_char));
+
+      // add index column length if set
+      if (index_column_length > 0)
+        sql_pk_cols= str_g_append_and_free(sql_pk_cols, g_strdup_printf("(%d)", index_column_length));
     }
 
     if (sql_pk_cols)
@@ -598,6 +666,9 @@ static char * get_sql_create_table(MYX_GRT *grt, MYX_GRT_VALUE *table)
       MYX_GRT_VALUE *index_columns= myx_grt_dict_item_get_value(index, "columns");
       char *sql_index_cols= NULL;
 
+      if (myx_grt_dict_item_get_as_int(index, "isPrimary") == 1)
+        continue;
+
       for (j= 0; j < myx_grt_list_item_count(index_columns); j++)
       {
         MYX_GRT_VALUE *index_col= myx_grt_list_item_get(index_columns, j);
@@ -1030,3 +1101,85 @@ static char * comment_out(char *sql)
   
   return sql_commented_out;
 }
+
+MYX_GRT_VALUE *get_script_header(MYX_GRT_VALUE *param, void *data)
+{
+  MYX_GRT *grt= (MYX_GRT *) data;
+  const char *app= NULL;
+  const char *script_type= NULL;
+  char *sql;
+  MYX_GRT_VALUE *header;
+  MYX_GRT_VALUE *options= NULL;
+
+  if (param && (myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) == 1))
+    options= myx_grt_list_item_get(param, 0);
+  else if (param)
+    options= param;
+
+  if (options)
+  {
+    app= myx_grt_dict_item_get_as_string(options, "AppName");
+    script_type= myx_grt_dict_item_get_as_string(options, "ScriptType");
+  }
+
+  if (!app)
+    app= "MySQL GRT Application";
+
+  if (!script_type)
+    script_type= "SQL Script";
+
+
+  sql= g_strdup_printf(
+      "-- ----------------------------------------------------------------------" _br
+      "-- %s" _br
+		  "-- %s" _br
+		  "-- ----------------------------------------------------------------------" _br 
+      _br
+      "SET FOREIGN_KEY_CHECKS = 0;" _br _br, app, script_type);
+
+  header= myx_grt_value_from_string(sql);
+
+  g_free(sql);
+
+  return myx_grt_function_create_result(header);
+}
+
+MYX_GRT_VALUE *get_script_footer(MYX_GRT_VALUE *param, void *data)
+{
+  MYX_GRT *grt= (MYX_GRT *) data;
+  MYX_GRT_VALUE *footer= myx_grt_dict_new_obj(grt, "db.DatabaseObject", "ScriptFooter", "", "");
+
+  footer= myx_grt_value_from_string(
+      "SET FOREIGN_KEY_CHECKS = 1;" _br _br
+      "-- ----------------------------------------------------------------------" _br
+      "-- EOF" _br _br);
+
+  return myx_grt_function_create_result(footer);
+}
+
+int process_split_sql_commands(const char *sql, void *user_data)
+{
+  MYX_GRT_VALUE *sql_commands= (MYX_GRT_VALUE *) user_data;
+
+  myx_grt_list_item_add_as_string(sql_commands, sql);
+
+  return 1;
+}
+
+MYX_GRT_VALUE *split_sql_commands(MYX_GRT_VALUE *param, void *data)
+{
+  MYX_GRT_VALUE *sql_commands= myx_grt_list_new(MYX_STRING_VALUE, "");
+  MYX_GRT_VALUE *sql;
+
+  if (!param)
+    return myx_grt_function_create_error_result("You have to submit a SQL script string as parameter.", NULL);
+
+  if ((myx_grt_value_get_type(param) == MYX_LIST_VALUE) && (myx_grt_list_item_count(param) >= 1))
+    sql= myx_grt_list_item_get(param, 0);
+  else
+    sql= param;
+
+  myx_process_sql_statements(myx_grt_value_as_string(sql), &process_split_sql_commands, sql_commands, MYX_SPM_DELIMS_REQUIRED);
+
+  return myx_grt_function_create_result(sql_commands);
+}
\ No newline at end of file
