@@ -844,7 +844,7 @@ public class Grt {
 				childNode = childNode.getNextSibling();
 			}
 
-			return null;
+			return "";
 		} else if (valueNodeType == Grt.MYX_INT_VALUE) {
 			Node childNode = valueNode.getChildNodes().item(0);
 			do {
@@ -855,7 +855,7 @@ public class Grt {
 				childNode = childNode.getNextSibling();
 			} while (childNode != null);
 
-			return null;
+			return new Integer(0);
 		} else if (valueNodeType == Grt.MYX_REAL_VALUE) {
 			Node childNode = valueNode.getChildNodes().item(0);
 			do {
@@ -866,7 +866,7 @@ public class Grt {
 				childNode = childNode.getNextSibling();
 			} while (childNode != null);
 
-			return null;
+			return new Double(0);
 		} else if (valueNodeType == Grt.MYX_DICT_VALUE) {
 			return getDictFromGrtXmlNode(valueNode, parent);
 		} else if (valueNodeType == Grt.MYX_LIST_VALUE) {
@@ -1073,6 +1073,9 @@ public class Grt {
 		if (errorMessage == null) {
 			e = e.getCause();
 			errorMessage = e.getMessage();
+			
+			if (errorMessage == null)
+				errorMessage= e.getClass().getName();
 		}
 
 		StackTraceElement[] stack = e.getStackTrace();
@@ -1433,4 +1436,14 @@ public class Grt {
 		return getCallback().getGrtGlobalDictKey(getMyxGrtPointer(),
 				objectPath, index);
 	}
+	
+	public void setBridgeDataObject(String objectPath, Object dataObj) {
+		getCallback().setBridgeDataObject(getMyxGrtPointer(),
+				objectPath, dataObj);
+	}
+	
+	public Object getBridgeDataObject(String objectPath) {
+		return getCallback().getBridgeDataObject(getMyxGrtPointer(),
+				objectPath);
+	}
 }
\ No newline at end of file
