@@ -106,7 +106,7 @@ bool MGConnectDialog::init(int flags, const std::string &header_image_file)
   Gtk::Image *prog;
 
   try {
-    _xml= new MGGladeXML(myg_datadir+"/"GLADE_FILE);
+    _xml= new MGGladeXML(myg_datadir+"/"GLADE_FILE, "", "mysql-gui-common");
   } catch (...) {
     g_warning("could not load glade xml file '%s'", 
               std::string(myg_datadir+"/"GLADE_FILE).c_str());
