@@ -326,7 +326,7 @@ bool MGTableBrowserList::refresh_sp_list(const Gtk::TreeIter &piter,
         column_iter= _store->append(nrow.children());
         Gtk::TreeModel::Row column_row= *column_iter;
         
-        //        column_row[_columns.icon]= ;
+        column_row[_columns.icon]= _sp_icon;
         column_row[_columns.text]= ufmt("%s: %s", param->name, param->datatype);
         column_row[_columns.type]= Parameter;
         column_row[_columns.weight]= Pango::WEIGHT_NORMAL;
