@@ -226,7 +226,7 @@ MGTableEditor::MGTableEditor(bool windowed)
   ((Gtk::CellRendererToggle*)_xml->get_tree("flags_tree")->get_column(0)->get_first_cell_renderer())->signal_toggled().connect(sigc::mem_fun(*this,&MGTableEditor::column_option_toggled));
   _xml->get_entry("name_entry")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"name"));
   _xml->get_entry("default_entry")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"default"));
-  _xml->get_combo_entry("datatype_combo")->get_entry()->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"type"));
+  ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"type"));
   _xml->get_combo("column_charset_combo")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"charset"));
   _xml->get_combo("column_collation_combo")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"collation"));
   _xml->get_toggle("pk_check")->signal_toggled().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::column_option_changed),"pk"));
@@ -262,7 +262,7 @@ MGTableEditor::MGTableEditor(bool windowed)
   _xml->get_entry("fk_name_entry")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::fk_option_changed),"name"));
   _xml->get_option("ondelete_option")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::fk_option_changed),"ondelete"));
   _xml->get_option("onupdate_option")->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::fk_option_changed),"onupdate"));
-  _xml->get_combo_entry("fk_table_combo")->get_entry()->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::fk_option_changed),"table"));
+  ((Gtk::Entry*)_xml->get_combo_entry("fk_table_combo")->get_child())->signal_changed().connect(sigc::bind<char*>(sigc::mem_fun(*this,&MGTableEditor::fk_option_changed),"table"));
 
   _xml->get_button("fk_add_button")->signal_clicked().connect(sigc::mem_fun(*this,&MGTableEditor::add_fk));
   _xml->get_button("fk_remove_button")->signal_clicked().connect(sigc::mem_fun(*this,&MGTableEditor::remove_fk));
@@ -279,6 +279,9 @@ MGTableEditor::MGTableEditor(bool windowed)
                          Gtk::Stock::GO_BACK, _("Back"));
 
   
+  _xml->get_combo("charset_combo")->signal_changed().connect(sigc::mem_fun(*this,&MGTableEditor::charset_changed));
+  
+  
   // menu
   {
     Gtk::MenuItem *item;
@@ -311,7 +314,7 @@ void MGTableEditor::show_table_info(MYX_DBM_TABLE_DATA *data)
 {
   _xml->get_entry("name_entry")->set_text(data->name?:"");
   
-  _xml->get_combo_entry("database_combo")->get_entry()->set_text(data->schema?:"");
+  ((Gtk::Entry*)_xml->get_combo_entry("database_combo")->get_child())->set_text(data->schema?:"");
   _xml->get_combo_entry("database_combo")->set_sensitive(false);
 
   _xml->get_entry("comment_entry")->set_text(data->comment?:"");
@@ -468,7 +471,7 @@ void MGTableEditor::show_column_info(MYX_DBM_TABLE_DATA *data, const Gtk::TreeIt
   if (iter)
     row= *iter;
   _xml->get_entry("column_name_entry")->set_text(iter ? (Glib::ustring)row[_tcolumns.name] : "");
-  _xml->get_combo_entry("datatype_combo")->get_entry()->set_text(iter ? (Glib::ustring)row[_tcolumns.type] : "");
+  ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->set_text(iter ? (Glib::ustring)row[_tcolumns.type] : "");
   _xml->get_entry("default_entry")->set_text(iter ? (Glib::ustring)row[_tcolumns.defval] : "");
   _xml->get_toggle("null_toggle")->set_active(iter ? (bool)row[_tcolumns.defnull] : false);
   _xml->get_toggle("pk_check")->set_active(iter ? (bool)row[_tcolumns.pk] : false);
@@ -698,7 +701,8 @@ void MGTableEditor::show_fk_info(MYX_DBM_TABLE_DATA *tdata, const Gtk::TreeIter
   _xml->get_entry("fk_name_entry")->set_text(iter ? (Glib::ustring)row[_fk_columns.name]:"");
   _xml->get_option("ondelete_option")->set_history(iter ? (int)row[_fk_columns.on_delete]:0);
   _xml->get_option("onupdate_option")->set_history(iter ? (int)row[_fk_columns.on_update]:0);
-  _xml->get_combo_entry("fk_table_combo")->get_entry()->set_text(iter ? (Glib::ustring)row[_fk_columns.table]:"");
+  ((Gtk::Entry*)_xml->get_combo_entry("fk_table_combo")->get_child())->set_text(iter ? (Glib::ustring)row[_fk_columns.table]:"");
+
   std::list<FKColumns::FKMapping> list;
   if (iter)
     list= row[_fk_columns.columns];
@@ -834,7 +838,6 @@ void MGTableEditor::fill_aux_data()
   _xml->get_combo_entry("datatype_combo")->set_model(typelist);
   _xml->get_combo_entry("datatype_combo")->set_text_column(0);
   
-  
   // set character set lists
   iter= cslist1->append();
   row= *iter;
@@ -879,21 +882,10 @@ void MGTableEditor::fill_collation_combo(Gtk::ComboBox *combo, const Glib::ustri
 
   for (unsigned int i= 0; _charsets && i < _charsets->charsets_num; i++)
   {
-    if (!_charsets->charsets[i].desc || !*_charsets->charsets[i].desc)
+    if (charset.compare(_charsets->charsets[i].name)==0)
     {
-      if (charset.compare(_charsets->charsets[i].name)==0)
-      {
-        chs= _charsets->charsets+i;
-        break;
-      }
-    }
-    else
-    {
-      if (charset.compare(_charsets->charsets[i].desc)==0)
-      {
-        chs= _charsets->charsets+i;
-        break;
-      }
+      chs= _charsets->charsets+i;
+      break;
     }
   }
 
@@ -1036,6 +1028,14 @@ bool MGTableEditor::delete_window_event(GdkEventAny *event)
 }
 
 
+void MGTableEditor::charset_changed()
+{
+  Glib::ustring charset= get_combo_item(_xml->get_combo("charset_combo"), "");
+  
+  fill_collation_combo(_xml->get_combo("collation_combo"), charset);
+}
+
+
 void MGTableEditor::column_tree_clicked(GdkEventButton *ev)
 {
   if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 3))
@@ -1189,9 +1189,9 @@ void MGTableEditor::column_edited(const Glib::ustring &path, const Glib::ustring
     if (row[_tcolumns.type] != new_text)
     {
       if (check_datatype(new_text))
-        _xml->get_combo_entry("datatype_combo")->get_entry()->set_text(new_text);
+        ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->set_text(new_text);
       else
-        row[_tcolumns.type]= _xml->get_combo_entry("datatype_combo")->get_entry()->get_text();
+        row[_tcolumns.type]= ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->get_text();
       
       {
         Glib::ustring a, b;
@@ -1311,7 +1311,7 @@ void MGTableEditor::column_option_changed(const char *option)
       
       split_datatype((Glib::ustring)row[_tcolumns.type], otype, param);
 
-      row[_tcolumns.type]= _xml->get_combo_entry("datatype_combo")->get_entry()->get_text();
+      row[_tcolumns.type]= ((Gtk::Entry*)_xml->get_combo_entry("datatype_combo")->get_child())->get_text();
       
       split_datatype((Glib::ustring)row[_tcolumns.type], type, param);
       
@@ -1613,7 +1613,7 @@ void MGTableEditor::fk_option_changed(const char *option)
     else if (strcmp(option,"onupdate")==0)
       row[_fk_columns.on_update]= (MYX_DBM_FK_ACTION)_xml->get_option("onupdate_option")->get_history();
     else if (strcmp(option,"table")==0)
-      row[_fk_columns.table]= _xml->get_combo_entry("fk_table_combo")->get_entry()->get_text();
+      row[_fk_columns.table]= ((Gtk::Entry*)_xml->get_combo_entry("fk_table_combo")->get_child())->get_text();
   }
 }
 
@@ -1690,11 +1690,20 @@ void MGTableEditor::revert_changes()
 }
 
 
-void MGTableEditor::validate()
+bool MGTableEditor::validate()
 {
   Gtk::TreeIter iter;
   Gtk::TreeRow row;
   bool flag;
+
+  if (_xml->get_entry("name_entry")->get_text().empty())
+  {
+    Gtk::MessageDialog dlg(_("<b>Cannot Create Table</b>\n\nPlease specify a name for the table."),
+                           true, Gtk::MESSAGE_ERROR);
+    dlg.run();
+
+    return false;
+  }
   
   // remove blank columns
   do {
@@ -1709,8 +1718,24 @@ void MGTableEditor::validate()
         flag= true;
         break;
       }
+      if (((Glib::ustring)row[_tcolumns.type]).empty())
+      {
+        Gtk::MessageDialog dlg(_("<b>Cannot Create Table</b>\n\nOne or more columns are missing their data types."),
+                               true, Gtk::MESSAGE_ERROR);
+        dlg.run();
+        return false;
+      }
     }
   } while (flag);
+  
+  if (_column_store->children().size() == 0)
+  {
+    Gtk::MessageDialog dlg(_("<b>Cannot Create Table</b>\n\nThere are no columns defined for the table."),
+                           true, Gtk::MESSAGE_ERROR);
+    column_add();
+    return false;
+  }  
+  return true;
 }
 
 
@@ -1786,7 +1811,8 @@ void MGTableEditor::commit_changes()
   if (!_data)
     return;
   
-  validate();
+  if (!validate())
+    return;
 
 #define ASSIGN(var, val) do { if ((val).compare(var?:"")!=0) {if (var) g_free(var); var= g_strdup((val).c_str());} } while (0)
 #define ASSIGN_N(var, val) do { if ((val).compare(var?:"")!=0) {if (var) g_free(var); if ((val).empty()) var= NULL; else var= g_strdup((val).c_str());} } while (0)
@@ -1794,7 +1820,7 @@ void MGTableEditor::commit_changes()
   
   // table options
   ASSIGN(_data->name, _xml->get_entry("name_entry")->get_text());
-  ASSIGN(_data->schema, _xml->get_combo_entry("database_combo")->get_entry()->get_text());
+  ASSIGN(_data->schema, ((Gtk::Entry*)_xml->get_combo_entry("database_combo")->get_child())->get_text());
   ASSIGN(_data->comment, _xml->get_entry("comment_entry")->get_text());
   
   // more table options
@@ -1811,11 +1837,11 @@ void MGTableEditor::commit_changes()
   
   _data->merge_insert= (MYX_DBM_TABLE_MERGE_INSERT)_xml->get_option("insert_method_option")->get_history();
   
-  if (_xml->get_combo_entry("charset_combo")->get_entry()->get_text() == DEFAULT_CHARSET)
+  if (get_combo_item(_xml->get_combo("charset_combo"), DEFAULT_CHARSET) == DEFAULT_CHARSET)
     SETNULL(_data->charset);
   else
   {
-    Gtk::TreeIter iter= _xml->get_combo_entry("charset_combo")->get_active();
+    Gtk::TreeIter iter= _xml->get_combo("charset_combo")->get_active();
     Gtk::TreeRow row= *iter;
     Glib::ustring charset= (char*)(void*)row[_columns.object];
     ASSIGN_N(_data->charset, charset);
@@ -2122,6 +2148,11 @@ void MGTableEditor::commit_changes()
         }
       }
     }
+    else
+    {
+      // add back placeholder column removed in validate()
+      column_add();
+    }
   }
 }
 
