@@ -18,7 +18,9 @@
 #ifndef _MGCANVAS_H_
 #define _MGCANVAS_H_
 
+#include <gtkmm/layout.h>
 #include <gtkmm/drawingarea.h>
+#include <gdkmm/cursor.h>
 #include <GL/gl.h>
 #include <GL/glx.h>
 
@@ -27,7 +29,11 @@
 
 #include "myx_gc_canvas.h"
 
-class MGCanvas : public Gtk::DrawingArea, CGCListener {
+
+class MGCanvasScroller;
+
+class MGCanvas : public Gtk::DrawingArea, CGCListenerInterface {
+    typedef Gtk::DrawingArea superclass;
   public:
     struct Point {
       float x, y;
@@ -48,6 +54,7 @@ class MGCanvas : public Gtk::DrawingArea, CGCListener {
       inline float get_width() { return size.width; };
       inline float get_height() { return size.height; };
     };
+        
   protected:
     GLXContext _context;
     
@@ -57,19 +64,23 @@ class MGCanvas : public Gtk::DrawingArea, CGCListener {
     Point _offset;
     
     Gdk::Point _lastClickPoint;
-    
+
     Gdk::Rectangle _viewport;
     
+    Gdk::Cursor _hand_cursor;
+    Gdk::Cursor _hand_closed_cursor;
+    
     double _zoom;
     int _currentZoomStep;
     
     bool _grabPanning;
+    bool _changingScrollers;
 
     Glib::Dispatcher _refresh;
     Glib::Dispatcher _switchView;
     
     struct {
-      unsigned panning:1;
+      unsigned panning:2;
     } _state;
     
     sigc::slot<bool,MGCanvas*,int,Point> _handleMouseDown;
@@ -85,24 +96,26 @@ class MGCanvas : public Gtk::DrawingArea, CGCListener {
     virtual bool on_scroll_event(GdkEventScroll* event);
     virtual bool on_motion_notify_event(GdkEventMotion* event);
     
-    virtual void onAction(CGCBase *sender, CGCBase *origin, TActionType &actionType);
+    virtual void onAction(CGCBase *sender, CGCBase *origin, TAction **action);
     virtual void onChange(CGCBase *sender, CGCBase *origin, TGCChangeReason Reason);
     virtual void onError(CGCBase *sender, CGCBase *origin, const char* Message);
     virtual void onDestroy(CGCBase *sender);
     
     void handle_switch_view();
     
-    bool set_scroll_adjustments(Gtk::Adjustment& hadjustment,
-                                Gtk::Adjustment& vadjustment);
-
     Point get_center_for_zoom();
 
+    MGCanvasScroller *get_enclosing_scroller();
+    
   public:
     MGCanvas();
     virtual ~MGCanvas();
+    
+    void set_hand_cursor(const Gdk::Cursor &hand, const Gdk::Cursor &closed_hand);
    
     void set_zoom(float zoom, Point center);
     void set_zoom_level(int level, Point center);
+    void set_zoom_level(int level=-1);
     int zoom_level() { return _currentZoomStep; };
     
     void toggle_overview();
@@ -124,6 +137,8 @@ class MGCanvas : public Gtk::DrawingArea, CGCListener {
     void update_from_scrollers();
     void update_scrollers();
     
+    void force_reconfigure();
+    
     void set_grab_panning(bool flag);
     
     void create_view(const Glib::ustring &name);
@@ -132,11 +147,15 @@ class MGCanvas : public Gtk::DrawingArea, CGCListener {
     
     void set_grid_enabled(bool flag);
     
-    void start_area_selection_at(Gdk::Point point);
+    void start_area_selection_at(Point point);
     Rectangle finish_area_selection();
 
+    void set_mouse_down_handler(sigc::slot<bool,MGCanvas*,int,Point> handler);
+    void set_mouse_up_handler(sigc::slot<bool,MGCanvas*,int,Point> handler);
+    void set_mouse_move_handler(sigc::slot<bool,MGCanvas*,Point> handler);
+
     CGenericCanvas *canvas() { return _canvas; };
-    
+
     std::vector<float> get_zoom_steps();
 };
 
