@@ -936,7 +936,8 @@ function getDriverMysqlNative(owner)
     {
       ReverseEngineeringModule= "ReverseEngineeringMysql",
       MigrationModule= "MigrationGeneric",
-      TransformationModule= "TransformationMysql"
+      TransformationModule= "TransformationMysql",
+      QueryModule= "QueryMysql"
     }
 
   driver.isInstalled= 1
