@@ -419,12 +419,8 @@ static int getModifiers(NSEvent *event)
     {
       _lastClickPoint= point;
       // panning
-      if (_grabPanning)
-      {
-        _state.panning= 1;
-        [[NSCursor closedHandCursor] push];
-      }
-      else if (([event modifierFlags] & NSCommandKeyMask) == NSCommandKeyMask)
+      if (_grabPanning
+	 || ([event modifierFlags] & NSCommandKeyMask) == NSCommandKeyMask)
       {
         _state.panning= 1;
         [[NSCursor closedHandCursor] push];
@@ -653,7 +649,7 @@ static int getModifiers(NSEvent *event)
 {
   struct timeval tm1, tm2;
   gettimeofday(&tm1, NULL);
-  _canvas->render();
+  _canvas->render(false);
   gettimeofday(&tm2, NULL);
   
   _lastFPS= 1 / ((tm2.tv_sec - tm1.tv_sec) + (tm2.tv_usec - tm1.tv_usec) / 1000000.0);
