@@ -15,6 +15,7 @@
 @end
 
 @implementation MConnectionEditor(Private)
+
 - (void)showUser:(MYX_USER_CONNECTION*)user
 {
   [name     setStringValue: user ? NSStr(user->connection_name) :@""];
@@ -29,8 +30,8 @@
                                                               user->advanced_options_num,
                                                               "SOCKET_PATH")) : @""];
   [compress setState:       user && myx_get_option_value((const char**)user->advanced_options,
-														 user->advanced_options_num,
-														 "COMPRESS") ? NSOnState : NSOffState];
+                                                         user->advanced_options_num,
+                                                         "COMPRESS") ? NSOnState : NSOffState];
 }
 
 
@@ -54,22 +55,22 @@
     {
       user->advanced_options= g_realloc(user->advanced_options, ++user->advanced_options_num * sizeof(char*));
       user->advanced_options[user->advanced_options_num-1]= 
-        g_strdup_printf("SOCEKT_PATH=%s", new_value);
+        g_strdup_printf("SOCKET_PATH=%s", new_value);
     }
     g_free(new_value);
-	
-	if ([compress state] == NSOnState)
-	{
-	  if (set_value(user->advanced_options, user->advanced_options_num, "COMPRESS", "1") < 0)
-	  {
-		user->advanced_options= g_realloc(user->advanced_options, ++user->advanced_options_num * sizeof(char*));
-		user->advanced_options[user->advanced_options_num-1]= g_strdup("COMPRESS=1");
-	  }	  
-	}
-	else
-	{
-	  set_value(user->advanced_options, user->advanced_options_num, "COMPRESS", NULL);
-	}
+    
+    if ([compress state] == NSOnState)
+    {
+      if (set_value(user->advanced_options, user->advanced_options_num, "COMPRESS", "1") < 0)
+      {
+        user->advanced_options= g_realloc(user->advanced_options, ++user->advanced_options_num * sizeof(char*));
+        user->advanced_options[user->advanced_options_num-1]= g_strdup("COMPRESS=1");
+      }	  
+    }
+    else
+    {
+      set_value(user->advanced_options, user->advanced_options_num, "COMPRESS", NULL);
+    }
   }
   
   if ([[name stringValue] length] > 0)
@@ -88,18 +89,18 @@
 {
   MYX_LIB_ERROR error;
   unsigned int i;
-
+  
   if (!(connections= myx_load_user_connections([filename fileSystemRepresentation],&error)))
   {
     connections= g_new0(MYX_USER_CONNECTIONS, 1);
   }
-
+  
   [indices removeAllObjects];
   for (i= 0; i < connections->user_connections_num; i++)
   {
     [indices addObject:[NSNumber numberWithInt:i]];
   }
-
+  
   [list reloadData];
 }
 
@@ -110,7 +111,7 @@
     connections->last_connection= [[list itemAtRow:[list selectedRow]] intValue];
   else
     connections->last_connection= 0;
-
+  
   myx_store_user_connections(connections, [MPreferences preferences]->passwordStorageType, 
                              [filename fileSystemRepresentation]);
 }
@@ -122,17 +123,19 @@
   id item;
   
   memset(&data, 0, sizeof(data));
-
+  
   data.connection_name= g_strdup("new connection");
   data.port= 3306;
-
+  
   connections->user_connections= vec_insert_resize(connections->user_connections, sizeof(MYX_USER_CONNECTION),
-                    &connections->user_connections_num, connections->user_connections_num,
-                    &data);
+                                                   &connections->user_connections_num, connections->user_connections_num,
+                                                   &data);
+  memset(&connections->user_connections[connections->user_connections_num-1], 0,
+         sizeof(MYX_USER_CONNECTION));
   
   item= [NSNumber numberWithInt:connections->user_connections_num-1];
   [indices addObject:item];
-
+  
   [list reloadData];
   
   [list selectRow:[list rowForItem:item] byExtendingSelection:NO];
@@ -165,10 +168,11 @@
   }  
 }
 
+
 - (void)textDidChange:(NSNotification *)aNotification
 {
   id item= [list itemAtRow:[list selectedRow]];
-
+  
   if (item && !([item isEqualTo:@"saved"] || [item isEqualTo:@"history"]))
   {
     [self updateUser: connections->user_connections+[item intValue]];
@@ -185,7 +189,7 @@
     {
       unsigned int i;
       MYX_USER_CONNECTION *conn= connections->user_connections+[item intValue];
- 
+      
       for (i= 0; i < connections->user_connections_num; i++)
       {
         if (connections->user_connections[i].connection_name &&
@@ -210,7 +214,7 @@
 - (void)controlTextDidChange:(NSNotification *)aNotification
 {
   id item= [list itemAtRow:[list selectedRow]];
-
+  
   if ([aNotification object] != name)
   {
     if (item && !([item isEqualTo:@"saved"] || [item isEqualTo:@"history"]))
@@ -250,6 +254,8 @@
     [self load:nil];
     [list expandItem:@"saved" expandChildren:YES];
     [list expandItem:@"history" expandChildren:YES];
+    
+    [self outlineViewSelectionDidChange:nil];
   }
   return self;
 }
@@ -257,6 +263,7 @@
 
 - (void)dealloc
 {
+  [list setDataSource:nil];
   [indices release];
   [filename release];
   [connIcon release];
@@ -340,7 +347,7 @@
       {
         if (count == index)
           return [indices objectAtIndex:i];
-
+        
         count++;
       }
     }    
@@ -365,7 +372,7 @@
   if (item && !([item isEqualTo:@"saved"] || [item isEqualTo:@"history"]))
   {
     [self showUser: connections->user_connections+[item intValue]];
-
+    
     [[topView viewWithTag:100] setEnabled:YES];
     [tabview setEnabledRecursive:YES];
   }
