@@ -34,19 +34,19 @@
 - (void)finish:(id)arg
 {
   int rc;
-  MYX_USER_CONNECTION *info= NULL;
+  MConnectionInfo *info= nil;
 
   if (_mysql && myx_mysql_errno(_mysql)==0)
   {
     MYX_USER_CONNECTION *conn= [self getDisplayedConnection];
     
-    info= myx_copy_user_connection(conn);
+    info= [MConnectionInfo connectionInfoFromUserConnection:conn];
     rc= 1;
     
     // save
     if (conn && !conn->connection_name)
     {
-      int i;
+      unsigned int i;
       // look for connection with same contents
       for (i= 0; i < _connections->user_connections_num; i++)
       {
@@ -81,7 +81,7 @@
     rc= 0;
 
   [[self window] close];
-  
+
   if (rc > 0)
     [_delegate connectionPanel:self finished:_mysql info:info];
   else
@@ -117,14 +117,19 @@
   
   if (!_mysql)
   {
-    NSRunAlertPanelRelativeToWindow(@"Error",
-                                    @"Could not initialize MySQL connection.",
-                                    @"OK", nil, nil, [self window]);
+    NSRunAlertPanel(@"Error",
+                    @"Could not initialize MySQL connection.",
+                    @"OK", nil, nil);
   }
   else if ((err= myx_mysql_errno(_mysql))!=0)
   {
-    [self showConnectionError: _mysql
-                         info: [[result valueForKey:@"info"] pointerValue]];
+    if (myx_get_mysql_major_version(_mysql) < 4 && myx_get_mysql_major_version(_mysql) > 1)
+      NSRunAlertPanel(@"Error",
+                      @"Sorry. MySQL server version 4.0 or newer is required, but you are connecting an older, unsupported version.",
+                      @"OK", nil, nil);
+    else
+      [self showConnectionError: _mysql
+                           info: [[result valueForKey:@"info"] pointerValue]];
     if (_mysql)
       myx_mysql_close(_mysql);
     _mysql= NULL;
@@ -138,11 +143,11 @@
   }
   else
   {
-	[[NSRunLoop currentRunLoop] performSelector:@selector(finish:)
-										 target:self
-									   argument:nil
-										  order:1000
-										  modes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
+    [[NSRunLoop currentRunLoop] performSelector:@selector(finish:)
+                                         target:self
+                                       argument:nil
+                                          order:1000
+                                          modes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
   }
 }
 
@@ -150,25 +155,30 @@
 - (void)connectThread: (id)info
 {
   MYSQL *mysql;
-  NSDictionary *result;
   BOOL ok= YES;
-  id tmp;
+  MYX_USER_CONNECTION *conn;
+  char *saveSchema= NULL;
   
   mysql_thread_init();
   
-  mysql= myx_mysql_init();
-
-  if (myx_connect_to_instance([info pointerValue], mysql) < 0)
-    ok= NO;
+  mysql= [[info objectForKey:@"mysql"] pointerValue];
+  conn= [[info objectForKey:@"info"] pointerValue];
   
-  tmp= [[NSValue alloc] initWithBytes:&mysql objCType:@encode(void *)];
-  result= [[NSDictionary alloc] initWithObjectsAndKeys:
-    tmp, @"mysql", info, @"info", 0];
+  if (!_pickSchema)
+  {
+    saveSchema= conn->schema;
+    conn->schema= NULL;
+  }
+  
+  if (myx_connect_to_instance(conn, mysql) < 0)
+    ok= NO;
 
+  if (!_pickSchema)
+    conn->schema= saveSchema;
+  
   [self performSelectorOnMainThread:@selector(connectionFinished:)
-                          withObject:result
+                          withObject:info
                        waitUntilDone:YES];
-  [result release];
   
   mysql_thread_end();
 }
@@ -180,7 +190,10 @@
   {
     [NSThread detachNewThreadSelector: @selector(connectThread:)
                              toTarget: self
-                           withObject: [NSValue valueWithPointer:connectionInfo]];
+                           withObject: [NSDictionary dictionaryWithObjectsAndKeys:
+                             [NSValue valueWithPointer:connectionInfo], @"info",
+                             [NSValue valueWithPointer:myx_mysql_init()], @"mysql",
+                             nil]];
   }
 }
 
@@ -211,7 +224,7 @@
     g_free(_new_connection.hostname);
     _new_connection.hostname= g_strdup([[hostname stringValue] UTF8String]);
     _new_connection.port= [port intValue];
-	_new_connection.schema= g_strdup([[schema stringValue] UTF8String]);
+    _new_connection.schema= g_strdup([[schema stringValue] UTF8String]);
 	
     _new_connection.storage_type= MYX_HISTORY_USER_CONNECTION;
 
@@ -221,15 +234,27 @@
     if (_new_connection.advanced_options)
       g_free(_new_connection.advanced_options);
     
-    _new_connection.advanced_options= g_new0(char*, 2);
+    _new_connection.advanced_options= g_new0(char*, 3);
     
     i= 0;
     if ([[socketText stringValue] length] > 0)
       _new_connection.advanced_options[i++]= g_strdup_printf("SOCKET_PATH=%s", 
                                                              [[[socketText stringValue] stringByExpandingTildeInPath] UTF8String]);
-	if ([compress state] == NSOnState)
-      _new_connection.advanced_options[i++]= g_strdup("COMPRESS=1");
-	
+    if ([compress state] == NSOnState)
+      _new_connection.advanced_options[i++]= g_strdup("COMPRESS=Y");
+    
+    switch ([ansiQuotes state])
+    {
+      case NSOnState:
+        _new_connection.advanced_options[i++]= g_strdup("ANSI_QUOTES=Y");
+        break;
+      case NSOffState:
+        _new_connection.advanced_options[i++]= g_strdup("ANSI_QUOTES=N");
+        break;
+      case NSMixedState:
+        break;
+    }
+    
     _new_connection.advanced_options_num= i;
     
     return &_new_connection;
@@ -244,7 +269,7 @@
     [password setStringValue: [NSString stringWithUTF8String: info->password?:""]];
     [hostname setStringValue: [NSString stringWithUTF8String: info->hostname?:""]];
     [port setIntValue: info->port];
-	[schema setStringValue: [NSString stringWithUTF8String: info->schema?:""]];
+    [schema setStringValue: [NSString stringWithUTF8String: info->schema?:""]];
     
     [socketText setStringValue: 
       [NSString stringWithUTF8String: 
@@ -252,10 +277,15 @@
                              info->advanced_options_num,
                              "SOCKET_PATH") ? : ""]];
 	
-	[compress setState: 
-        myx_get_option_value((const char**)info->advanced_options,
-                             info->advanced_options_num,
-                             "COMPRESS") ? NSOnState : NSOffState];
+    [compress setState: 
+      myx_get_option_value((const char**)info->advanced_options,
+                           info->advanced_options_num,
+                           "COMPRESS") ? NSOnState : NSOffState];
+    
+    const char *op= myx_get_option_value((const char**)info->advanced_options,
+                                   info->advanced_options_num,
+                                   "ANSI_QUOTES");
+    [ansiQuotes setState: op ? (*op == 'Y' ? NSOnState : NSOffState) : NSMixedState];
   }
 }
 
@@ -374,9 +404,9 @@
     _connections= conns;
   }
 
-  [connectionPopUp selectItemAtIndex:_lastSelectedConnection];
   [self fillConnectionPopUp:YES];
-  
+  [connectionPopUp selectItemAtIndex:_lastSelectedConnection];
+
   [editor release];
 }
 
@@ -432,18 +462,12 @@
 
 @implementation MConnectionPanel
 
-+ (NSString*)representationForConnection:(MYX_USER_CONNECTION*)conn
-{
-  if (conn->connection_name)
-    return [NSString stringWithUTF8String:conn->connection_name];
-  else
-    return [NSString stringWithFormat:@"%s@%s", conn->username, conn->hostname];
-}
-
 - (id)initWithConnectionsFile:(NSString*)file
 {
   MYX_LIB_ERROR error;
 
+  [MConnectionInfo initialize];
+  
   self= [super initWithWindowNibName: @"ConnectPanel"];
 
   _connectionsFile= [file retain];
@@ -487,6 +511,8 @@
   [[self window] setMinSize:frame.size];
   
   [password setNextKeyView:schema];
+  
+  _pickSchema= YES;
 }
 
 - (void)dealloc
@@ -509,6 +535,12 @@
   [super dealloc];
 }
 
+- (IBAction)advancedOptionChanged:(id)sender
+{
+  [connectionPopUp selectItemAtIndex:0];  
+}
+
+
 - (IBAction)cancel:(id)sender
 {
   if (!_connecting)
@@ -536,6 +568,7 @@
     [self startConnection: conn];
 }
 
+
 - (IBAction)connectionChanged:(id)sender
 {
   id pitem= [connectionPopUp selectedItem];
@@ -559,7 +592,7 @@
   {
     expanded= YES;
     newSize= [[self window] minSize];
-	newSize.height+= 88-22;
+    newSize.height+= 108-22;
   }
   else
   {
@@ -572,15 +605,15 @@
 
   if (expanded)
   {
-	[[self window] setFrame:frame display:YES animate:YES];
-	[extraView setFrame:NSMakeRect(20, 43, 388, 88)];
+    [[self window] setFrame:frame display:YES animate:YES];
+    [extraView setFrame:NSMakeRect(20, 43, 388, 108)];
   }
   else
   {
-	[[self window] setFrame:frame display:YES animate:YES];
-	[extraView setFrame:NSMakeRect(20, 43, 388, 22)];
+    [[self window] setFrame:frame display:YES animate:YES];
+    [extraView setFrame:NSMakeRect(20, 43, 388, 22)];
   }
-  
+
   [[[self window] contentView] setNeedsDisplay:YES];
 }
 
