@@ -8,6 +8,7 @@
 
 #import <Cocoa/Cocoa.h>
 #import "MSchemaDataSource.h"
+#import "mxUtils.h"
 
 NSString *MSchemaItemPboardType= @"MSchemaItemPboardType";
 
@@ -180,6 +181,9 @@ NSString *MSchemaItemPboardType= @"MSchemaItemPboardType";
   MSchemaItem *item= nil;
   switch (type)
   {
+    case MCatalogItemType:
+      return NULL;
+      break;
     case MSchemaItemType:
       item= self;
       break;
@@ -334,14 +338,14 @@ NSString *MSchemaItemPboardType= @"MSchemaItemPboardType";
     _leafType= leaf;
 	_children= [[NSMutableDictionary alloc] init];
 	_items= [[NSMutableDictionary alloc] init];  
-	
-	_catalogIcon= [[NSImage imageNamed:@"16x16_Catalog.png"] retain];
-	_schemaIcon= [[NSImage imageNamed:@"myx_schema_16x16.png"] retain];
-	_tableIcon= [[NSImage imageNamed:@"myx_schema_table_16x16.png"] retain];
-    _viewIcon= [[NSImage imageNamed:@"myx_schema_view_16x16.png"] retain];
-    _spIcon= [[NSImage imageNamed:@"myx_schema_sp_16x16.png"] retain];
-	_columnIcon= [[NSImage imageNamed:@"16x16_Field.png"] retain];
-	_keyIcon= [[NSImage imageNamed:@"16x16_KeyColumn.png"] retain];	
+
+	_catalogIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"16x16_Catalog.png") retain];
+	_schemaIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"myx_schema_16x16.png") retain];
+	_tableIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"myx_schema_table_16x16.png") retain];
+    _viewIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"myx_schema_view_16x16.png") retain];
+    _spIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"myx_schema_sp_16x16.png") retain];
+	_columnIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"16x16_Field.png") retain];
+	_keyIcon= [MXGetImageFromBundle([NSBundle bundleForClass:[self class]], @"16x16_KeyColumn.png") retain];	
   }    
   return self;
 }
@@ -767,7 +771,7 @@ objectValueForTableColumn:(NSTableColumn *)tableColumn
 {
   MSchemaItem *item= [items lastObject];
   
-  [pboard declareTypes:[NSArray arrayWithObject:MSchemaItemPboardType] owner:nil];
+  [pboard declareTypes:[NSArray arrayWithObjects:MSchemaItemPboardType,nil] owner:nil];
   
   _draggedItem= item;
   
