@@ -261,6 +261,40 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   }
 }
 
+
+static int addColumnToIndex(MYX_DBM_INDEX_DATA *idata, const char *colname)
+{
+  int i= idata->columns_num++;
+  idata->columns= g_realloc(idata->columns, sizeof(MYX_DBM_INDEX_COLUMN_DATA)*idata->columns_num);
+  memset(&idata->columns[i], 0, sizeof(MYX_DBM_INDEX_COLUMN_DATA));
+  idata->columns[i].name= g_strdup(colname);  
+  
+  return idata->columns_num-1;
+}
+
+
+static void removeColumnFromIndex(MYX_DBM_INDEX_DATA *idata, const char *colname)
+{
+  int i;
+  
+  for (i= 0; i < idata->columns_num; i++)
+  {
+    if (strcmp2(idata->columns[i].name, colname)==0)
+      break;
+  }
+  
+  if (idata->columns_num == i)
+    return;
+  
+  g_free(idata->columns[i].name);
+  g_free(idata->columns[i].len);
+  g_free(idata->columns[i].value_order);
+  memmove(idata->columns+i, idata->columns+i+1,
+          sizeof(MYX_DBM_INDEX_COLUMN_DATA)*(idata->columns_num-i-1));
+  idata->columns_num--;
+}
+
+
 - (void)addIndexColumn:(id)sender
 {
   int index= [indexList selectedRow];
@@ -272,17 +306,25 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   if (index < 0)
     return;
   
+  idata= _table->indices+index;
+  
   if (column < 0)
     colname= "";
   else
+  {
     colname= _table->columns[column].name;
-  
-  idata= _table->indices+index;
+    
+    // if this is the PRIMARY index, then set the column's PRIMARY flag
+    if (strcmp2(idata->name, "PRIMARY")==0)
+    {
+      _table->columns[column].primary_key= 1;
+      
+      [self showColumn:_table->columns+column];
+    }
+  }
 
-  i= idata->columns_num++;
-  idata->columns= g_realloc(idata->columns, sizeof(MYX_DBM_INDEX_COLUMN_DATA)*idata->columns_num);
-  memset(&idata->columns[i], 0, sizeof(MYX_DBM_INDEX_COLUMN_DATA));
-  idata->columns[i].name= g_strdup(colname);
+  i= addColumnToIndex(idata, colname);
+  
   [indColumnList reloadData];
   [indColumnList selectRow:i byExtendingSelection:NO];  
   [indColumnList editColumn:0 row:i withEvent:nil select:YES];
@@ -299,12 +341,21 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
 
   idata= _table->indices+index;
 
-  g_free(idata->columns[icolumn].name);
-  g_free(idata->columns[icolumn].len);
-  g_free(idata->columns[icolumn].value_order);
-  memmove(idata->columns+icolumn, idata->columns+icolumn+1,
-          sizeof(MYX_DBM_INDEX_COLUMN_DATA)*(idata->columns_num-icolumn-1));
-  idata->columns_num--;
+  // if this is the PRIMARY index, then set the column's PRIMARY flag
+  if (strcmp2(idata->name, "PRIMARY")==0)
+  {
+    int column;
+    for (column= 0; column < _table->columns_num; column++)
+    {
+      if (strcmp2(_table->columns[column].name, idata->columns[icolumn].name)==0)
+        break;
+    }
+    _table->columns[column].primary_key= 0;
+    
+    [self showColumn:_table->columns+column];
+  }
+  
+  removeColumnFromIndex(idata, idata->columns[icolumn].name);
 
   [indColumnList reloadData];
 }
@@ -431,11 +482,11 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   }
   else
   {
-    [indNameText setStringValue:@""];
+    [fkNameText setStringValue:@""];
     [fkOnDeletePop selectItemAtIndex:0];
     [fkOnUpdatePop selectItemAtIndex:0];
-	if ([fkRefTablePop numberOfItems]>0)
-	  [fkRefTablePop selectItemAtIndex:0];
+    if ([fkRefTablePop numberOfItems]>0)
+      [fkRefTablePop selectItemAtIndex:0];
   }  
   [fkColumnList reloadData];  
 }
@@ -1142,6 +1193,8 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   }
   
   [fkContainer setEnabledRecursive:fkEnabled];
+  if (fkEnabled)
+    [fkBox setEnabledRecursive:NO];
 }
 
 @end
@@ -1328,20 +1381,41 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   }
   else if ([sender isMemberOfClass:[NSMatrix class]])
   {
+    int i;
+    MYX_DBM_INDEX_DATA *idata= NULL;
+    
     _table->columns[column].primary_key= [colPKCheck state] == NSOnState ? 1 : 0;
     if (COLUMN_TYPE(_table->columns[column]) != MYX_DBM_DTG_NUMERIC)
       _table->columns[column].auto_inc= 0;
     else
       _table->columns[column].auto_inc= [colAIncCheck state] == NSOnState ? 1 : 0;
 
+    // find the PRIMARY index
+    for (i= 0; i < _table->indices_num; i++)
+    {
+      if (strcmp2(_table->indices[i].name, "PRIMARY")==0)
+      {
+        idata= _table->indices+i;
+        break;
+      }
+    }
+
     if (_table->columns[column].primary_key)
     {
-      _table->columns[column].not_null= 1;
+      addColumnToIndex(idata, _table->columns[column].name);
+      [indColumnList reloadData];
+      
       [colNNullCheck setState:NSOnState];
+      
+      _table->columns[column].not_null= 1;
     }
     else
+    {
+      removeColumnFromIndex(idata, _table->columns[column].name);
+      [indColumnList reloadData];
+      
       _table->columns[column].not_null= [colNNullCheck state] == NSOnState ? 1 : 0;
-    
+    }
     [colNNullCheck setEnabled:_table->columns[column].primary_key?NO:YES];
     
     [columnList reloadData];
