@@ -9,11 +9,11 @@
 #import <Cocoa/Cocoa.h>
 
 
-@interface MTableView : NSTableView {
-  SEL _deleteAction;
+@interface MTableView : NSTableView 
+{
+  BOOL _tooltips;
 }
 
 - (void)keyDown:(NSEvent *)event;
-- (void)setDeleteAction:(SEL)action;
 
 @end
