@@ -13,8 +13,10 @@
 
 #include "myx_grt_public_interface.h"
 #include "myx_grt_builtin_module_public_interface.h"
+#include "myx_grt_private.h"
+#include "myx_grt_python.h"
 #include "myx_grt_lua.h"
-#include "myx_grt_java.h"
+//#include "myx_grt_java.h"
 
 
 #define FMT(s, ...) [NSString stringWithFormat:s,##__VA_ARGS__]
@@ -211,14 +213,14 @@ static MYX_GRT_VALUE *createArglist(NSArray *args)
     case MYX_GRT_VALIDATION_ERROR: return @"Validation error";
     case MYX_GRT_FUNCTION_CALL_ERROR: return @"Function call error";
     case MYX_GRT_MODULE_INIT_ERROR: return @"Module init error";
-    case MYX_GRT_BAD_MODULE: @"Bad module";
+    case MYX_GRT_BAD_MODULE: return @"Bad module";
     case MYX_GRT_UNKNOWN_MODULE_TYPE: return @"Unknown module type";
     
     case MYX_GRT_JAVA_NOT_FOUND: return @"Java Runtime not found";
     case MYX_GRT_JAVA_REGISTRY_CORRUPTED: return @"Java registry corrupted";
     case MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED: return @"JRE cannot be loaded";
-    default: return nil;
   }
+  return nil;
 }
 
 
@@ -303,6 +305,8 @@ static int process_grt_input(const char *caption, MYX_GRT_INPUT_OPTIONS options,
   myx_grt_set_message_callback(_grt, self, process_grt_message);
   myx_grt_set_input_callback(_grt, self, process_grt_input);
   
+  myx_grt_setup_shell(_grt, MYX_GRT_SHELL_LUA);
+  
   myx_grt_shell_print_welcome(_grt);
   
   [self scanStructsInPath:FMT(@"%@/xml",resourcePath)];
@@ -387,7 +391,7 @@ static int process_grt_input(const char *caption, MYX_GRT_INPUT_OPTIONS options,
     [self outText:FMT(@"Error initializing ObjC module loader (%i)\n",error)];
   
   
-  myx_grt_init_lua_shell(_grt);
+  myx_grt_shell_init(_grt);
 }
 
 
@@ -538,7 +542,7 @@ static int process_grt_input(const char *caption, MYX_GRT_INPUT_OPTIONS options,
 {
   MYX_GRT_SHELL_COMMAND error;
 
-  error= myx_grt_lua_shell_execute(_grt, [command UTF8String]);
+  error= myx_grt_shell_execute(_grt, [command UTF8String]);
   //if (error != MYX_GRT_SHELL_
   
   myx_grt_messages_stack_flush(_grt, 0);
@@ -546,7 +550,7 @@ static int process_grt_input(const char *caption, MYX_GRT_INPUT_OPTIONS options,
 
 - (NSString*)shellPrompt
 {
-  return [NSString stringWithUTF8String:myx_grt_lua_shell_get_prompt(_grt)];
+  return [NSString stringWithUTF8String:myx_grt_shell_get_prompt(_grt)];
 }
 
 
