@@ -254,7 +254,7 @@ NSString *MGRTTableRowPboardType= @"MGRTTableRowPboardType";
   else if (obj == colNameText)
   {
     MGRTValue column(_tableData->getColumn([columnList selectedRow]));
-    column.set("name", [str UTF8String]);
+    _tableData->setColumnName(column, [str UTF8String]);
     
     [self columnListChanged];
     [columnList reloadData];
@@ -288,10 +288,9 @@ NSString *MGRTTableRowPboardType= @"MGRTTableRowPboardType";
 {  
   if (column)
   {
-    char *tmp;
     [colNameText setStringValue:NSStr((const char*)(*column)["name"])];
     
-    [colTypeCombo setStringValue:NSStr(tmp= _tableData->formatColumnType(*column))]; g_free(tmp);
+    [colTypeCombo setStringValue:NSStr(_tableData->formatColumnType(*column).c_str())];
     [colDefaultNull setState:(int)(*column)["defaultValueIsNull"] != 0 ? NSOnState : NSOffState];
 
     if ((int)(*column)["defaultValueIsNull"] != 0)
@@ -355,7 +354,7 @@ NSString *MGRTTableRowPboardType= @"MGRTTableRowPboardType";
     int index= [indexList selectedRow];
     if (index < 0)
       return 0;
-    return _tableData->getIndex(index)["columns"].count();
+    return _tableData->indexColumnCount(_tableData->getIndex(index));
   }
   else if (aTableView == fkList)
   {
@@ -366,7 +365,7 @@ NSString *MGRTTableRowPboardType= @"MGRTTableRowPboardType";
     int index= [fkList selectedRow];
     if (index < 0)
       return 0;
-    return _tableData->getFK(index)["columns"].count(); 
+    return _tableData->fkColumnCount(_tableData->getFK(index));
   }
   return 0;
 }
@@ -440,7 +439,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     if ([[aTableColumn identifier] isEqualTo:@"name"])
       return NSStr((const char*)column["name"]);
     else if ([[aTableColumn identifier] isEqualTo:@"type"])
-      return NSStr(_tableData->formatColumnType(column));
+      return NSStr(_tableData->formatColumnType(column).c_str());
     else if ([[aTableColumn identifier] isEqualTo:@"nnull"])
     {
       if (_tableData->columnIsPK(column))
@@ -457,21 +456,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     }
     else if ([[aTableColumn identifier] isEqualTo:@"flags"])
     {
-      int i, j;
-      NSMutableString *ms= [[[NSMutableString alloc] init] autorelease];
-      MGRTValue types(_tableData->getColumnFlags(column));
-      j= 0;
-      for (i= 0; i < types.count(); i++)
-      {
-        if (_tableData->columnFlagState(column, types[i].asString()))
-        {
-          if (j++ > 0)
-            [ms appendFormat:@",%s",types[i].asString()];
-          else
-            [ms appendFormat:@"%s",types[i].asString()];
-        }
-      }
-      return ms;
+      return NSStr(_tableData->getEnabledColumnFlags(column).c_str());
     }
     else if ([[aTableColumn identifier] isEqualTo:@"default"])
     {
@@ -600,7 +585,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
         
     if ([[aTableColumn identifier] isEqualTo:@"name"])
     {
-      column.set("name", [anObject UTF8String]);
+      _tableData->setColumnName(column, [anObject UTF8String]);
       [self columnListChanged];
     }
     else if ([[aTableColumn identifier] isEqualTo:@"type"])
@@ -629,24 +614,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     }  
     else if ([[aTableColumn identifier] isEqualTo:@"flags"])
     {
-      MGRTValue types(_tableData->getColumnFlags(column));
-      int i, t;
-      char **tokens= g_strsplit([anObject UTF8String], ",", 0);
-      
-      for (i= 0; i < types.count(); i++)
-      {
-        bool on= false;
-        
-        for (t= 0; tokens[t]; t++)
-        {
-          if (strcasecmp(types[i].asString(), tokens[t])==0)
-          {
-            on= true;
-            break;
-          }
-        }
-        _tableData->setColumnFlagState(column, types[i].asString(), on);
-      }      
+      _tableData->setEnabledColumnFlags(column, [anObject UTF8String]);
     }
     else if ([[aTableColumn identifier] isEqualTo:@"default"])
     {
@@ -847,8 +815,8 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   if (tableView == columnList)
   {
-    if (row >= _tableData->columnCount())
-      row= _tableData->columnCount()-1;
+    if (row > _tableData->columnCount())
+      row= _tableData->columnCount();
     
     _tableData->moveColumn(_draggedRow, row);
     
@@ -984,21 +952,34 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 - (BOOL)commitChanges
 {
   MGRTValue table(_tableData->value());
-  
+  static char *pack_key_values[]= {
+    "", "0", "1"
+  };
+  static char *row_format_values[]= {
+    "", "dynamic", "fixed", "compressed"
+  };
+  static char *merge_insert_values[]= {
+    "no", "last", "first"
+  };
+  static char *raid_type_values[]= {
+    "", "striped"
+  };
+
 #define ASSIGNSTR(v,s) table.set(v, [s UTF8String])
-  
+#define OPTIONVALUE(v,s,options) table.set(v, options[[s indexOfSelectedItem]])
+
   _tableData->setName([[tableNameText stringValue] UTF8String]);
   
   _tableData->setComment([[[descrText textStorage] string] UTF8String]);
   
   // Misc. Options
-  ASSIGNSTR("packKeys", [[miscOpView viewWithTag:1] titleOfSelectedItem]);
+  OPTIONVALUE("packKeys", [miscOpView viewWithTag:1], pack_key_values);
   ASSIGNSTR("password", [[miscOpView viewWithTag:2] stringValue]);
   ASSIGNSTR("nextAutoInc", [[miscOpView viewWithTag:3] stringValue]);
   table.set("delayKeyWrite", [[miscOpView viewWithTag:4] state] == NSOnState);
   
   // Row Options
-  ASSIGNSTR("rowFormat", [[rowOpView viewWithTag:1] titleOfSelectedItem]);
+  OPTIONVALUE("rowFormat", [rowOpView viewWithTag:1], row_format_values);
   table.set("checksum", [[rowOpView viewWithTag:2] state] == NSOnState);
   ASSIGNSTR("avgRowLength", [[rowOpView viewWithTag:3] stringValue]);
   ASSIGNSTR("minRows", [[rowOpView viewWithTag:4] stringValue]);
@@ -1010,10 +991,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   
   // Merge Options
   ASSIGNSTR("mergeUnion", [[mergeOpView viewWithTag:1] stringValue]);
-  ASSIGNSTR("mergeInsert", [[mergeOpView viewWithTag:2] titleOfSelectedItem]);
+  OPTIONVALUE("mergeInsert", [mergeOpView viewWithTag:2], merge_insert_values);
   
   // Table RAID stuff
-  ASSIGNSTR("raidType", [[raidOpView viewWithTag:1] titleOfSelectedItem]);
+  OPTIONVALUE("raidType", [raidOpView viewWithTag:1], raid_type_values);
   ASSIGNSTR("raidChunks", [[raidOpView viewWithTag:2] stringValue]);
   ASSIGNSTR("raidChunkSize", [[raidOpView viewWithTag:3] stringValue]);
 
