@@ -7,7 +7,9 @@
 
 #include <myx_public_interface.h>
 
+#import <MySQLToolsCommon/MConnectionInfo.h>
 #import <MySQLToolsCommon/mxUtils.h>
+#import <MySQLToolsCommon/NSString_extras.h>
 
 #import "MAccessoryScrollView.h"
 #import "MSplitView.h"
@@ -28,7 +30,7 @@ typedef struct {
 
 @interface MessageDS : NSObject
 {
-@public
+  @public
   MessageItem *items;
   int itemCount;
   NSImage *images[3];
@@ -37,10 +39,10 @@ typedef struct {
 - (int)numberOfRowsInTableView:(NSTableView *)aTableView;
 - (id)tableView:(NSTableView *)aTableView 
 objectValueForTableColumn:(NSTableColumn *)aTableColumn
-			row:(int)rowIndex;
+            row:(int)rowIndex;
 - (void)addError:(int)error
           ofType:(MYX_QUERY_ERROR_LEVEL)level
-		 message:(NSString*)message
+         message:(NSString*)message
            atRow:(unsigned int)row
           column:(unsigned int)column;
 - (void)reset;
@@ -54,8 +56,8 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   self= [super init];
   if (self)
   {
-	items= NULL;
-	itemCount= 0;
+    items= NULL;
+    itemCount= 0;
     images[0]= MXGetCommonImage(MXMiniNoticeImage);
     images[1]= MXGetCommonImage(MXMiniWarningImage);
     images[2]= MXGetCommonImage(MXMiniErrorImage);
@@ -68,7 +70,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   int i;  
   for (i= 0; i < itemCount; i++)
-	[items[i].message release];
+    [items[i].message release];
   g_free(items);
   items= 0;
   [super dealloc];
@@ -83,18 +85,18 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 
 - (id)tableView:(NSTableView *)aTableView 
 objectValueForTableColumn:(NSTableColumn *)aTableColumn
-			row:(int)rowIndex
+            row:(int)rowIndex
 {
   if ([[aTableColumn identifier] isEqualTo:@"error"])
-	return [NSNumber numberWithInt:items[rowIndex].error];
+    return [NSNumber numberWithInt:items[rowIndex].error];
   else
-	return items[rowIndex].message;
+    return items[rowIndex].message;
 }
 
 
 - (void)addError:(int)error
           ofType:(MYX_QUERY_ERROR_LEVEL)level
-		 message:(NSString*)message
+         message:(NSString*)message
            atRow:(unsigned int)row
           column:(unsigned int)column
 {
@@ -134,7 +136,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   int i;
   for (i= 0; i < itemCount; i++)
-	[items[i].message release];
+    [items[i].message release];
   g_free(items);
   itemCount= 0;
   items= NULL;
@@ -152,34 +154,34 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   if (_dataSource)
   {
-	if ([_dataSource editing])
-	{
-	  [[view viewWithTag:50] setEnabled:NO];
-	  [[view viewWithTag:51] setEnabled:YES];
-	  [[view viewWithTag:52] setEnabled:YES];
-	}
-	else
-	{
-	  if ([_dataSource isEditable])
-		[[view viewWithTag:50] setEnabled:YES];
-	  else
-		[[view viewWithTag:50] setEnabled:NO];
-	  [[view viewWithTag:51] setEnabled:NO];
-	  [[view viewWithTag:52] setEnabled:NO];
-	}
-	[[view viewWithTag:53] setEnabled:YES];
-	[[view viewWithTag:54] setEnabled:YES];
-	[[view viewWithTag:55] setEnabled:NO]; // SEARCH not supported yet
+    if ([_dataSource editing])
+    {
+      [[view viewWithTag:50] setEnabled:NO];
+      [[view viewWithTag:51] setEnabled:YES];
+      [[view viewWithTag:52] setEnabled:YES];
+    }
+    else
+    {
+      if ([_dataSource isEditable])
+        [[view viewWithTag:50] setEnabled:YES];
+      else
+        [[view viewWithTag:50] setEnabled:NO];
+      [[view viewWithTag:51] setEnabled:NO];
+      [[view viewWithTag:52] setEnabled:NO];
+    }
+    [[view viewWithTag:53] setEnabled:YES];
+    [[view viewWithTag:54] setEnabled:YES];
+    [[view viewWithTag:55] setEnabled:NO]; // SEARCH not supported yet
   }
   else
   {
-	[[view viewWithTag:50] setEnabled:NO];
-	[[view viewWithTag:51] setEnabled:NO];
-	[[view viewWithTag:52] setEnabled:NO];
-	
-	[[view viewWithTag:53] setEnabled:NO];
-	[[view viewWithTag:54] setEnabled:NO];
-	[[view viewWithTag:55] setEnabled:NO];	
+    [[view viewWithTag:50] setEnabled:NO];
+    [[view viewWithTag:51] setEnabled:NO];
+    [[view viewWithTag:52] setEnabled:NO];
+    
+    [[view viewWithTag:53] setEnabled:NO];
+    [[view viewWithTag:54] setEnabled:NO];
+    [[view viewWithTag:55] setEnabled:NO];	
   }
 }
 
@@ -191,53 +193,63 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     [splitView resizeSubview:[[messageTable superview] superview] toSize:0];
 }
 
-- (id)initWithConnectionTo:(MYX_USER_CONNECTION*)info
+- (id)initWithConnectionTo:(MConnectionInfo*)info
 {
   self= [super init];
   if (self)
   {
     int i;
+    MYX_USER_CONNECTION *conn;
     
-	if (![NSBundle loadNibNamed:@"ResultSet" owner:self])
-	{
-	  NSLog(@"Could not load nib ResultSet.nib");
-	  [self release];
-	  return nil;
-	}
+    if (![NSBundle loadNibNamed:@"ResultSet" owner:self])
+    {
+      NSLog(@"Could not load nib ResultSet.nib");
+      [self release];
+      return nil;
+    }
     
-	_mysql= myx_mysql_init();
-	if (!_mysql)
-	{
-	  NSLog(@"could not create MySQL connection");
-	  [self release];
-	  return nil;
-	}
-	if (myx_connect_to_instance(info, _mysql) < 0)
-	{
-	  MXRunAlertPanelWithMySQLError(@"Error",@"Could not connect to MySQL instance.",_mysql);
-	  [self release];
+    _mysql= myx_mysql_init();
+    if (!_mysql)
+    {
+      NSLog(@"could not create MySQL connection");
+      [self release];
+      return nil;
+    }
+    
+    conn= [info createUserConnection];
+    if (myx_connect_to_instance(conn, _mysql) < 0)
+    {
+      MXRunAlertPanelWithMySQLError(@"Error",@"Could not connect to MySQL instance.",_mysql);
+      myx_free_user_connection_content(conn);
+      g_free(conn);
+      [self release];
       myx_mysql_close(_mysql);
-	  return nil;
-	}
+      return nil;
+    }
+    myx_free_user_connection_content(conn);
+    g_free(conn);
     
-	_messageDS= [[MessageDS alloc] init];
-	[messageTable setDataSource:_messageDS];
+    // limit memory usage by resultsets to 50% of physical mem
+    myx_mysql_limit_resultset_size(_mysql, (get_physical_memory_size()/(1024*1024LL))/2);
+    
+    _messageDS= [[MessageDS alloc] init];
+    [messageTable setDataSource:_messageDS];
     [messageTable setDelegate:_messageDS];
-
+    
     [[NSNotificationCenter defaultCenter] addObserver:self
                                              selector:@selector(selectedError:) 
                                                  name:NSTableViewSelectionDidChangeNotification
                                                object:messageTable];
-
-	[self updateButtonBar];
-
-	_statusColors[MESUnchanged] = [[NSColor colorWithDeviceRed:0.92 green:0.92 blue:1.0 alpha:1.0] retain];
-	_statusColors[MESChanged] = [[NSColor colorWithDeviceRed:0.8 green:0.8 blue:1.0 alpha:1.0] retain];
+    
+    [self updateButtonBar];
+    
+    _statusColors[MESUnchanged] = [[NSColor colorWithDeviceRed:0.92 green:0.92 blue:1.0 alpha:1.0] retain];
+    _statusColors[MESChanged] = [[NSColor colorWithDeviceRed:0.8 green:0.8 blue:1.0 alpha:1.0] retain];
     _statusColors[MESPlaceHolder] = [[NSColor colorWithDeviceRed:0.95 green:1.0 blue:0.95 alpha:1.0] retain];
-	_statusColors[MESAdded] = [[NSColor colorWithDeviceRed:0.8 green:1.0 blue:0.8 alpha:1.0] retain];
-	_statusColors[MESDeleted] = [[NSColor colorWithDeviceRed:1.0 green:0.8 blue:0.8 alpha:1.0] retain];
-  
-
+    _statusColors[MESAdded] = [[NSColor colorWithDeviceRed:0.8 green:1.0 blue:0.8 alpha:1.0] retain];
+    _statusColors[MESDeleted] = [[NSColor colorWithDeviceRed:1.0 green:0.8 blue:0.8 alpha:1.0] retain];
+    
+    
     _diffColors[MCSMatch] = nil;
     _diffColors[MCSDiffers] = [[NSColor colorWithDeviceRed:0xaa/255.0
                                                      green:0xbb/255.0
@@ -251,13 +263,13 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
                                                          green:0xaa/255.0
                                                           blue:0xaa/255.0
                                                          alpha:1.0] retain];
-      
-      
+    
+    
     _viewers= [[NSMutableArray alloc] init];
     
     [splitView setInactiveColor:[table gridColor]];
     [splitView setDividerThickness:2.0];
-
+    
     [self setMessagesShown:NO];
     [self changeSplitDirection:MQDNone];
     
@@ -265,12 +277,12 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     
     for (i= 0; i < 7; i++)
       _editBarSizes[i]= [[view viewWithTag:editBarTags[i]] frame].size;
-  
-//#ifdef MAC_OS_X_VERSION_10_4
-//    [table setColumnAutoresizingStyle:NSTableViewNoColumnAutoresizing];
-//#else
-//    [table setAutoresizesAllColumnsToFit:NO];
-//#endif
+    
+    //#ifdef MAC_OS_X_VERSION_10_4
+    //    [table setColumnAutoresizingStyle:NSTableViewNoColumnAutoresizing];
+    //#else
+    //    [table setAutoresizesAllColumnsToFit:NO];
+    //#endif
   }
   return self;
 }
@@ -280,12 +292,12 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   int i;
   for (i= 0; i < 5; i++)
-	[_statusColors[i] release];
+    [_statusColors[i] release];
   for (i= 0; i < 4; i++)
-	[_diffColors[i] release];
+    [_diffColors[i] release];
   [_query release];
   if (_mysql)
-	myx_mysql_close(_mysql);
+    myx_mysql_close(_mysql);
   [_dataSource release];
   
   [[NSNotificationCenter defaultCenter] removeObserver:self];
@@ -317,11 +329,11 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     
     [button setTarget:self];
     [button setAction:@selector(splitH:)];
-
+    
     [sview addSubview:button];
     [sview addVerticalAccessoryView:button];
   }
-
+  
   if (newDir == MQDNone || newDir == MQDVertical)
   {
     button= [[[NSButton alloc] initWithFrame:NSMakeRect(0,0,size,size)] autorelease];
@@ -329,14 +341,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     [button setBordered:NO];
     [[button cell] setImagePosition:NSImageOnly];
     [button setImage: [NSImage imageNamed: @"sbar_splith.png"]];
-
+    
     [button setTarget:self];
     [button setAction:@selector(splitV:)];
-
+    
     [sview addSubview:button];
     [sview addHorizontalAccessoryView:button];
   }
-
+  
   if (_closeable)
   {
     button= [[[NSButton alloc] initWithFrame:NSMakeRect(0,0,size,size)] autorelease];
@@ -350,19 +362,19 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     
     [button setTarget:self];
     [button setAction:@selector(unsplit:)];
-
+    
     [sview addSubview:button];
     if (newDir == MQDHorizontal)
       [sview addVerticalAccessoryView:button];
     else
       [sview addHorizontalAccessoryView:button];
   }
-
+  
   if (newDir == MQDNone || newDir == MQDHorizontal)
     [self setCompact:NO];
   else
     [self setCompact:YES];
-
+  
   _direction= newDir;
 }
 
@@ -415,7 +427,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     return;
   
   pboard= [NSPasteboard generalPasteboard];
-    
+  
   if (rs->columns[column].column_type == MYX_RSCT_BLOB)
   {
     NSData *data= [NSData dataWithBytes:rs->rows[row].fields[column].value
@@ -483,7 +495,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
                        row:row];
     
     [table setNeedsDisplay];
-
+    
     [statusLabel setStringValue:@"Loaded file into field."];
   }
 }
@@ -552,7 +564,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   [_dataSource setEditing:NO];
   [self updateButtonBar];
-
+  
   [statusLabel setStringValue:@"Saving changes..."];
   
   [_dataSource preCommit];
@@ -573,7 +585,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   if ([_dataSource numberOfRowsInTableView:table] > 0)
   {
-	[table selectRow:0 byExtendingSelection:NO];
+    [table selectRow:0 byExtendingSelection:NO];
     [table scrollRowToVisible:0];
   }
 }
@@ -582,7 +594,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 {
   if ([_dataSource numberOfRowsInTableView:table] > 0)
   {
-	[table selectRow:[_dataSource numberOfRowsInTableView:table]-1
+    [table selectRow:[_dataSource numberOfRowsInTableView:table]-1
 byExtendingSelection:NO];
     [table scrollRowToVisible:[_dataSource numberOfRowsInTableView:table]-1];
   }
@@ -659,7 +671,7 @@ byExtendingSelection:NO];
     [table setBackgroundColor:[table gridColor]];
   [splitView setActive:flag];
   [splitView setNeedsDisplay:YES];
-
+  
   if (flag)
     [_delegate performSelector:@selector(resultSetViewDidGetFocus:) withObject:self];
 }
@@ -682,14 +694,20 @@ byExtendingSelection:NO];
   MYX_RESULTSET *rs= (MYX_RESULTSET*)[[info objectForKey:@"resultset"] pointerValue];
   MYX_LIB_ERROR error= (MYX_LIB_ERROR)[[info objectForKey:@"error"] intValue];
   
-  //XXX display errors and warnings
-  [_messageDS reset];
-  
   [table reloadData];
-
+  
   if (error == MYX_STOP_EXECUTION)
     [statusLabel setStringValue:[NSString stringWithFormat:@"Interrupted. %i rows fetched.", 
       rs->rows_num]];
+  else if (error == MYX_MEMORY_LIMIT_EXCEEDED)
+  {
+    [statusLabel setStringValue:[NSString stringWithFormat:@"Memory usage limit exceeded. %i rows fetched", 
+      rs->rows_num]];
+    NSRunAlertPanel(@"Memory Limit Exceeded",
+                    [NSString stringWithFormat:@"The query resultset has exceeded the memory size limit of %@B and was interrupted. It is advisable to use the LIMIT keyword in SELECT queries on large tables.",
+                      [NSString stringWithMultNumber:myx_mysql_get_resultset_size_limit(_mysql)*1024*1024LL]],
+                    @"OK", nil, nil);
+  }
   else
     [statusLabel setStringValue:[NSString stringWithFormat:@"%i rows fetched.", 
       rs->rows_num]];
@@ -698,6 +716,14 @@ byExtendingSelection:NO];
                   withObject:self
                   withObject:(id)NO];
   
+  if (myx_check_whether_commits_transaction(_mysql, [[info objectForKey:@"query"] UTF8String]))
+  {
+    _transactionOpen= NO;
+    [_delegate performSelector:@selector(resultSetViewTransactionEnded:commited:)
+                    withObject:self
+                    withObject:(id)NO];
+  }
+  
   [self updateButtonBar];  
 }
 
@@ -712,7 +738,7 @@ byExtendingSelection:NO];
   NSFont *font;
   
   _busy= NO;
-
+  
   if (!rs)
   {
     MYX_MYSQL_ERROR_MSGS *errors;
@@ -760,19 +786,19 @@ byExtendingSelection:NO];
       [self setMessagesShown:YES];      
     }
     [_delegate performSelector:@selector(resultSetViewFinishedQuery:error:) withObject:self withObject:(id)YES];
-	return;
+    return;
   }
   [_dataSource release];
-
+  
   _dataSource= [[MQResultSetDataSource alloc] initWithResultSet:rs];
   while ([columns count] > 0)
-	[table removeTableColumn:[columns objectAtIndex:0]];
+    [table removeTableColumn:[columns objectAtIndex:0]];
   
   column=  [[NSTableColumn alloc] initWithIdentifier:@"indicator"];
   [column setDataCell:[[[MQIndicatorCell alloc] init] autorelease]];
-  [column setWidth:40];
-  [column setMinWidth:40];
-  [column setMaxWidth:40];
+  [column setWidth:30];
+  [column setMinWidth:30];
+  [column setMaxWidth:30];
   [column setEditable:NO];
   [[column headerCell] setTitle:@""];
   [table addTableColumn:column];
@@ -781,24 +807,25 @@ byExtendingSelection:NO];
   
   for (i= 0; i < rs->columns_num; i++)
   {
-	column= [[NSTableColumn alloc] initWithIdentifier:[NSNumber numberWithInt:i]];
-	NSString *title= [NSString stringWithUTF8String:rs->columns[i].name];
-	float width= 0.0;
-	float twidth;
+    column= [[NSTableColumn alloc] initWithIdentifier:[NSNumber numberWithInt:i]];
+    NSString *title= [NSString stringWithUTF8String:rs->columns[i].name];
+    float width= 0.0;
+    float twidth;
     unsigned int j;
-
+    
     [column setDataCell:[[[MQResultSetCell alloc] init] autorelease]];
     [[column dataCell] setEditable:YES];
-   
-	switch (rs->columns[i].column_type)
-	{
-	  case MYX_RSCT_INTEGER:
-		width= [font widthOfString:@"0000000000"];
-		break;
-	  case MYX_RSCT_FLOAT:
-		width= [font widthOfString:@"0000000000"];
-		break;
-	  case MYX_RSCT_STRING:
+    
+    switch (rs->columns[i].column_type)
+    {
+      case MYX_RSCT_DECIMAL:
+      case MYX_RSCT_INTEGER:
+        width= [font widthOfString:@"0000000000"];
+        break;
+      case MYX_RSCT_FLOAT:
+        width= [font widthOfString:@"0000000000"];
+        break;
+      case MYX_RSCT_STRING:
         width= 0;
         for (j= 0; j < rs->rows_num; j++)
         {
@@ -806,58 +833,58 @@ byExtendingSelection:NO];
           if (width < w)
             width= w;
         }
-        if (width == 0.0)
-        {
-          width= [font widthOfString:@"XYZW"] * rs->columns[i].type_size / 4;
-        }
-        else
-          width+= 4.0;
+          if (width == 0.0)
+          {
+            width= [font widthOfString:@"XYZW"] * rs->columns[i].type_size / 4;
+          }
+          else
+            width+= 4.0;
         if (width < 40.0)
           width= 40.0;
-		break;
-	  case MYX_RSCT_DATE:
-		width= [font widthOfString:@"00/00/0000"];
-		break;
-	  case MYX_RSCT_TIME:
-		width= [font widthOfString:@"00:00:00.0000"];
-		break;
-	  case MYX_RSCT_DATETIME:
-		width= [font widthOfString:@"00/00/0000  00:00:00.0000"];
-		break;
-	  case MYX_RSCT_BLOB:
+          break;
+      case MYX_RSCT_DATE:
+        width= [font widthOfString:@"00/00/0000"];
+        break;
+      case MYX_RSCT_TIME:
+        width= [font widthOfString:@"00:00:00.0000"];
+        break;
+      case MYX_RSCT_DATETIME:
+        width= [font widthOfString:@"00/00/0000  00:00:00.0000"];
+        break;
+      case MYX_RSCT_BLOB:
         [[column dataCell] setIsBlob:YES];
-		width= 50;
-		break;
-	  case MYX_RSCT_TEXT:
-		width= 250;
-		break;
-	  case MYX_RSCT_ENUM:
-		width= 100;
-		break;
-	  case MYX_RSCT_SET:
-		width= 100;
-		break;
-	}
-
+        width= 50;
+        break;
+      case MYX_RSCT_TEXT:
+        width= 250;
+        break;
+      case MYX_RSCT_ENUM:
+        width= 100;
+        break;
+      case MYX_RSCT_SET:
+        width= 100;
+        break;
+    }
+    
     [[column dataCell] setDrawsBackground:YES];
     [[column dataCell] setFont: font];
-
-	twidth= [[[column headerCell] font] widthOfString:title];
-	width= MIN(MAX(width, twidth), 250);
+    
+    twidth= [[[column headerCell] font] widthOfString:title];
+    width= MIN(MAX(width, twidth), 250);
     [[column headerCell] setWraps:NO];
-
-	if (rs->columns[i].table_column && rs->columns[i].table_column->is_pk)
-	  [[column headerCell] setStringValue:[NSString stringWithFormat:@"%@%@",
-		[NSString stringWithUTF8String:"\xe2\x80\xbb"], title]];
-	else
-	  [[column headerCell] setStringValue:title];
-
-	[table addTableColumn:column];
+    
+    if (rs->columns[i].table_column && rs->columns[i].table_column->is_pk)
+      [[column headerCell] setStringValue:[NSString stringWithFormat:@"%@%@",
+        [NSString stringWithUTF8String:"\xe2\x80\xbb"], title]];
+    else
+      [[column headerCell] setStringValue:title];
+    
+    [table addTableColumn:column];
     
     if (rs->rows_num > 0)
       [column setWidth:width];
   }
-
+  
   [table setDataSource:_dataSource];
   
   if (![info objectForKey:@"current"]) // if we're not being called from the progress callback
@@ -879,19 +906,19 @@ byExtendingSelection:NO];
 }
 
 static int fetchProgress(unsigned long current_row_count, 
-						 unsigned long previous_row_count, 
-						 MYX_RESULTSET *result_set, void *user_data)
+                         unsigned long previous_row_count, 
+                         MYX_RESULTSET *result_set, void *user_data)
 {
   NSDictionary *dict;
   
   dict= [NSDictionary dictionaryWithObjectsAndKeys:
-	[NSNumber numberWithLong:current_row_count], @"current",
-	[NSNumber numberWithLong:previous_row_count], @"previous",
-	[NSValue valueWithPointer:result_set], @"resultset",
-	nil];
+    [NSNumber numberWithLong:current_row_count], @"current",
+    [NSNumber numberWithLong:previous_row_count], @"previous",
+    [NSValue valueWithPointer:result_set], @"resultset",
+    nil];
   [(id)user_data performSelectorOnMainThread:@selector(queryProgress:)
-								  withObject:dict
-							   waitUntilDone:NO];
+                                  withObject:dict
+                               waitUntilDone:NO];
   return ((MQResultSetView*)user_data)->_stopQuery; 
 }
 
@@ -933,22 +960,23 @@ static int fetchProgress(unsigned long current_row_count,
   mysql_thread_init();
   
   result= myx_query_execute(mysql, [query UTF8String],
-							TRUE,
-							params, &error_code, self,
-							fetchProgress,
-							NULL,
-							NULL);
-
+                            TRUE,
+                            params, &error_code, self,
+                            fetchProgress,
+                            NULL,
+                            NULL);
+  
   if (params)
     myx_free_stringlist(params);
   
   info= [NSDictionary dictionaryWithObjectsAndKeys:
-	[NSValue valueWithPointer:result], @"resultset",
-	[NSNumber numberWithInt:error_code], @"error", nil];
-
+    [NSValue valueWithPointer:result], @"resultset",
+    [NSNumber numberWithInt:error_code], @"error", 
+    query, @"query", nil];
+  
   [self performSelectorOnMainThread:@selector(queryFinished:)
-						 withObject:info
-					  waitUntilDone:YES];
+                         withObject:info
+                      waitUntilDone:YES];
   
   mysql_thread_end();
 }
@@ -956,6 +984,10 @@ static int fetchProgress(unsigned long current_row_count,
 
 - (void)performQuery:(NSString*)query
 {
+  [_messageDS reset];
+  [messageTable reloadData];
+  [self setMessagesShown:NO];
+  
   [self performQuery:query withParameters:NULL];
 }
 
@@ -969,16 +1001,16 @@ static int fetchProgress(unsigned long current_row_count,
   [_delegate performSelector:@selector(resultSetView:willStartQuery:)
                   withObject:self
                   withObject:query];
- 
+  
   _resultDisplayed= NO;
   _stopQuery= NO;
   [NSApplication detachDrawingThread:@selector(doQuery:)
-							toTarget:self
-						  withObject:[NSDictionary dictionaryWithObjectsAndKeys:
-							[NSValue valueWithPointer:_mysql], @"mysql",
+                            toTarget:self
+                          withObject:[NSDictionary dictionaryWithObjectsAndKeys:
+                            [NSValue valueWithPointer:_mysql], @"mysql",
                             [NSValue valueWithPointer:parameters], @"params",
-							query, @"query",
-							nil]];
+                            query, @"query",
+                            nil]];
   _busy= YES;
 }
 
@@ -986,7 +1018,7 @@ static int fetchProgress(unsigned long current_row_count,
 - (void)applyFinished:(id)errorsVal
 {
   MYX_RS_ACTION_ERRORS *errors= (MYX_RS_ACTION_ERRORS*)[errorsVal pointerValue];
-
+  
   if (errors)
   {
     unsigned int i;
@@ -1003,16 +1035,16 @@ static int fetchProgress(unsigned long current_row_count,
                     column:[_dataSource resultset]->columns - errors->errors[i].action->column];
     }
     myx_query_free_action_errors(errors);
-
+    
     [_dataSource postCommit:YES];
     [messageTable noteNumberOfRowsChanged];
     [self setMessagesShown:YES];
-
+    
     if (errcount > 0)
       [statusLabel setStringValue:@"Error while applying actions."];
     else
       [statusLabel setStringValue:@"Warnings while applying actions."];
-
+    
     [_dataSource setEditing:YES];
     [self updateButtonBar];
   }
@@ -1021,27 +1053,31 @@ static int fetchProgress(unsigned long current_row_count,
     [_dataSource postCommit:NO];
     [_messageDS reset];
     [messageTable reloadData];
-	[statusLabel setStringValue:@"Applied actions."];    
+    [statusLabel setStringValue:@"Applied actions."];    
   }
   [table reloadData];
-
+  
   _busy= NO;
 }
 
+
 - (void)doApply:(NSDictionary*)args
 {
   MYX_RESULTSET *rset= (MYX_RESULTSET*)[[args objectForKey:@"resultset"] pointerValue];
   MYX_RS_ACTION_ERRORS *errors;
   
-  mysql_thread_init();
-  
-  errors= myx_query_apply_actions(rset);
-  
-  [self performSelectorOnMainThread:@selector(applyFinished:)
-						 withObject:[NSValue valueWithPointer:errors]
-					  waitUntilDone:YES];
-  
-  mysql_thread_end();
+  if (rset->actions)
+  {
+    mysql_thread_init();
+    
+    errors= myx_query_apply_actions(rset);
+    
+    [self performSelectorOnMainThread:@selector(applyFinished:)
+                           withObject:[NSValue valueWithPointer:errors]
+                        waitUntilDone:YES];
+    
+    mysql_thread_end();
+  }
 }
 
 
@@ -1054,7 +1090,7 @@ static int fetchProgress(unsigned long current_row_count,
     id part;
     float x= 5.0;
     float width= NSWidth([view frame]);
-
+    
     x= width;
     for (i= 6; i >= 1; i--)
     {
@@ -1082,17 +1118,68 @@ static int fetchProgress(unsigned long current_row_count,
 
 - (void)performApply
 {
-  [statusLabel setStringValue:@"Applying actions..."];
+  if ([_dataSource resultset]->actions)
+  {
+    [statusLabel setStringValue:@"Applying actions..."];
+    
+    //  [_delegate resultSetView:self willStartQuery:query];
+    
+    [NSApplication detachDrawingThread:@selector(doApply:)
+                              toTarget:self
+                            withObject:[NSDictionary dictionaryWithObjectsAndKeys:
+                              [NSValue valueWithPointer:_mysql], @"mysql",
+                              [NSValue valueWithPointer:[_dataSource resultset]], @"resultset",
+                              nil]];
+    _busy= YES;
+  }
+}
 
-//  [_delegate resultSetView:self willStartQuery:query];
-  
-  [NSApplication detachDrawingThread:@selector(doApply:)
-							toTarget:self
-						  withObject:[NSDictionary dictionaryWithObjectsAndKeys:
-							[NSValue valueWithPointer:_mysql], @"mysql",
-							[NSValue valueWithPointer:[_dataSource resultset]], @"resultset",
-							nil]];
-  _busy= YES;
+
+
+- (BOOL)transactionOpen
+{
+  return _transactionOpen;
+}
+
+
+- (void)startTransaction
+{
+  if (myx_mysql_query(_mysql, "START TRANSACTION") == 0)
+  {
+    _transactionOpen= YES;
+    [_delegate performSelector:@selector(resultSetViewTransactionStarted:)
+                    withObject:self];
+  }
+  else
+    MXRunAlertPanelWithMySQLError(@"Error", @"Error starting transaction.", _mysql);
+}
+
+
+- (void)commitTransaction
+{
+  if (myx_mysql_query(_mysql, "COMMIT") == 0)
+  {
+    _transactionOpen= NO;
+    [_delegate performSelector:@selector(resultSetViewTransactionEnded:commited:)
+                    withObject:self
+                    withObject:(id)YES];
+  }
+  else
+    MXRunAlertPanelWithMySQLError(@"Error", @"Error commiting transaction.", _mysql);
+}
+
+
+- (void)rollbackTransaction
+{
+  if (myx_mysql_query(_mysql, "ROLLBACK")== 0)
+  {
+    _transactionOpen= NO;
+    [_delegate performSelector:@selector(resultSetViewTransactionEnded:commited:)
+                    withObject:self
+                    withObject:(id)NO];
+  }
+  else
+    MXRunAlertPanelWithMySQLError(@"Error", @"Error rolling back transaction.", _mysql);
 }
 
 
@@ -1139,7 +1226,7 @@ static int fetchProgress(unsigned long current_row_count,
 {
   if (_query != query)
   {
-	[_query release];
+    [_query release];
     if ([query isMemberOfClass:[NSMutableString class]])
       _query= [[NSString stringWithString:query] retain];
     else
@@ -1155,15 +1242,15 @@ static int fetchProgress(unsigned long current_row_count,
 
 - (BOOL)tableView:(NSTableView *)aTableView 
 shouldEditTableColumn:(NSTableColumn *)aTableColumn 
-			  row:(int)rowIndex
+              row:(int)rowIndex
 {
   int column= [[aTableColumn identifier] intValue];
   if ([_dataSource editing] && [_dataSource statusOfRow:rowIndex
-												 column:column] != MESDeleted
+                                                 column:column] != MESDeleted
       && [_dataSource resultset]->columns[column].column_type!=MYX_RSCT_BLOB)
-	return YES;
+    return YES;
   else
-	return NO;
+    return NO;
 }
 
 - (int)numberOfItemsInMenu:(NSMenu *)menu
@@ -1173,38 +1260,38 @@ shouldEditTableColumn:(NSTableColumn *)aTableColumn
 
 - (BOOL)menu:(NSMenu *)menu
   updateItem:(NSMenuItem *)item
-	 atIndex:(int)index
+     atIndex:(int)index
 shouldCancel:(BOOL)shouldCancel
 {
   BOOL flag= NO;
   BOOL hasSelection= [table selectedRow]>=0;
-
+  
   switch (index)
   {
-	case 0: //add
-	  flag= [_dataSource editing];
-	  break;
-	case 1: //delete
-	  flag= [_dataSource editing] && hasSelection;
-	  break;
-	case 3: //load
-	  flag= [_dataSource editing] && hasSelection;
-	  break;
-	case 4: //save
-	  flag= hasSelection;
-	  break;
-	case 5: //copy
-	  flag= hasSelection;
-	  break;
-	case 6: //clear
-	  flag= [_dataSource editing] && hasSelection;
-	  break;	  
-	case 8: //view
-	  flag= hasSelection;
-	  break;
-	case 9: //edit
-	  flag= [_dataSource editing] && hasSelection;
-	  break;
+    case 0: //add
+      flag= [_dataSource editing];
+      break;
+    case 1: //delete
+      flag= [_dataSource editing] && hasSelection;
+      break;
+    case 3: //load
+      flag= [_dataSource editing] && hasSelection;
+      break;
+    case 4: //save
+      flag= hasSelection;
+      break;
+    case 5: //copy
+      flag= hasSelection;
+      break;
+    case 6: //clear
+      flag= [_dataSource editing] && hasSelection;
+      break;	  
+    case 8: //view
+      flag= hasSelection;
+      break;
+    case 9: //edit
+      flag= [_dataSource editing] && hasSelection;
+      break;
   }
   [item setEnabled:flag];
   [menu itemChanged:item];
@@ -1222,7 +1309,7 @@ shouldCancel:(BOOL)shouldCancel
 - (void)tableView:(NSTableView *)aTableView 
   willDisplayCell:(id)aCell
    forTableColumn:(NSTableColumn *)aTableColumn
-			  row:(int)rowIndex
+              row:(int)rowIndex
 {
   if ([[aTableColumn identifier] isEqualTo:@"indicator"])
   {
@@ -1230,7 +1317,7 @@ shouldCancel:(BOOL)shouldCancel
       [aCell setSelected:YES];
     else
       [aCell setSelected:NO];
-
+    
     if ([_dataSource statusOfRow:rowIndex column:0] == MESPlaceHolder)
       [aCell setPlaceholder:YES];
     else
@@ -1238,7 +1325,7 @@ shouldCancel:(BOOL)shouldCancel
   }
   else if ([_dataSource editing])
   {
-	int column= [[aTableColumn identifier] intValue];
+    int column= [[aTableColumn identifier] intValue];
     MYXRSCompareStatus diff;
     
     [aCell setPlaceholder:NO];
@@ -1252,7 +1339,7 @@ shouldCancel:(BOOL)shouldCancel
       [aCell setTextColor:[NSColor alternateSelectedControlTextColor]];
       return;
     }
-
+    
     [aCell setTextColor:[NSColor blackColor]];
     
     if (rowIndex >= (int)[_dataSource resultset]->rows_num ||
@@ -1288,7 +1375,7 @@ shouldCancel:(BOOL)shouldCancel
   {
     int column= [[aTableColumn identifier] intValue];
     MYXRSCompareStatus diff;
-
+    
     [aCell setPlaceholder:NO];
     if ((diff=[_dataSource compareStatusOfRow:rowIndex column:column])==MCSMatch)
     {
@@ -1320,11 +1407,11 @@ shouldCancel:(BOOL)shouldCancel
   NSView *top= [children objectAtIndex:0];
   NSView *bottom= [children lastObject];
   NSRect rect;
-
+  
   rect= [sender frame];
   rect.size.height-= [sender dividerThickness] + NSHeight([bottom frame]);  
   [top setFrame:rect];
-
+  
   rect.origin.y= rect.size.height + [sender dividerThickness];
   rect.size.height= NSHeight([bottom frame]);
   [bottom setFrame:rect];
@@ -1356,11 +1443,13 @@ shouldCancel:(BOOL)shouldCancel
   [table setNeedsDisplay];
 }
 
+
 - (void)selectedError:(NSNotification*)notif
 {
-  MessageItem *item= ((MessageDS*)_messageDS)->items+[messageTable selectedRow];
+  int row= [messageTable selectedRow];
+  MessageItem *item= ((MessageDS*)_messageDS)->items + row;
   
-  if (item->row != (unsigned int)-1)
+  if (row >= 0 && item->row != (unsigned int)-1)
   {
     [table selectCellAtRow:item->row
                     column:item->column];
@@ -1368,5 +1457,9 @@ shouldCancel:(BOOL)shouldCancel
 }
 
 
+- (BOOL)hasChanges
+{
+  return [_dataSource hasChanges];
+}
 @end
 
