@@ -25,8 +25,8 @@
 
 #include "myx_public_interface.h"
 #include "myx_qb_public_interface.h"
-#include "myx_aux_functions.h"
-#include <myx_shared_aux_functions.h>
+#include "myx_util_functions.h"
+#include <myx_shared_util_functions.h>
 #include "glib.h"
 
 /*
