@@ -24,7 +24,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 #include <libxml/xpath.h>
 
 #include <assert.h>
-#include <myx_xml_aux_functions.h>
+#include <myx_xml_util_functions.h>
 #include <string.h>
 #include <time.h>
 
