@@ -69,7 +69,7 @@ void CellRendererBlob::set_max_text_width(guint width)
 }
 
 
-Glib::PropertyProxy<std::string> CellRendererBlob::property_field()
+Glib::PropertyProxy<std::string> CellRendererBlob::property_field() const
 {
   return _property_field.get_proxy();
 }
@@ -96,7 +96,7 @@ Glib::PropertyProxy<int> CellRendererBlob::property_column()
 void CellRendererBlob::get_size_vfunc(Gtk::Widget& widget,
                                       const Gdk::Rectangle* cell_area,
                                       int* x_offset, int* y_offset,
-                                      int* width,    int* height)
+                                      int* width,    int* height) const
 {
   gpointer fdata;
   gsize fsize;
@@ -164,9 +164,12 @@ void CellRendererBlob::get_size_vfunc(Gtk::Widget& widget,
 }
 
 
-void CellRendererBlob::render_vfunc(const Glib::RefPtr<Gdk::Window>& window, Gtk::Widget& widget,
+void CellRendererBlob::render_vfunc(const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget,
                                     const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area,
                                     const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags)
+//void CellRendererBlob::render_vfunc(const Glib::RefPtr<Gdk::Window>& window, Gtk::Widget& widget,
+//                                    const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area,
+//                                    const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags)
 {
   gpointer fdata; 
   gsize fsize; 
@@ -237,12 +240,11 @@ void CellRendererBlob::render_vfunc(const Glib::RefPtr<Gdk::Window>& window, Gtk
     window->draw_pixbuf(_gc, _load_icon, 0, 0,
                         x, cell_area.get_y()+(cell_area.get_height()-_load_icon->get_height())/2, -1, -1,
                         Gdk::RGB_DITHER_NONE, 0, 0);
-/*
+
     x-= _edit_icon->get_width()+4;
     window->draw_pixbuf(_gc, _edit_icon, 0, 0,
                         x, cell_area.get_y()+(cell_area.get_height()-_edit_icon->get_height())/2, -1, -1,
                         Gdk::RGB_DITHER_NONE, 0, 0);
- */
 
     x-= _save_icon->get_width()+4;
     window->draw_pixbuf(_gc, _save_icon, 0, 0,
@@ -308,14 +310,14 @@ bool CellRendererBlob::activate_vfunc(GdkEvent* event,
         load_clicked(widget.get_parent(), path);
         return true;
       }
-/*
+
       x-= _edit_icon->get_width()+4;
       if (cx >= x && cx < x+_edit_icon->get_width())
       {
         edit_clicked(widget.get_parent(), path);
         return true;
       }
- */
+
       
       x-= _save_icon->get_width()+4;
       if (cx >= x && cx < x+_save_icon->get_width())
