@@ -64,7 +64,7 @@ class CellRendererBlob : public Gtk::CellRenderer
 
     // property_field doesn't really contain the data. it's a hack containing
     // a string with the pointer and size of the data
-    Glib::Property<std::string> _property_field;
+    mutable Glib::Property<std::string> _property_field;
     Glib::Property<bool> _property_editable;
     Glib::Property<std::string> _property_background;
     Glib::Property<int> _property_column;
@@ -73,15 +73,15 @@ class CellRendererBlob : public Gtk::CellRenderer
     
     guint _max_text_length;
 
-    guint _max_text_width;
-    int _text_char_width;
-    int _text_height;
+    mutable guint _max_text_width;
+    mutable int _text_char_width;
+    mutable int _text_height;
 
     bool _binary;
     
     Glib::RefPtr<Gdk::GC> _gc;
     Glib::RefPtr<Gdk::Pixbuf> _blob_icon;
-    Glib::RefPtr<Pango::Layout> _layout;
+    mutable Glib::RefPtr<Pango::Layout> _layout;
     
     Glib::RefPtr<Gdk::Pixbuf> _clear_icon;
     Glib::RefPtr<Gdk::Pixbuf> _edit_icon;
@@ -101,9 +101,13 @@ class CellRendererBlob : public Gtk::CellRenderer
     virtual void get_size_vfunc(Gtk::Widget& widget,
                                 const Gdk::Rectangle* cell_area,
                                 int* x_offset, int* y_offset,
-                                int* width,    int* height);
+                                int* width,    int* height) const;
     
-    virtual void render_vfunc(const Glib::RefPtr<Gdk::Window>& window, Gtk::Widget& widget, 
+  
+    //virtual void render_vfunc(const Glib::RefPtr<Gdk::Window>& window, Gtk::Widget& widget, 
+    //                          const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, 
+    //                          const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags);
+    virtual void render_vfunc(const Glib::RefPtr<Gdk::Drawable>& window, Gtk::Widget& widget, 
                               const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, 
                               const Gdk::Rectangle& expose_area, Gtk::CellRendererState flags);
 
@@ -124,7 +128,7 @@ class CellRendererBlob : public Gtk::CellRenderer
 
     void set_max_text_width(guint width);
     
-    Glib::PropertyProxy<std::string> property_field(); // should be built with data2str()
+    Glib::PropertyProxy<std::string> property_field() const; // should be built with data2str()
     Glib::PropertyProxy<bool> property_editable();
     Glib::PropertyProxy<std::string> property_background();
     Glib::PropertyProxy<int> property_column();
