@@ -63,6 +63,16 @@ extern "C" {
                                        GtkSourceLanguagesManager *lm);
 };
 
+void MGCodeEditor::set_dirty(bool dirty)
+{
+  this->dirty= dirty;
+}
+
+bool MGCodeEditor::get_dirty()
+{
+  return this->dirty= dirty;
+}
+
 void MGCodeEditor::set_language(const std::string &mime)
 {
   static GtkSourceLanguagesManager *lm= 0;
@@ -324,6 +334,3 @@ bool MGCodeEditor::has_breakpoint(const Gtk::TextIter &iter)
 
   return false;
 }
-
-
-
