@@ -175,7 +175,7 @@ bool MGResultSetModel::iter_nth_child_vfunc (const Gtk::TreeModel::iterator& par
   if (!parent)
   {
     iter.set_stamp(_stamp);
-    iter.gobj()->user_data= (gpointer)n;
+    iter.gobj()->user_data= (gpointer)(long)n;
     return true;
   }
   iter.set_stamp(0);
@@ -239,7 +239,7 @@ bool MGResultSetModel::get_iter_vfunc (const Gtk::TreeModel::Path& path,
     return false;
   }
 
-  unsigned int row_index= path[0];
+  unsigned long row_index= path[0];
 
   if (row_index >= _resultset->rows_num)
   {
@@ -256,7 +256,7 @@ bool MGResultSetModel::get_iter_vfunc (const Gtk::TreeModel::Path& path,
 }
 
 bool MGResultSetModel::get_value(const Gtk::TreeModel::iterator& row, 
-  unsigned int column, gpointer &value, guint &value_length) const
+  unsigned int column, gpointer &value, size_t &value_length) const
 {
   const GtkTreeIter *iter= row.gobj();
 
@@ -434,10 +434,10 @@ void MGResultSetModel::set_value_impl(const Gtk::TreeModel::iterator& row, int c
 }
 
 
-void MGResultSetModel::set_value(const Gtk::TreeModel::iterator& row, int column, const gpointer value, guint value_length)
+void MGResultSetModel::set_value(const Gtk::TreeModel::iterator& row, int column, const gpointer value, size_t value_length)
 {
   unsigned long row_num= (unsigned long)row.gobj()->user_data;
-  
+
   switch ((column%2))
   {
   case 0:
@@ -520,7 +520,7 @@ MYX_RS_ROW *MGResultSetModel::get_iter_row(const Gtk::TreeModel::iterator &iter)
 Gtk::TreeModel::iterator MGResultSetModel::append()
 {
   Gtk::TreeModel::iterator iter(this);
-  unsigned int new_row= add_row();
+  unsigned long new_row= add_row();
 
   // make new iterator
   iter.set_stamp(_stamp);
