@@ -26,7 +26,7 @@ MQNewBookmarkDialog::MQNewBookmarkDialog(GtkWindow *win, MGGladeXML *xml)
   _xml->get_button("add_button")->signal_clicked().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::add_clicked));
   _xml->get_button("cancel_button")->signal_clicked().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::cancel_clicked));
   _xml->get_entry("name_entry")->signal_changed().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::name_changed));
-  _xml->get_combo_entry("folder_combo")->get_entry()->signal_changed().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::name_changed));
+  ((Gtk::Entry*)_xml->get_combo_entry("folder_combo")->get_child())->signal_changed().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::name_changed));
   _xml->get_button("expand_button")->signal_clicked().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::toggle_detail));
   _xml->get_button("new_folder_button")->signal_clicked().connect(sigc::mem_fun(*this,&MQNewBookmarkDialog::new_folder_clicked));
 
@@ -63,7 +63,7 @@ void MQNewBookmarkDialog::name_changed()
   if (!_xml->get_entry("name_entry")->get_text().empty())
     ok= true;
   
-  if (!_xml->get_combo_entry("folder_combo")->get_entry()->get_text().empty())
+  if (!((Gtk::Entry*)_xml->get_combo_entry("folder_combo")->get_child())->get_text().empty())
     ok= true;
 
   _xml->get_button("add_button")->set_sensitive(ok);
@@ -176,7 +176,7 @@ bool MQNewBookmarkDialog::run(const Glib::ustring &catalog,
   _xml->get_entry("name_entry")->select_region(0, -1);
   _xml->get_entry("name_entry")->grab_focus();
 
-  if (_xml->get_combo_entry("folder_combo")->get_entry()->get_text().empty())
+  if (((Gtk::Entry*)_xml->get_combo_entry("folder_combo")->get_child())->get_text().empty())
     _xml->get_button("add_button")->set_sensitive(false);
   else
     _xml->get_button("add_button")->set_sensitive(true);
@@ -193,7 +193,7 @@ bool MQNewBookmarkDialog::run(const Glib::ustring &catalog,
   if (_added)
   {
     Glib::ustring name= _xml->get_entry("name_entry")->get_text();
-    Glib::ustring folder= _xml->get_combo_entry("folder_combo")->get_entry()->get_text();
+    Glib::ustring folder= ((Gtk::Entry*)_xml->get_combo_entry("folder_combo")->get_child())->get_text();
     MQBookmarks::BookmarkGroup *group;
 
     // find the folder group
@@ -249,7 +249,7 @@ void MQNewBookmarkDialog::folder_selected()
 
     MQBookmarks::BookmarkGroup *group= row[_columns.group];
     
-    _xml->get_combo_entry("folder_combo")->get_entry()->set_text(group->name);
+    ((Gtk::Entry*)_xml->get_combo_entry("folder_combo")->get_child())->set_text(group->name);
     
     _xml->get_button("new_folder_button")->set_sensitive(true);
   }
