@@ -27,6 +27,9 @@ class MQPreferences : public MGPreferences
     MQPreferences();
 
     bool dont_beep;
+    bool auto_add_pk_check;
+    
+    int default_limit_value; // default value for LIMIT
     
     int view_type; // 'N'ormal, 'Q'uery area, 'R'resultset
     bool show_sidebar;
