@@ -20,6 +20,7 @@
 #include "myx_public_interface.h"
 #include "myx_qb_public_interface.h"
 #include "MDispatcher.h"
+#include "MQPreferences.h"
 
 #include <gtkmm/widget.h>
 
@@ -29,19 +30,21 @@
 
 class MGMySQLConsole;
 
+extern MQPreferences prefs;
+
 class MQQueryDispatcher : public Glib::ObjectBase {
   public:
-    typedef SigC::Slot2<void,MYX_RESULTSET*,MYX_MYSQL_ERROR_MSGS*> FinishedCallback;
-    typedef SigC::Slot2<void,MYX_RESULTSET*,MYX_MYSQL_ERROR_MSGS*> ErrorCallback;
-    typedef SigC::Slot3<bool,MYX_RESULTSET*,unsigned long,unsigned long> ProgressCallback;
-    typedef SigC::Slot1<void,MYSQL*> FetchMoreCallback;
+    typedef sigc::slot2<void,MYX_RESULTSET*,MYX_MYSQL_ERROR_MSGS*> FinishedCallback;
+    typedef sigc::slot3<void,MYX_RESULTSET*,MYX_MYSQL_ERROR_MSGS*,MYX_LIB_ERROR> ErrorCallback;
+    typedef sigc::slot3<bool,MYX_RESULTSET*,unsigned long,unsigned long> ProgressCallback;
+    typedef sigc::slot1<void,MYSQL*> FetchMoreCallback;
 
-    typedef SigC::Signal0<void> SchemaChangedSignal;
-    typedef SigC::Signal0<void> TransactionStartedSignal;
-    typedef SigC::Signal1<void,bool> TransactionEndedSignal;
-    typedef SigC::Signal1<void,Glib::ustring> TransactionCommandSignal;
+    typedef sigc::signal0<void> SchemaChangedSignal;
+    typedef sigc::signal0<void> TransactionStartedSignal;
+    typedef sigc::signal1<void,bool> TransactionEndedSignal;
+    typedef sigc::signal1<void,Glib::ustring> TransactionCommandSignal;
 
-    typedef SigC::Slot1<void,MYX_RS_ACTION_ERRORS*> SaveFinishedCallback;
+    typedef sigc::slot1<void,MYX_RS_ACTION_ERRORS*> SaveFinishedCallback;
 
     class Message {
       public:
@@ -65,7 +68,7 @@ class MQQueryDispatcher : public Glib::ObjectBase {
     
     class QueryRequest : public Request {
       public:
-        QueryRequest() : result(0) {};
+        QueryRequest() : enforceEditable(prefs.auto_add_pk_check), result(0) {};
 
         MQQueryDispatcher *dispatcher;
       
