@@ -1414,7 +1414,7 @@ void MQResultSetView::column_action_mi(FieldAction action)
       Gtk::FileSelection fsel(_("Save Field Data"));
       gsize written;
       gpointer fdata;
-      guint fsize;
+      size_t fsize;
       
       if (fsel.run() == Gtk::RESPONSE_OK)
       {
@@ -1438,7 +1438,7 @@ void MQResultSetView::column_action_mi(FieldAction action)
     if (!is_blob)
     {
       gpointer fdata;
-      guint fsize;
+      size_t fsize;
       _model->get_value(_model->get_iter(path), column_index*2, fdata, fsize);
 
       if (fdata)
@@ -1459,7 +1459,7 @@ void MQResultSetView::column_action_mi(FieldAction action)
     else
     {
       gpointer fdata;
-      guint fsize;
+      size_t fsize;
       char *cpath;
       MGBlobEditor *editor= new MGBlobEditor((MGBlobEditor::ViewType)(MGBlobEditor::VText|MGBlobEditor::VBinary),true);
 
@@ -1485,7 +1485,7 @@ void MQResultSetView::column_action_mi(FieldAction action)
     else
     {
       gpointer fdata;
-      guint fsize;
+      size_t fsize;
       MGBlobEditor *editor= new MGBlobEditor((MGBlobEditor::ViewType)(MGBlobEditor::VText|MGBlobEditor::VBinary));
 
       _model->get_value(_model->get_iter(path), column_index*2, fdata, fsize);
@@ -1851,7 +1851,7 @@ void MQResultSetView::editor_save(MGBlobEditor *editor)
   editor->get_data(buffer, size);
 
   path= (char*)((Glib::Object*)editor)->get_data("path");
-  column= (int)((Glib::Object*)editor)->get_data("column");
+  column= (int)(long)((Glib::Object*)editor)->get_data("column");
 
   Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
   _model->set_value(iter, column*2, buffer, size);
@@ -1890,7 +1890,7 @@ void MQResultSetView::blob_edit(CellRendererBlob *cell, Gtk::Widget *widget, con
     editor->set_delete_on_close();
 
     ((Glib::Object*)editor)->set_data("path", g_strdup(path.c_str()), g_free);
-    ((Glib::Object*)editor)->set_data("column", (char *)column, NULL);
+    ((Glib::Object*)editor)->set_data("column", (char *)(long)column, NULL);
     
     editor->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
     editor->set_transient_for(*(Gtk::Window*)widget->get_toplevel());
@@ -2006,5 +2006,5 @@ void MQResultSetView::blob_data_edited(const Glib::ustring& path, gpointer data,
 
 void MQResultSetView::string_data_edited(const Glib::ustring& path, const Glib::ustring& new_text, int column)
 {
-  _model->set_value(_model->get_iter(path), column, (const gpointer)new_text.data(), new_text.size());
+  _model->set_value(_model->get_iter(path), column, (const gpointer)new_text.data(), new_text.bytes());
 }
