@@ -43,13 +43,13 @@ class MGBlobEditor;
 
 class MQResultSetView : public Gtk::EventBox, public MGBlobDelegate {
   public:
-    typedef SigC::Signal2<void,MQResultSetView*,bool> QueryStartedSignal;
-    typedef SigC::Signal1<void,MQResultSetView*> QueryFinishedSignal;
-    typedef SigC::Signal2<void,MQResultSetView*,MYSQL*> QueryMoreDataSignal;
+    typedef sigc::signal2<void,MQResultSetView*,bool> QueryStartedSignal;
+    typedef sigc::signal1<void,MQResultSetView*> QueryFinishedSignal;
+    typedef sigc::signal2<void,MQResultSetView*,MYSQL*> QueryMoreDataSignal;
     
-    typedef SigC::Signal1<void,MQResultSetView*> ActivateSignal;
+    typedef sigc::signal1<void,MQResultSetView*> ActivateSignal;
     
-    typedef SigC::Signal1<void,MQResultSetView*> RowChangedSignal;
+    typedef sigc::signal1<void,MQResultSetView*> RowChangedSignal;
 
   protected:
     Gtk::VBox _box;
@@ -150,11 +150,11 @@ class MQResultSetView : public Gtk::EventBox, public MGBlobDelegate {
         virtual bool on_button_press_event(GdkEventButton *ev);
         Gtk::Menu *_popup;
         
-        SigC::Signal0<void> _signal_will_popup;
+        sigc::signal0<void> _signal_will_popup;
       public:
         PopupTreeView(Gtk::Menu *popup) : Gtk::TreeView(), _popup(popup) {};
         
-        SigC::Signal0<void> signal_will_popup() { return _signal_will_popup; };
+        sigc::signal0<void> signal_will_popup() { return _signal_will_popup; };
     };
     
     enum FieldAction {
@@ -215,7 +215,7 @@ class MQResultSetView : public Gtk::EventBox, public MGBlobDelegate {
     QueryMoreDataSignal _query_more_data_signal;
     ActivateSignal _activate_signal;
     RowChangedSignal _row_changed_signal;
-    SigC::Signal0<void> _editable_change_signal;
+    sigc::signal0<void> _editable_change_signal;
 
     bool _editing;
     bool _is_busy;
@@ -240,7 +240,7 @@ class MQResultSetView : public Gtk::EventBox, public MGBlobDelegate {
     void query_finished(MYX_RESULTSET *result,MYX_MYSQL_ERROR_MSGS*warnings);
     bool query_progress(MYX_RESULTSET *result, unsigned long count,
                         unsigned long prev_count);
-    void query_error(MYX_RESULTSET *result,MYX_MYSQL_ERROR_MSGS*errors);
+    void query_error(MYX_RESULTSET *result, MYX_MYSQL_ERROR_MSGS*errors, MYX_LIB_ERROR merror);
     void query_fetch_more(MYSQL *mysql);
     
     void save_finished(MYX_RS_ACTION_ERRORS *errors);
@@ -315,7 +315,7 @@ class MQResultSetView : public Gtk::EventBox, public MGBlobDelegate {
     
     void set_compact_editbar(bool flag);
     
-    void set_search_func(const SigC::Slot1<void,MQResultSetView*> &slot);
+    void set_search_func(const sigc::slot1<void,MQResultSetView*> &slot);
 
     MYX_RESULTSET *get_resultset() { return _result; };
     MYX_RS_ROW *get_selected_row();
@@ -345,7 +345,7 @@ class MQResultSetView : public Gtk::EventBox, public MGBlobDelegate {
     ActivateSignal signal_activate() { return _activate_signal; };
     RowChangedSignal signal_row_changed() { return _row_changed_signal; };
     
-    SigC::Signal0<void> signal_editable_changed() { return _editable_change_signal; };
+    sigc::signal0<void> signal_editable_changed() { return _editable_change_signal; };
 
     Gtk::ScrolledWindow *get_scrollwin() { return &_scroll; };
     
