@@ -591,7 +591,7 @@ bool MQResultTab::copy_row_contents(MQResultSetView *from_rs,
   for (unsigned int i= 0; i < resultset->columns_num; i++)
   {
     gpointer fdata, tdata;
-    guint fsize, tsize;
+    size_t fsize, tsize;
 
     if (!from_model->get_value(from_iter, i*2, fdata, fsize))
     {
