@@ -38,16 +38,17 @@ class MQSQLCodeEditor : public MGCodeEditor {
     
     MYX_SQL_TEXT *_sql_text;
 
-    SigC::Signal0<void> _cursor_moved_signal;
+    sigc::signal0<void> _cursor_moved_signal;
     int _last_line_count;
 
-    void move_cursor(Gtk::MovementStep step, int count, bool extend_selection);
+    //void move_cursor(Gtk::MovementStep step, int count, bool extend_selection);
+    void move_cursor(const Gtk::TextBuffer::iterator&, const Glib::RefPtr<Gtk::TextBuffer::Mark>&);
     virtual bool is_statement_start(int line);
   public:
     MQSQLCodeEditor();
     virtual ~MQSQLCodeEditor();
 
-    SigC::Signal0<void> signal_cursor_moved() { return _cursor_moved_signal; };
+    sigc::signal0<void> signal_cursor_moved() { return _cursor_moved_signal; };
 
     bool is_error_ignored(int err);
     void ignore_error(int err);
@@ -81,8 +82,8 @@ class MQScriptEditorTab : public MQBaseTab {
       SError
     };
     
-    typedef SigC::Signal2<void,MQScriptEditorTab*,State> SignalStateChanged;
-    typedef SigC::Signal1<void,MQScriptEditorTab*> SignalScriptEditor;
+    typedef sigc::signal2<void,MQScriptEditorTab*,State> SignalStateChanged;
+    typedef sigc::signal1<void,MQScriptEditorTab*> SignalScriptEditor;
     
   protected:
     class MessageColumns : public Gtk::TreeModel::ColumnRecord
@@ -125,6 +126,8 @@ class MQScriptEditorTab : public MQBaseTab {
     
     bool button_pressed(GdkEventButton *ev);
     
+    virtual bool tab_will_close();
+    
   public:
     MQScriptEditorTab(MQQueryDispatcher *disp);
     virtual ~MQScriptEditorTab();
