@@ -224,13 +224,15 @@ class MQWorkArea : public MQBaseModule {
 
     void export_resultset(const char *fmt);
 
-    bool get_schema_from_selection_data(GtkSelectionData* selection_data,
+    //bool get_schema_from_selection_data(GtkSelectionData* selection_data,
+    bool get_schema_from_selection_data(const Gtk::SelectionData& selection_data,
                                         Glib::ustring &catalog,
                                         Glib::ustring &schema,
                                         Glib::ustring &query);
                               
     
-    void query_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, GtkSelectionData* selection_data, guint info, guint time);
+    //void query_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, GtkSelectionData* selection_data, guint info, guint time);
+    void query_drag_data_received(const Glib::RefPtr<Gdk::DragContext>& context, int x, int y, const Gtk::SelectionData& selection_data, guint info, guint time);
 
     bool query_get_completion_list(const Gtk::TextIter&,std::list<Glib::ustring>&);
     
@@ -321,6 +323,7 @@ class MQWorkArea : public MQBaseModule {
 
     virtual void catalogs_refreshed();
     void select_schema_from_browser();
+    void copy_sql();
     
     void schemata_button_press(GdkEventButton *ev);
     void schemata_button_release(GdkEventButton *ev);
