@@ -15,6 +15,7 @@
 #import <MySQLToolsCommon/MSQLSyntaxColoring.h>
 
 #import "MQueryBrowserController.h"
+#import "MQueryBrowserDocument.h"
 #import "MQResultSetTab.h"
 #import "MQQueryController.h"
 #import "MQueryBrowser.h"
@@ -22,6 +23,9 @@
 #import "MQActionBar.h"
 #import "MQParameters.h"
 
+
+#include <myx_public_interface.h>
+
 @interface MQActionBarTextView : MSourceTextView
 {
 @public
@@ -36,6 +40,7 @@
   if (self)
   {
     actionBar= [[MQActionBar alloc] init];
+    [self registerForDraggedTypes:[NSArray arrayWithObject:MSchemaItemPboardType]];
   }
   return self;
 }
@@ -43,7 +48,6 @@
 - (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
 {
   NSPoint pt= [self frame].origin;
-  
   pt.y+= NSHeight([self frame]);
   if (![actionBar superview])
   {
@@ -52,19 +56,21 @@
     [actionBar setHidden:YES];
     [view addSubview:actionBar positioned:NSWindowAbove relativeTo:[[view subviews] objectAtIndex:0]];
   }
-  [actionBar popupAt:[[self superview] convertPoint:pt toView:nil]];
+  
+  if ([[[sender draggingSource] dataSource] isKindOfClass:[MSchemaDataSource class]])
+    [actionBar popupAt:[[self superview] convertPoint:pt toView:nil]];
 
   return NSDragOperationNone;
 }
 
 - (void)draggingExited:(id <NSDraggingInfo>)sender
 {
-  [actionBar hide];
+  [actionBar delayedHide];
 }
 
 - (void)draggingEnd:(id <NSDraggingInfo>)sender
 {
-  [actionBar hide];
+  [actionBar delayedHide];
 }
 
 /*
@@ -82,9 +88,7 @@
 
 - (MSourceTextView*)createTextViewWithFrame:(NSRect)frame
 {
-  id tv= [[MQActionBarTextView alloc] initWithFrame:frame];
-  [tv registerForDraggedTypes:[NSArray arrayWithObject:MSchemaItemPboardType]];
-  return tv;
+  return [[MQActionBarTextView alloc] initWithFrame:frame];
 }
 
 @end
@@ -96,6 +100,8 @@
 {
   NSRect wrect= [[owner window] contentRectForFrameRect:[[owner window] frame]];
   NSRect trect;
+  NSUserDefaults *defaults= [NSUserDefaults standardUserDefaults];
+  
   trect= [toolbar frame];
   [toolbar setFrameOrigin:NSMakePoint(-1,NSHeight(wrect)-NSHeight(trect))];
   
@@ -120,7 +126,11 @@
   [[[queryArea enclosingScrollView] horizontalScroller] setControlSize:NSSmallControlSize];
   [[queryArea enclosingScrollView] setAutohidesScrollers:YES];
   
-  [queryArea setFont:[NSFont fontWithName:@"Andale Mono" size:11.0]];
+  [queryArea registerForDraggedTypes:[NSArray arrayWithObject:MSchemaItemPboardType]];
+  
+  [queryArea setFont:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"EditorFont"]]];
+  [queryArea setBackgroundColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"EditorBackgroundColor"]]];
+  [queryArea setTextColor:[NSUnarchiver unarchiveObjectWithData:[defaults objectForKey:@"EditorForegroundColor"]]];
 
   _idleImage= [[NSImage imageNamed:@"sakila.png"] retain];
   _busyImage= [[NSImage imageNamed:@"dolphin_anim.gif"] retain];
@@ -151,7 +161,6 @@
   MQResultSetTab *page= [[MQResultSetTab alloc] initWithIdentifier:@"aa"];
   
   [page setIcon:[NSImage imageNamed:@"tabsheet_icon_resultset.png"]];
-  [self addNewResultSet:YES toPage:page];
   [page setLabel:@"Query"];
   
   return [page autorelease];
@@ -170,7 +179,7 @@
   NSString *query= [NSString stringWithString:[queryArea string]];
   MQResultSetTab *page;
   MQResultSetView *rsview;
-  int i, c;
+  int i;
   
   switch ([sender tag])
   {
@@ -219,6 +228,38 @@
   }
 }
 
+
+- (void)exportResultSet:(id)sender
+{
+  MQResultSetTab *page= [owner currentTab];
+
+  if ([page isKindOfClass:[MQResultSetTab class]])
+  {
+    const char *format= NULL;
+    switch ([sender tag])
+    {
+      case 1: // CSV
+        format= "CSV";
+        break;
+      case 2: // HTML
+        format= "HTML";
+        break;
+      case 3: // XML
+        format= "XML";
+        break;
+      case 4: // Excel
+        format= "Excel";
+        break;
+      case 5: // PropList
+        format= "PLIST";
+        break;
+    }
+    if (format)
+      [page exportResultSet:format];
+  }
+}
+
+
 - (void)openLocalHistory:(id)sender
 {
   id item;
@@ -231,6 +272,7 @@
   _localHistoryPos= [_localHistory indexOfObject:item];
 }
 
+
 - (void)updateHistoryMenus
 {
   int i, c;
@@ -288,22 +330,35 @@
   [self updateHistoryMenus];
 }
 
+
+- (void)updateTransactionState
+{
+  BOOL flag= [[[owner currentTab] activeResultSet] transactionOpen];
+    
+  [[toolbar viewWithTag:14] setEnabled:!flag];
+  [[toolbar viewWithTag:15] setEnabled:flag];
+  [[toolbar viewWithTag:16] setEnabled:flag];
+}
+
+
 - (IBAction)rollbackTransaction:(id)sender
 {
-  ///XXXXX
+  [[[owner currentTab] activeResultSet] rollbackTransaction];
 }
 
 - (IBAction)startTransaction:(id)sender
 {
+  [[[owner currentTab] activeResultSet] startTransaction];
 }
 
 - (IBAction)commitTransaction:(id)sender
 {
+  [[[owner currentTab] activeResultSet] commitTransaction];
 }
 
 - (IBAction)stopQuery:(id)sender
 {
-  [owner killConnection:mysql_thread_id([[self currentResultSetView] mysql])];
+  [[owner document] killConnection:mysql_thread_id([[self currentResultSetView] mysql])];
   [[self currentResultSetView] stopQuery];
 }
 
@@ -316,14 +371,16 @@
 
 - (void)setDefaultSchema:(NSString*)schema
 {
-  _syntaxInfo= myx_refresh_dbinfo([owner mysql], _syntaxInfo);
+  MYSQL *mysql= [[owner document] mysql];
+  
+  _syntaxInfo= myx_refresh_dbinfo(mysql, _syntaxInfo);
   
   MSQLSyntaxColoring *colorer= [[MSQLSyntaxColoring alloc] initForTextView:queryArea
                                                                 syntaxInfo:_syntaxInfo];
   [queryArea setSyntaxColorer: colorer];
   [colorer release];
   
-  [((MQActionBarTextView*)queryArea)->actionBar setMySQL:[owner mysql]];
+  [((MQActionBarTextView*)queryArea)->actionBar setMySQL:mysql];
 }
 
 
@@ -333,7 +390,7 @@
 {
   MQResultSetView *rsview;
   
-  rsview= [[[MQResultSetView alloc] initWithConnectionTo:[owner serverInfo]] autorelease];
+  rsview= [[[MQResultSetView alloc] initWithConnectionTo:[[owner document] serverInfo]] autorelease];
   [[rsview tableView] registerForDraggedTypes:
     [NSArray arrayWithObjects:MQBookmarkPboardType,MSchemaItemPboardType,nil]];
   
@@ -344,8 +401,7 @@
                                target:self];
   
   [rsview setDelegate:self];
-  if ([owner mysql]->db)
-	[rsview setDefaultSchema:[NSString stringWithUTF8String:[owner mysql]->db]];
+  [rsview setDefaultSchema:[[owner document] defaultSchema]];
   
   [page setVertical:vertical];
   [page addResultSet:rsview];
@@ -412,18 +468,23 @@
   else
   {
     MYX_SCHEMA_STORED_PROCEDURE *sp= [item sp];
-    char **paraml;
-    char *params;
+    char **paraml= NULL;
+    char *params= NULL;
     unsigned int i;
     
-    paraml= g_new0(char*,sp->params_num);
-    for (i= 0; i < sp->params_num; i++)
+    if (sp->params_num > 0)
     {
-      paraml[i]= g_strdup_printf("<%s:%s>", sp->params[i].name, sp->params[i].datatype); 
+      paraml= g_new0(char*,sp->params_num);
+      for (i= 0; i < sp->params_num; i++)
+      {
+        paraml[i]= g_strdup_printf("<%s:%s>", sp->params[i].name, sp->params[i].datatype); 
+      }
+  
+      params= g_strjoinv(", ",paraml);
+      query= g_strdup_printf("CALL %s(%s)", sp->name, params);
     }
-    
-    params= g_strjoinv(", ",paraml);
-    query= g_strdup_printf("CALL %s(%s)", sp->name, params);
+    else
+      query= g_strdup_printf("CALL %s()", sp->name);
     cursor= strchr(query, '(')+1-query;
     g_free(params);
     g_strfreev(paraml);
@@ -445,6 +506,16 @@
 }
 
 
+- (void)setupSelectQueryWithItem:(MSchemaItem*)item 
+                        rowLimit:(unsigned int)limit
+{
+  [self setupQueryWithItem:item];
+  
+  if (limit > 0)
+    [queryArea insertText:[NSString stringWithFormat:@"\nLIMIT 0,%i", limit]];
+}
+
+
 - (MQResultSetView*)resultSetViewWithTable:(MQTableView*)table
 {
   unsigned int i, c= [_queryViews count];
@@ -482,6 +553,32 @@
   }
 }
 
+- (void)resultSetViewTransactionStarted:(MQResultSetView*)rsview
+{
+  [self updateTransactionState];
+  if ([[owner currentTab] isKindOfClass:[MQResultSetTab class]])
+  {
+    MQResultSetTab *rstab= [owner currentTab];
+    [rstab setStatusText:@"Transaction Started" icon:nil];
+  }
+}
+
+
+- (void)resultSetViewTransactionEnded:(MQResultSetView*)rsview
+                             commited:(BOOL)commited
+{
+  if ([[owner currentTab] isKindOfClass:[MQResultSetTab class]])
+  {
+    MQResultSetTab *rstab= [owner currentTab];
+    if (commited)
+      [rstab setStatusText:@"Transaction Commited" icon:nil];
+    else
+      [rstab setStatusText:@"Transaction Rolled Back" icon:nil];
+  }  
+  [self updateTransactionState];
+}
+
+
 - (void)resultSetViewWantsFocus:(MQResultSetView*)rsview
 {
   [[(MQResultSetTab*)[owner currentTab] activeResultSet] setLocalParameters:[[owner parameters] localParameters]];
@@ -497,7 +594,7 @@
   _switching= YES;
   [queryArea setString:query];
   _switching= NO;
-  [owner setTitleForDocumentName:[rsview defaultSchema]];
+//XXX  [owner setTitleForDocumentName:[rsview defaultSchema]];
 
   [[owner parameters] setLocalParameters:[rsview localParameters]];
   if (master)
@@ -512,21 +609,28 @@
                                  row:-1];
     [[owner parameterList] reloadData];    
   }
+  [self updateTransactionState];
 }
 
 - (void)resultSetView:(MQResultSetView*)rsview willStartQuery:(NSString*)query
 {
   [animbox setImage:_busyImage];
+
   [[toolbar viewWithTag:10] setEnabled:NO];
-  [nextButton setEnabled:NO];
-  [[toolbar viewWithTag:11] setEnabled:NO];
-  [executeButton setEnabled:NO];
-  [[toolbar viewWithTag:12] setEnabled:NO];  
-  [stopButton setEnabled:YES];
-  [[toolbar viewWithTag:13] setEnabled:YES];
-  [executeButton setEnabled:NO];
-  [[toolbar viewWithTag:14] setEnabled:NO];
   [backButton setEnabled:NO];  
+  [[toolbar viewWithTag:11] setEnabled:NO];
+  [nextButton setEnabled:NO];
+  
+  [[toolbar viewWithTag:12] setEnabled:NO];
+  [executeButton setEnabled:NO];
+  [[toolbar viewWithTag:13] setEnabled:YES];  
+  [stopButton setEnabled:YES];
+  
+  [[toolbar viewWithTag:14] setEnabled:NO];
+  [[toolbar viewWithTag:15] setEnabled:NO];
+  [[toolbar viewWithTag:16] setEnabled:NO];
+  [[toolbar viewWithTag:17] setEnabled:NO];
+  [[toolbar viewWithTag:18] setEnabled:NO];
   
   [queryArea setEditable:NO];
 }
@@ -542,7 +646,11 @@
   [[toolbar viewWithTag:12] setEnabled:YES];
   [stopButton setEnabled:NO];
   [[toolbar viewWithTag:13] setEnabled:NO];
-  
+
+  [[toolbar viewWithTag:17] setEnabled:YES];
+  [[toolbar viewWithTag:18] setEnabled:YES];
+
+  [self updateTransactionState];
   [queryArea setEditable:YES];
   
   if (!flag)
@@ -679,7 +787,8 @@
     [page setEmbeddedQueryArea:NO];
   }
   [owner setDocumentEdited:NO];
-  [owner setTitleForDocumentName:[rsview defaultSchema]];
+//XXX  [owner setTitleForDocumentName:[rsview defaultSchema]];
+  [self updateTransactionState];
 
   _lastSelectedPage= page;
 }
@@ -758,4 +867,46 @@ static BOOL hasVariables(MYX_RESULTSET *rs, const char *query)
   }
 }
 
+
+- (BOOL)stateOfMenuItem:(NSMenuItem*)item atIndex:(int)index inMenu:(NSMenu*)menu
+{
+  if ([[[NSApp mainMenu] itemAtIndex:[[NSApp mainMenu] indexOfItemWithSubmenu:menu]] tag]== 105)
+  {
+    MQResultSetTab *page= [owner currentTab];
+    BOOL flag= NO;
+
+    if (![page isKindOfClass:[MQResultSetTab class]])
+    {
+      [item setEnabled:NO];
+      return YES;
+    }
+
+    switch (index)
+    {
+      case 0: // Bookmark
+        if ([[[page activeResultSet] query] length] > 0)
+          flag= YES;
+        break;
+      case 2: // Execute
+        if (![[page activeResultSet] isBusy])
+          flag= YES;
+        break;
+      case 3: // Stop
+        if ([[page activeResultSet] isBusy])
+          flag= YES;
+        break;
+      case 5: // Explain Query
+        if (![[page activeResultSet] isBusy] && [[page activeResultSet] resultset])
+          flag= YES;
+        break;
+      case 6: // Compare Resultsets
+        if (![[page activeResultSet] isBusy])
+          flag= YES;
+        break;
+    }
+    [item setEnabled:flag];
+  }
+  return YES;
+}
+
 @end
