@@ -45,7 +45,6 @@ extern NSString *MQScriptEditStatusChangedNotification;
 
 - (id)initWithIdentifier:(NSString*)identifier withConnectionTo:(MYX_USER_CONNECTION*)info;
 
-- (IBAction)openScript:(id)sender;
 - (IBAction)saveScript:(id)sender;
 
 - (IBAction)copyMessage:(id)sender;
@@ -54,6 +53,7 @@ extern NSString *MQScriptEditStatusChangedNotification;
 - (MQScriptExecutionHelper*)scriptHelper;
 - (void)setDefaultSchema:(NSString*)schema;
 
+- (void)setDocumentEdited:(BOOL)flag;
 - (BOOL)isDocumentEdited;
 - (NSString*)documentName;
 - (NSString*)defaultSchema;
@@ -61,5 +61,8 @@ extern NSString *MQScriptEditStatusChangedNotification;
 - (MQScriptEditor*)scriptEditor;
 - (void)setStatusText:(NSString*)text icon:(NSImage*)icon;
 
+- (void)setFilename:(NSString*)name;
+
 - (void)setScript:(NSString*)script;
+- (NSString*)script;
 @end
