@@ -7,6 +7,8 @@
 //
 
 #import "MQueryBrowserController.h"
+#import "MQueryBrowserDocument.h"
+
 #import "MQueryBrowser.h"
 #import "MQHistory.h"
 #import "MQAddBookmarkPanel.h"
@@ -16,6 +18,7 @@
 
 #import <MySQLToolsCommon/MQResultSetView.h>
 #import <MySQLToolsCommon/MAccessoryScrollView.h>
+#import <MySQLToolsCommon/MDialogs.h>
 #import "MQScriptController.h"
 #import "MQQueryController.h"
 #import "MQResultSetTab.h"
@@ -35,7 +38,6 @@
 - (void)requestTablesForSchema:(MSchemaDataSource*)sender item:(MSchemaItem*)item;
 - (void)selectSchema:(NSString*)schema;
 - (void)updateHistory:(NSNotification*)notif;
-- (NSString*)defaultSchema;
 @end
 
 @implementation MQueryBrowser(Private)
@@ -44,9 +46,9 @@
 {
   MYX_CATALOGS *cats;
   
-  cats= [_dispatcher performCallback:(void*(*)(MYSQL*))myx_get_catalogs
-                       waitForWindow:[self window]
-                             message:@"Retrieving catalog list..."];
+  cats= [[[self document] dispatcher] performCallback:(void*(*)(MYSQL*))myx_get_catalogs
+                                        waitForWindow:[self window]
+                                              message:@"Retrieving catalog list..."];
   if (!cats)
   {
     NSLog(@"ERROR: Could not retrieve catalog list");
@@ -65,24 +67,25 @@
 - (void)requestTablesForSchema:(MSchemaDataSource*)sender item:(MSchemaItem*)item
 {
   MYX_SCHEMA *schema= [item schema];
+  MMySQLDispatcher *dispatcher= [[self document] dispatcher];
  
   [_columnDS resetSchemaChildren:item];
   
-  [_dispatcher performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_tables
-                      argument:schema->catalog_name
-                      argument:schema->schema_name
-              finishedSelector:@selector(tablesArrived:result:)
-                      argument:[[NSArray arrayWithObjects:sender,item,nil] retain]
-                        target:self];
+  [dispatcher performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_tables
+                     argument:schema->catalog_name
+                     argument:schema->schema_name
+             finishedSelector:@selector(tablesArrived:result:)
+                     argument:[[NSArray arrayWithObjects:sender,item,nil] retain]
+                       target:self];
   
-  if (mysql_version_is_later_or_equal_than(_mysql,5,0))
+  if ([[self document] connectedMySQLIsAtLeastMajor:5 minor:0])
   {
-    [_dispatcher performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_sps
-                        argument:schema->catalog_name
-                        argument:schema->schema_name
-                finishedSelector:@selector(spsArrived:result:)
-                        argument:[[NSArray arrayWithObjects:sender,item,nil] retain]
-                          target:self];
+    [dispatcher performCallback:(void*(*)(MYSQL*,void*,void*))myx_get_schema_sps
+                       argument:schema->catalog_name
+                       argument:schema->schema_name
+               finishedSelector:@selector(spsArrived:result:)
+                       argument:[[NSArray arrayWithObjects:sender,item,nil] retain]
+                         target:self];
   }
 }
 
@@ -144,11 +147,8 @@
 {
   unsigned int i;
   
-  g_free(_info->schema);
-  _info->schema= schema == nil ? NULL : g_strdup([schema UTF8String]);
-  
-  if (schema)
-	myx_use_schema(_mysql, [schema UTF8String]);  
+  if (![[[self document] defaultSchema] isEqualToString:schema])
+    [[self document] setDefaultSchema:schema];
   
   for (i= 0; i < [tabView numberOfPages]; i++)
   {
@@ -172,34 +172,26 @@
 {
   [historyList reloadData];
 }  
-
-
-- (NSString*)defaultSchema
-{
-  if (_mysql->db)
-    return [NSString stringWithUTF8String:_mysql->db];
-  return nil;
-}
 @end
 
 //------------------------------------------------------------------------------
 
 @implementation MQueryBrowser
 
+
 - (id)init
 { 
   self = [super initWithWindowNibName:@"QueryBrowser" owner:self];
   if (!self)
     return nil;
-  
-  [self window]; // force load
-  if (![self isWindowLoaded])
-    NSLog(@"ERROR: Could not load document window");
+  return self;
+}
 
+
+- (void)awakeFromNib
+{
   [tabView setMaxTabSize:100];
   
-  _mysqlLock= [[NSLock alloc] init];
-  
   _historyGroupIcon= [[NSImage imageNamed:@"history_interval_14x14.png"] retain];
   _historyItemIcon= [[NSImage imageNamed:@"query_14x14.png"] retain];
 
@@ -269,69 +261,45 @@
   
   _toolbarHeight= 58;
   
-  return self;
-}
+  _columnDS= [[MSchemaDataSource alloc] initWithRoot:MSchemaItemType 
+                                                leaf:MColumnItemType
+                                               icons:[NSDictionary dictionaryWithObjectsAndKeys:
+                                                 [NSImage imageNamed:@"16x16_Catalog.png"], @"catalog",
+                                                 [NSImage imageNamed:@"myx_schema_16x16.png"], @"schema",
+                                                 [NSImage imageNamed:@"myx_schema_table_16x16.png"], @"table",
+                                                 [NSImage imageNamed:@"myx_schema_view_16x16.png"], @"view",
+                                                 [NSImage imageNamed:@"myx_schema_sp_16x16.png"], @"sp",
+                                                 [NSImage imageNamed:@"16x16_Field.png"], @"column",
+                                                 [NSImage imageNamed:@"16x16_KeyColumn.png"], @"key",
+                                                 nil]];
+  [_columnDS setTableFetcher:self selector:@selector(requestTablesForSchema:item:)];
+  [self refreshSchemata:nil];
 
+  [schemaHelper setConnection:[[self document] mysql]];
 
-- (id)initWithConnection: (MYSQL*)mysql
-                    info: (MYX_USER_CONNECTION*)info
-{
-  self = [self init];
+  [catalogList setDataSource:_columnDS];
+  [catalogList reloadData];
+  MXExpandOutline(catalogList, NO);
+  [catalogList setDoubleAction:@selector(schemataDoubleClicked:)];
   
-  if (self)
-  {
-    _mysql=mysql;
-    _info= info;
-    
-    _dispatcher= [[MMySQLDispatcher alloc] initWithMySQL:mysql instance:info];
-	_columnDS= [[MSchemaDataSource alloc] initWithRoot:MSchemaItemType 
-												  leaf:MColumnItemType
-												 icons:[NSDictionary dictionaryWithObjectsAndKeys:
-												   [NSImage imageNamed:@"16x16_Catalog.png"], @"catalog",
-												   [NSImage imageNamed:@"myx_schema_16x16.png"], @"schema",
-												   [NSImage imageNamed:@"myx_schema_table_16x16.png"], @"table",
-                                                   [NSImage imageNamed:@"myx_schema_view_16x16.png"], @"view",
-                                                   [NSImage imageNamed:@"myx_schema_sp_16x16.png"], @"sp",
-												   [NSImage imageNamed:@"16x16_Field.png"], @"column",
-												   [NSImage imageNamed:@"16x16_KeyColumn.png"], @"key",
-												   nil]];
-    [_columnDS setTableFetcher:self selector:@selector(requestTablesForSchema:item:)];
-	[self refreshSchemata:nil];
-    [schemaHelper setConnection:mysql];
-
-	[catalogList setDataSource:_columnDS];
-	[catalogList reloadData];
-	MXExpandOutline(catalogList, NO);
-    [catalogList setDoubleAction:@selector(schemataDoubleClicked:)];
-
-	[historyList setDataSource:[[MQueryBrowserController sharedDocumentController] history]];
-	MXExpandOutline(historyList, NO);
-    [historyList reloadData];
-    [historyList setDoubleAction:@selector(executeHistory:)];
-
-    [bookmarkList setDataSource:[[MQueryBrowserController sharedDocumentController] bookmarks]];
-	MXExpandOutline(bookmarkList, NO);
-    [bookmarkList reloadData];
-
-    if (mysql->unix_socket)
-      _connectedInstance= [[NSString alloc] initWithFormat:@"%s via socket", info->hostname];
-    else
-      _connectedInstance= [[NSString alloc] initWithFormat:@"%s:%i", info->hostname, info->port];
-
-    [self selectSchema: info->schema?[NSString stringWithUTF8String:info->schema]:nil];
-	[self addQueryTab: self];
-
-    [drawer open];
-  }
-  return self;
+  [historyList setDataSource:[[MQueryBrowserController sharedDocumentController] history]];
+  MXExpandOutline(historyList, NO);
+  [historyList reloadData];
+  [historyList setDoubleAction:@selector(executeHistory:)];
+  
+  [bookmarkList setDataSource:[[MQueryBrowserController sharedDocumentController] bookmarks]];
+  MXExpandOutline(bookmarkList, NO);
+  [bookmarkList reloadData];
+  
+  [self selectSchema: [[self document] defaultSchema]];
+  [self addQueryTab: self];
+  
+  [drawer open];
 }
 
 
 - (void)dealloc
 {
-  [_dispatcher release];
-  [_connectedInstance release];
-  [_mysqlLock release];
   [_columnDS release];
   [_parameters release];
   [_groupIcon release];
@@ -340,11 +308,6 @@
   if (_catalogs)
     myx_free_catalogs(_catalogs);
   
-  if (_mysql)
-    myx_mysql_close(_mysql);
-  if (_info)
-    myx_free_user_connection_content(_info);
-  
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   
   [super dealloc];
@@ -374,6 +337,9 @@
       if ([[tabView selectedPage] isKindOfClass:[MQResultSetTab class]])
         return [queryCtl setupQueryWithItem:item];
       break;
+    case MCatalogItemType:
+    case MColumnItemType:
+      break;
   }
   return NO;
 }
@@ -452,9 +418,11 @@
   MQResultSetTab *page= [queryCtl createPage];
   if (page)
   {
-    [page setDefaultSchema: [self defaultSchema]];
+    [page setDefaultSchema: [[self document] defaultSchema]];
     [tabView addTabViewItem:page];
     [tabView selectPage:[tabView numberOfPages]-1];
+    
+    [queryCtl addNewResultSet:NO toPage:page];
   }
 }
 
@@ -464,13 +432,37 @@
   MQScriptTab *page= [scriptCtl createPage];
   if (page)
   {
-    [page setDefaultSchema: [self defaultSchema]];
+    [page setDefaultSchema: [[self document] defaultSchema]];
     [tabView addTabViewItem:page];
     [tabView selectPage:[tabView numberOfPages]-1];
   }
 }
 
 
+- (void)exportResultset:(id)sender
+{
+  [queryCtl exportResultSet:sender];
+}
+
+
+- (void)openDocument:(id)sender
+{
+  [scriptCtl openScript:sender];
+}
+
+
+- (void)executeQuery:(id)sender
+{
+  [queryCtl executeQuery:sender];
+}
+
+
+- (void)stopQuery:(id)sender
+{
+  [queryCtl stopQuery:sender];
+}
+
+
 - (void)explainQuery:(id)sender
 {
   [queryCtl explainQuery:sender];
@@ -483,9 +475,48 @@
 }
 
 
-- (void)performCloseTab:(id)sender
+
+- (void)showTableRows:(id)sender
 {
-  [self tabView:tabView closeTab:[self currentTab]];
+  int row= [catalogList selectedRow];
+  
+  if (row < 0) return;
+  
+  if ([sender tag] == 10) 
+  {
+    [queryCtl setupSelectQueryWithItem:[catalogList itemAtRow:row] 
+                              rowLimit:[[NSUserDefaults standardUserDefaults] integerForKey:@"PartialTableListLimit"]];
+  }
+  else
+  {
+    [queryCtl setupSelectQueryWithItem:[catalogList itemAtRow:row] 
+                              rowLimit:0];
+  }
+  [self executeQuery:nil];
+}
+
+
+- (void)doSchemaAction:(id)sender
+{
+  switch ([sender tag])
+  {
+    case 20: // create view with query
+      if ([[self currentTab] isKindOfClass:[MQResultSetTab class]])
+      {
+        MStringRequestSheet *sheet= [MStringRequestSheet sheetWithTitle:@"Create view from the current query."
+                                                                 labels:[NSArray arrayWithObject:@"View Name:"]];
+        NSArray *results= [sheet runModal:[self window]];
+        
+        if (results)
+        {
+          [scriptCtl setupEditorWithScript:[NSString stringWithFormat:@"CREATE VIEW `%@` AS\n    %@;\n", [results objectAtIndex:0], [queryCtl currentQuery]]];
+        }
+      }
+      break;
+    case 21: // edit all SPs
+      [schemaHelper editStoredProceduresInCatalog:nil schema:[[self document] defaultSchema]];
+      break;
+  }
 }
 
 
@@ -648,7 +679,7 @@
   [panel showSheetForWindow:[self window]
                   bookmarks:[[MQueryBrowserController sharedDocumentController] bookmarks]
                     catalog:@"def"
-                     schema:_info->schema ? [NSString stringWithUTF8String:_info->schema] : nil
+                     schema:[[self document] defaultSchema]
                       query:[NSString stringWithString:query]];
   [bookmarkList reloadData];
 }
@@ -693,12 +724,6 @@
   return NO;
 }
 
-
-- (void)killConnection:(unsigned long)pid
-{
-  mysql_kill(_mysql, pid);
-}
-
 - (void)schemaHelperChangedSchemata:(MSchemaEditHelper*)helper
 {
   [self refreshSchemata:nil];
@@ -707,29 +732,17 @@
 
 - (void)schemaHelper:(MSchemaEditHelper*)helper editScript:(NSString*)data
 {
-  MQScriptTab *page= [scriptCtl createPage];
-  if (page)
-  {
-    [page setDefaultSchema: [self defaultSchema]];
-    [page setScript:data];
-    [tabView addTabViewItem:page];
-    [tabView selectPage:[tabView numberOfPages]-1];
-  }  
+  [scriptCtl setupEditorWithScript:data];
 }
 
 
-- (MMySQLDispatcher*)dispatcher
+- (NSString *)windowTitleForDocumentDisplayName:(NSString *)displayName
 {
-  return _dispatcher;
-}
-
-- (void)setTitleForDocumentName: (NSString*)name
-{
-  if (name)
-    [[self window] setTitle: [NSString stringWithFormat:@"%@ @ %@", 
-      name, _connectedInstance]];
+  if (displayName)
+    return [NSString stringWithFormat:@"%@ @ %@", 
+      displayName, [[self document] connectedInstance]];
   else
-    [[self window] setTitle: _connectedInstance];
+    return [[self document] connectedInstance];
 }
 
 
@@ -737,29 +750,7 @@
 {
   [_parameters saveGlobals];
   
-  [[MQueryBrowserController sharedDocumentController] removeInstance:self];
-}
-
-
-- (void)unlockMySQL
-{
-  [_mysqlLock unlock];
-}
-
-- (MYSQL*)mysql
-{
-  return _mysql;
-}
-
-- (MYSQL*)mysqlLock
-{
-  [_mysqlLock lock];
-  return _mysql;
-}
-
-- (MYX_USER_CONNECTION*)serverInfo
-{
-  return _info;
+//  [[MQueryBrowserController sharedDocumentController] removeInstance:self];
 }
 
 
@@ -818,13 +809,13 @@ shouldCancel:(BOOL)shouldCancel
           flag= NO;
         break;
       case 4: // Create SP
-        if (level >= 1 && mysql_version_is_later_or_equal_than(_mysql,5,0))
+        if (level >= 1 && [[self document] connectedMySQLIsAtLeastMajor:5 minor:0])
           flag= YES;
         else
           flag= NO;
         break;
       case 5: // Create View
-        if (level >= 1 && mysql_version_is_later_or_equal_than(_mysql,5,0))
+        if (level >= 1 && [[self document] connectedMySQLIsAtLeastMajor:5 minor:0])
           flag= YES;
         else
           flag= NO;
@@ -851,12 +842,12 @@ shouldCancel:(BOOL)shouldCancel
             else if ([sitem type] == MViewItemType)
             {
               [item setTitle:@"Edit View"];
-              flag= mysql_version_is_later_or_equal_than(_mysql,5,0)?YES:0;
+              flag= [[self document] connectedMySQLIsAtLeastMajor:5 minor:0]?YES:0;
             }
             else
             {
               [item setTitle:@"Edit Stored Procedure"];
-              flag= mysql_version_is_later_or_equal_than(_mysql,5,0)?YES:0;
+              flag= [[self document] connectedMySQLIsAtLeastMajor:5 minor:0]?YES:0;
             }
             break;
           default:
@@ -888,12 +879,12 @@ shouldCancel:(BOOL)shouldCancel
             else if ([sitem type] == MViewItemType)
             {
               [item setTitle:@"Drop View"];
-              flag= mysql_version_is_later_or_equal_than(_mysql,5,0)?YES:0;
+              flag= [[self document] connectedMySQLIsAtLeastMajor:5 minor:0]?YES:0;
             }
             else
             {
               [item setTitle:@"Drop Stored Procedure"];
-              flag= mysql_version_is_later_or_equal_than(_mysql,5,0)?YES:0;
+              flag= [[self document] connectedMySQLIsAtLeastMajor:5 minor:0]?YES:0;
             }
             break;
           }
@@ -966,6 +957,54 @@ shouldCancel:(BOOL)shouldCancel
     id hitem= sel < 0 ? nil : [historyList itemAtRow:sel];
     [item setEnabled:[hitem isKindOfClass:[MQHistoryItem class]]];
   }
+  else if ([[[[[NSApp mainMenu] itemWithTag:100] submenu] itemWithTag:100] submenu] == menu) // Export Resultset submenu
+  {
+    if ([[self currentTab] isKindOfClass:[MQResultSetTab class]]
+        && [[[self currentTab] activeResultSet] resultset])
+      [item setEnabled:YES];
+    else
+      [item setEnabled:NO];
+  }  
+  else
+  {    
+    switch ([[[NSApp mainMenu] itemAtIndex:[[NSApp mainMenu] indexOfItemWithSubmenu:menu]] tag])
+    {
+      case 104: // Schema menu
+      {
+        int row= [catalogList selectedRow];
+        MSchemaItem *sitem;
+        BOOL flag= NO;
+        
+        switch (index)
+        {
+          case 0: // Show Table Rows
+          case 1:
+            if (row >= 0)
+            {
+              sitem= [catalogList itemAtRow:row];
+              if ([sitem type] == MTableItemType || [sitem type] == MViewItemType)
+                flag= YES;
+            }
+            break;
+          case 3: // Create view with query
+            if ([[self currentTab] isKindOfClass:[MQResultSetTab class]])
+              flag= YES;
+            break;
+          case 4: // Edit all SPs
+            if ([[self document] defaultSchema] || (row >= 0 && [(MSchemaItem*)[catalogList itemAtRow:row] type] != MCatalogItemType))
+              flag= YES;
+            break;
+        }
+        [item setEnabled:flag];
+        return YES;
+      }
+      case 105: // Query menu
+        return [queryCtl stateOfMenuItem:item atIndex:index inMenu:menu];
+
+      case 106: // Script menu
+        return [scriptCtl stateOfMenuItem:item atIndex:index inMenu:menu];
+    }
+  }
   return YES;
 }
 
@@ -1065,8 +1104,64 @@ shouldCancel:(BOOL)shouldCancel
 }
 
 
+- (void)modalAlertDidEnd:(NSAlert*)alert returnCode:(int)returnCode contextInfo:(void*)info
+{
+  [NSApp stopModalWithCode:returnCode];
+}
+
 - (void)tabView:(MTabView*)aTabView closeTab:(MTabViewItem*)tab
 {
+  if ([tab isKindOfClass:[MQScriptTab class]])
+  {
+    if ([(MQScriptTab*)tab isDocumentEdited])
+    {
+      NSAlert *alert= [NSAlert alertWithMessageText:@"Save script before closing?"
+                                      defaultButton:@"Save"
+                                    alternateButton:@"Don't Save"
+                                        otherButton:@"Cancel"
+                          informativeTextWithFormat:@"This script has changes that were not yet saved."];
+      [alert beginSheetModalForWindow:[self window]
+                        modalDelegate:self
+                       didEndSelector:@selector(modalAlertDidEnd:returnCode:contextInfo:) 
+                          contextInfo:tab];
+      switch ([NSApp runModalForWindow:[self window]])
+      {
+        case NSAlertDefaultReturn:
+          [(MQScriptTab*)tab saveScript:nil];
+          break;
+        case NSAlertAlternateReturn:
+          break;
+        case NSAlertOtherReturn:
+          return;
+      }
+    }
+  }
+  else if ([tab isKindOfClass:[MQResultSetTab class]])
+  {
+    if ([(MQResultSetTab*)tab hasUnsavedChanges])
+    {
+      NSAlert *alert= [NSAlert alertWithMessageText:@"Discard resultset changes?"
+                                      defaultButton:@"Cancel"
+                                    alternateButton:@"Discard and Close"
+                                        otherButton:nil
+                          informativeTextWithFormat:@"There are resultset changes that were not yet saved and will be discarded if the tabsheet is closed."];
+      [alert beginSheetModalForWindow:[self window]
+                        modalDelegate:self
+                       didEndSelector:@selector(modalAlertDidEnd:returnCode:contextInfo:) 
+                          contextInfo:tab];
+      switch ([NSApp runModalForWindow:[self window]])
+      {
+        case NSAlertDefaultReturn:
+          return;
+        case NSAlertAlternateReturn:
+          break;
+        case NSAlertOtherReturn:
+          break;
+      }
+    }
+  }
+  
+  
   if ([aTabView numberOfPages] == 1)
   {
     [[NSAlert alertWithMessageText:@"Close Tabsheet"
@@ -1084,6 +1179,12 @@ shouldCancel:(BOOL)shouldCancel
 }
 
 
+- (void)performCloseTab:(id)sender
+{
+  [self tabView:tabView closeTab:[self currentTab]];
+}
+
+
 - (void)outlineView:(NSOutlineView *)outlineView 
     willDisplayCell:(id)cell 
      forTableColumn:(NSTableColumn *)tableColumn 
@@ -1091,7 +1192,7 @@ shouldCancel:(BOOL)shouldCancel
 {
   if (outlineView == catalogList)
   {
-	NSString *schema= _info->schema ? [NSString stringWithUTF8String:_info->schema] : nil;
+	NSString *schema= [[self document] defaultSchema];
 	
 	if ([item respondsToSelector:@selector(icon)])
 	  [cell setImage:[item icon]];
@@ -1124,14 +1225,39 @@ shouldCancel:(BOOL)shouldCancel
   }
   else if (outlineView == parameterList)
   {
-    if ([item isKindOfClass:[NSString class]])
-      [cell setImage:_groupIcon];
-    else
-      [cell setImage:_paramIcon];
+    if ([[tableColumn identifier] isEqualToString:@"name"])
+    {
+      if ([item isKindOfClass:[NSString class]])
+        [cell setImage:_groupIcon];
+      else
+        [cell setImage:_paramIcon];
+    }
   }
 }
 
 
+- (BOOL)outlineView:(NSOutlineView *)outlineView
+ shouldEditTableColumn:(NSTableColumn *)tableColumn 
+               item:(id)item
+{
+  if (outlineView == parameterList)
+  {
+    int row= [parameterList selectedRow];
+    
+    if (row >= 0)
+    {
+      id item= [parameterList itemAtRow:row];
+      if ([_parameters itemCanBeEdited:item])
+      {
+        return YES;
+      }
+    }
+    return NO;
+  }
+  return YES;
+}
+
+
 - (void)outlineViewSelectionDidChange:(NSNotification *)notification
 {
   if ([notification object] == parameterList)
