@@ -6,37 +6,34 @@
 @class MQueryBrowser;
 @class MQHistory;
 @class MQBookmarkList;
+@class MConnectionInfo;
+
+@class MGlobalSwitcher;
 
 // this does the role of a NSDocumentController
-@interface MQueryBrowserController : NSObject
+@interface MQueryBrowserController : NSDocumentController
 {
   IBOutlet id aboutPanel;
   IBOutlet id versionLabel;
-  IBOutlet id recentMenu;
+  IBOutlet id windowsMenu;
   
   MQHistory *_history;
   BOOL _historyNeedsSave;
   MQBookmarkList *_bookmarks;
 
+  MGlobalSwitcher *_switcher;
+  
   id _prefPanel;
-  NSMutableArray *_instances;
 }
 
 - (IBAction)showAbout:(id)sender;
 - (IBAction)showPreferences:(id)sender;
 - (IBAction)clearRecentMenu:(id)sender;
-
-+ (MQueryBrowserController*)sharedDocumentController;
-
-- (void)newDocument: (id)sender;
-
-- (void)addInstance: (MQueryBrowser*)instance;
-
-- (void)removeInstance: (MQueryBrowser*)instance;
+- (IBAction)newConnection:(id)sender;
 
 - (void)openInstanceWithConnectionName: (NSString*)connectionName;
 
-- (void)noteNewRecentConnection: (MYX_USER_CONNECTION*)info;
+- (void)noteNewRecentConnection: (MConnectionInfo*)info;
 
 
 
